/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  IBM Corporation - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.accservice.javapp;

import javax.accessibility.Accessible;
import javax.accessibility.AccessibleContext;
import javax.accessibility.AccessibleTable;

import org.eclipse.actf.accservice.core.IAccessibleTableElement;
import org.eclipse.actf.model.InvalidComponentException;

/**
 * implementation of the <code>IAccessibleImageElement</code> object for the Java Accesibility API's <code>AccessibleTable</code>.
 * 
 * @see javax.accessibility.AccessibleTable
 * @author Mike Smith
 */
public class JavaAccessibleTable implements IAccessibleTableElement
{

	private AccessibleTable _table;

	/**
	 * Constructor used to create an accessible table object. This constructor will usually 
	 * be invoked from <code>JavaAccessible</code> and is not typically called by clients.
	 * 
	 * @param table -
	 *            AccessibleTable object
	 */
	public JavaAccessibleTable (AccessibleTable table) {
		_table = table;
	}

	private void checkIsValid () throws InvalidComponentException {
		if (_table == null) { throw new InvalidComponentException("No accessible context"); }
	}

	/** {@inheritDoc} */
	public Object getCaption () throws InvalidComponentException {
		checkIsValid();
		return _table.getAccessibleCaption();
	}

	/** {@inheritDoc} */
	public Object getSummary () throws InvalidComponentException {
		checkIsValid();
		return _table.getAccessibleSummary();
	}

	/** {@inheritDoc} */
	public long getColumnCount () throws InvalidComponentException {
		checkIsValid();
		return _table.getAccessibleColumnCount();
	}

	/** {@inheritDoc} */
	public long getRowCount () throws InvalidComponentException {
		checkIsValid();
		return _table.getAccessibleRowCount();
	}

	/** {@inheritDoc} */
	public String getColumnDescription (long column)
		throws InvalidComponentException {
		checkIsValid();
		String res = "";
		if (column >= 0 && column < getColumnCount()) {
			// Note that getAccessibleRowCount can never return anything larger
			// than an int so we are safe casting this to an int
			res = getName(_table.getAccessibleColumnDescription((int) column));
		}
		return res;
	}

	/** {@inheritDoc} */
	public String getRowDescription (long row)
		throws InvalidComponentException {
		checkIsValid();
		String res = "";
		if (row >= 0 && row < getRowCount()) {
			// Note that getAccessibleRowCount can never return anything larger
			// than an int so we are safe casting this to an int
			res = getName(_table.getAccessibleRowDescription((int) row));
		}
		return res;
	}

	/** {@inheritDoc} */
	public int[] getSelectedColumns ()
		throws InvalidComponentException {
		checkIsValid();
		return _table.getSelectedAccessibleColumns();
	}

	/** {@inheritDoc} */
	public int[] getSelectedRows () throws InvalidComponentException {
		checkIsValid();
		return _table.getSelectedAccessibleRows();
	}

	/**
	 * get the accessible name of the given accessible object
	 * 
	 * @return name of the accessible object or empty sting if not available
	 */
	private String getName (Accessible acc) throws InvalidComponentException {
		String res = "";
		if (acc != null) {
			AccessibleContext context = acc.getAccessibleContext();
			if (context != null) {
				res = context.getAccessibleName();
			}
		}
		return res;
	}

	public int getSelectedChildCount() throws InvalidComponentException {
		// TODO Auto-generated method stub
		return 0;
	}

	public int[] getSelectedChildren() throws InvalidComponentException {
		// TODO Auto-generated method stub
		return null;
	}
}
