/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  IBM Corporation - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.accservice.javapp;

import javax.accessibility.AccessibleText;
import javax.swing.text.AttributeSet;

import org.eclipse.actf.accservice.core.IAccessibleTextElement;
import org.eclipse.actf.model.InvalidComponentException;

/**
 * implementation of the <code>IAccessibleTextElement</code> object for the Java Accesibility API's <code>AccessibleText</code>.
 * 
 * @see javax.accessibility.AccessibleText
 * 
 * @author Mike Smith
 */
public class JavaAccessibleText implements IAccessibleTextElement
{

	private AccessibleText _text;

	/**
	 * Constructor used to create an accessible text object. This constructor will usually 
	 * be invoked from <code>JavaAccessible</code> and is not typically called by clients.
	 * @param text -
	 *            AccessibleText object
	 */
	public JavaAccessibleText (AccessibleText text) {
		_text = text;
	}

	private void checkIsValid () throws InvalidComponentException {
		if (_text == null) { throw new InvalidComponentException("No accessible context"); }
	}

	/** {@inheritDoc} */
	public String getText () throws InvalidComponentException {
		checkIsValid();
		return _text.toString();
	}

	/** {@inheritDoc} */
	public long getCaretOffset () throws InvalidComponentException {
		checkIsValid();
		return _text.getCaretPosition();
	}

	/** {@inheritDoc} */
	public Object getAttributes (long offset) throws InvalidComponentException {
		checkIsValid();
		String res = null;
		if (_text != null && offset <= Integer.MAX_VALUE) {
			AttributeSet attributes = _text.getCharacterAttribute((int) offset);
			res = attributes == null ? "" : attributes.toString();
		}
		return res;
	}

	/** {@inheritDoc} */
	public long getSelectionCount () throws InvalidComponentException {
		checkIsValid();
		long res = 0;
		if (_text != null && _text.getSelectedText() != null) {
			res = 1;
		}
		return res;
	}

	/** {@inheritDoc} */
	public String getSelectedText (long index) throws InvalidComponentException {
		checkIsValid();
		String res = null;
		// Java supports only 1 selection so only return
		// the information if the index is 0
		if (index == 0) {
			res = _text.getSelectedText();
		}
		return res;
	}

	/** {@inheritDoc} */
	public long getSelectedTextStart (long index)
		throws InvalidComponentException {
		checkIsValid();
		long res = -1;
		// Java supports only 1 selection so only return
		// the information if the index is 0
		if (index == 0) {
			res = _text.getSelectionStart();
		}
		return res;
	}

	/** {@inheritDoc} */
	public long getSelectedTextEnd (long index)
		throws InvalidComponentException {
		checkIsValid();
		long res = -1;
		// Java supports only 1 selection so only return
		// the information if the index is 0
		if (index == 0) {
			res = _text.getSelectionEnd();
		}
		return res;
	}
}