/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  IBM Corporation - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.accservice.mozdom;

import java.awt.Rectangle;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.eclipse.actf.accservice.core.AccessibleConstants;
import org.eclipse.actf.accservice.core.IAccessibleElement;
import org.eclipse.actf.accservice.core.win32.ia2.IA2;
import org.eclipse.actf.accservice.core.win32.msaa.Msaa;
import org.eclipse.actf.model.InvalidComponentException;
import org.eclipse.actf.util.logging.LoggingUtil;
import org.mozilla.interfaces.nsIAccessible;
import org.mozilla.interfaces.nsIAccessibleRetrieval;
import org.mozilla.interfaces.nsIAccessibleSelectable;
import org.mozilla.interfaces.nsIArray;
import org.mozilla.interfaces.nsIDOMNode;
import org.mozilla.interfaces.nsISupports;
import org.mozilla.xpcom.Mozilla;
import org.mozilla.xpcom.XPCOMException;


public class MozillaAccessible implements IAccessibleElement
{

	private static final String NS_IACCESSIBLERETRIEVAL_1_9_IID = "{244e4c67-a1d3-44f2-9cab-cdaa31b68046}";
	private static final Map<Long, String> RELATIONKEY_MAP = new HashMap<Long, String>();
	static {
		RELATIONKEY_MAP.put(
			nsIAccessible.RELATION_CONTROLLER_FOR, AccessibleConstants.RELATION_CONTROLLER_FOR);
		RELATIONKEY_MAP.put(
			nsIAccessible.RELATION_CONTROLLED_BY, AccessibleConstants.RELATION_CONTROLLED_BY);
		RELATIONKEY_MAP.put(
			nsIAccessible.RELATION_LABEL_FOR, AccessibleConstants.RELATION_LABEL_FOR);
		RELATIONKEY_MAP.put(
			nsIAccessible.RELATION_LABELLED_BY, AccessibleConstants.RELATION_LABELED_BY);
		RELATIONKEY_MAP.put(
			nsIAccessible.RELATION_NODE_CHILD_OF, AccessibleConstants.RELATION_NODE_CHILD_OF);
		RELATIONKEY_MAP.put(
			nsIAccessible.RELATION_MEMBER_OF, AccessibleConstants.RELATION_MEMBER_OF);
		RELATIONKEY_MAP.put(
			nsIAccessible.RELATION_FLOWS_TO, AccessibleConstants.RELATION_FLOWS_TO);
		RELATIONKEY_MAP.put(
			nsIAccessible.RELATION_FLOWS_FROM, AccessibleConstants.RELATION_FLOWS_FROM);
		RELATIONKEY_MAP.put(
			nsIAccessible.RELATION_EMBEDS, AccessibleConstants.RELATION_EMBEDS);
		RELATIONKEY_MAP.put(
			nsIAccessible.RELATION_EMBEDDED_BY, AccessibleConstants.RELATION_EMBEDDED_BY);
		RELATIONKEY_MAP.put(
			nsIAccessible.RELATION_PARENT_WINDOW_OF, AccessibleConstants.RELATION_PARENT_WINDOW_OF);
		RELATIONKEY_MAP.put(
			nsIAccessible.RELATION_SUBWINDOW_OF, AccessibleConstants.RELATION_SUBWINDOW_OF);
		RELATIONKEY_MAP.put(
			nsIAccessible.RELATION_DESCRIPTION_FOR, AccessibleConstants.RELATION_DESCRIPTION_FOR);
		RELATIONKEY_MAP.put(
			nsIAccessible.RELATION_DESCRIBED_BY, AccessibleConstants.RELATION_DESCRIBED_BY);
	}

	//nsiAccessibleRole constants
	// these values were obtained from http://doxygen.db48x.net/mozilla/html/interfacensIAccessibleRole.html
	
	private static final long ROLE_NOTHING = 0;  
	private static final long ROLE_TITLEBAR = 1;
	private static final long ROLE_MENUBAR = 2;
	private static final long ROLE_SCROLLBAR = 3;
	private static final long ROLE_GRIP = 4;
	private static final long ROLE_SOUND = 5;
	private static final long ROLE_CURSOR = 6;
	private static final long ROLE_CARET = 7;
	private static final long ROLE_ALERT = 8;
	private static final long ROLE_WINDOW = 9;
	private static final long ROLE_INTERNAL_FRAME = 10;
	private static final long ROLE_MENUPOPUP = 11;
	private static final long ROLE_MENUITEM = 12;
	private static final long ROLE_TOOLTIP = 13;
	private static final long ROLE_APPLICATION = 14;
	private static final long ROLE_DOCUMENT = 15;
	private static final long ROLE_PANE = 16;
	private static final long ROLE_CHART = 17;
	private static final long ROLE_DIALOG = 18;
	private static final long ROLE_BORDER = 19;
	private static final long ROLE_GROUPING = 20;
	private static final long ROLE_SEPARATOR = 21;
	private static final long ROLE_TOOLBAR = 22;
	private static final long ROLE_STATUSBAR = 23;
	private static final long ROLE_TABLE = 24;
	private static final long ROLE_COLUMNHEADER = 25;
	private static final long ROLE_ROWHEADER = 26;
	private static final long ROLE_COLUMN = 27;
	private static final long ROLE_ROW = 28;
	private static final long ROLE_CELL = 29;
	private static final long ROLE_LINK = 30;
	private static final long ROLE_HELPBALLOON = 31;
	private static final long ROLE_CHARACTER = 32;
	private static final long ROLE_LIST = 33;
	private static final long ROLE_LISTITEM = 34;
	private static final long ROLE_OUTLINE = 35;
	private static final long ROLE_OUTLINEITEM = 36;
	private static final long ROLE_PAGETAB = 37;
	private static final long ROLE_PROPERTYPAGE = 38;
	private static final long ROLE_INDICATOR = 39;
	private static final long ROLE_GRAPHIC = 40;
	private static final long ROLE_STATICTEXT = 41;
	private static final long ROLE_TEXT_LEAF = 42;
	private static final long ROLE_PUSHBUTTON = 43;
	private static final long ROLE_CHECKBUTTON = 44;
	private static final long ROLE_RADIOBUTTON = 45;
	private static final long ROLE_COMBOBOX = 46;
	private static final long ROLE_DROPLIST = 47;
	private static final long ROLE_PROGRESSBAR = 48;
	private static final long ROLE_DIAL = 49;
	private static final long ROLE_HOTKEYFIELD = 50;
	private static final long ROLE_SLIDER = 51;
	private static final long ROLE_SPINBUTTON = 52;
	private static final long ROLE_DIAGRAM = 53;
	private static final long ROLE_ANIMATION = 54;
	private static final long ROLE_EQUATION = 55;
	private static final long ROLE_BUTTONDROPDOWN = 56;
	private static final long ROLE_BUTTONMENU = 57;
	private static final long ROLE_BUTTONDROPDOWNGRID = 58;
	private static final long ROLE_WHITESPACE = 59;
	private static final long ROLE_PAGETABLIST = 60;
	private static final long ROLE_CLOCK = 61;
	private static final long ROLE_SPLITBUTTON = 62;
	private static final long ROLE_IPADDRESS = 63;
	private static final long ROLE_ACCEL_LABEL = 64;
	private static final long ROLE_ARROW = 65;
	private static final long ROLE_CANVAS = 66;
	private static final long ROLE_CHECK_MENU_ITEM = 67;
	private static final long ROLE_COLOR_CHOOSER = 68;
	private static final long ROLE_DATE_EDITOR = 69;
	private static final long ROLE_DESKTOP_ICON = 70;
	private static final long ROLE_DESKTOP_FRAME = 71;
	private static final long ROLE_DIRECTORY_PANE = 72;
	private static final long ROLE_FILE_CHOOSER = 73;
	private static final long ROLE_FONT_CHOOSER = 74;
	private static final long ROLE_CHROME_WINDOW = 75;
	private static final long ROLE_GLASS_PANE = 76;
	private static final long ROLE_HTML_CONTAINER = 77;
	private static final long ROLE_ICON = 78;
	private static final long ROLE_LABEL = 79;
	private static final long ROLE_LAYERED_PANE = 80;
	private static final long ROLE_OPTION_PANE = 81;
	private static final long ROLE_PASSWORD_TEXT = 82;
	private static final long ROLE_POPUP_MENU = 83;
	private static final long ROLE_RADIO_MENU_ITEM = 84;
	private static final long ROLE_ROOT_PANE = 85;
	private static final long ROLE_SCROLL_PANE = 86;
	private static final long ROLE_SPLIT_PANE = 87;
	private static final long ROLE_TABLE_COLUMN_HEADER = 88;
	private static final long ROLE_TABLE_ROW_HEADER = 89;
	private static final long ROLE_TEAR_OFF_MENU_ITEM = 90;
	private static final long ROLE_TERMINAL = 91;
	private static final long ROLE_TEXT_CONTAINER = 92;
	private static final long ROLE_TOGGLE_BUTTON = 93;
	private static final long ROLE_TREE_TABLE = 94;
	private static final long ROLE_VIEWPORT = 95;
	private static final long ROLE_HEADER = 96;
	private static final long ROLE_FOOTER = 97;
	private static final long ROLE_PARAGRAPH = 98;
	private static final long ROLE_RULER = 99;
	private static final long ROLE_AUTOCOMPLETE = 100;
	private static final long ROLE_EDITBAR = 101;
	private static final long ROLE_ENTRY = 102;
	private static final long ROLE_CAPTION = 103;
	private static final long ROLE_DOCUMENT_FRAME = 104;
	private static final long ROLE_HEADING = 105;
	private static final long ROLE_PAGE = 106;
	private static final long ROLE_SECTION = 107;
	private static final long ROLE_REDUNDANT_OBJECT = 108;
	private static final long ROLE_FORM = 109;
	private static final long ROLE_IME = 110;
	private static final long ROLE_APP_ROOT = 111;
	private static final long ROLE_PARENT_MENUITEM = 112;
	private static final long ROLE_CALENDAR = 113;
	private static final long ROLE_COMBOBOX_LIST = 114;
	private static final long ROLE_COMBOBOX_OPTION = 115;
	private static final long ROLE_IMAGE_MAP = 116;
	private static final long ROLE_OPTION = 117;
	private static final long ROLE_RICH_OPTION = 118;
	private static final long ROLE_LISTBOX = 119;
	private static final long ROLE_FLAT_EQUATION = 120;
	private static final long ROLE_LAST_ENTRY = 121;
	                          
	private static final Map<Long, String> ROLEKEY_MAP = new HashMap<Long, String>();
	static {
		ROLEKEY_MAP.put(ROLE_NOTHING, "nothing");  
		ROLEKEY_MAP.put(ROLE_TITLEBAR,  "titlebar");
		ROLEKEY_MAP.put(ROLE_MENUBAR, "menubar");
		ROLEKEY_MAP.put(ROLE_SCROLLBAR, "scrollbar");
		ROLEKEY_MAP.put(ROLE_GRIP, "grip");
		ROLEKEY_MAP.put(ROLE_SOUND,"sound");
		ROLEKEY_MAP.put(ROLE_CURSOR,"cursor");
		ROLEKEY_MAP.put(ROLE_CARET, "caret");
		ROLEKEY_MAP.put(ROLE_ALERT, "alert");
		ROLEKEY_MAP.put(ROLE_WINDOW, "window");
		ROLEKEY_MAP.put(ROLE_INTERNAL_FRAME, "internalframe"); //iframe
		ROLEKEY_MAP.put(ROLE_MENUPOPUP, "menupopup"); //menu
		ROLEKEY_MAP.put(ROLE_MENUITEM, "menuitem");
		ROLEKEY_MAP.put(ROLE_TOOLTIP, "tooltip");
		ROLEKEY_MAP.put(ROLE_APPLICATION, "application");
		ROLEKEY_MAP.put(ROLE_DOCUMENT, "document");
		ROLEKEY_MAP.put(ROLE_PANE,"pane"); 
		ROLEKEY_MAP.put(ROLE_CHART, "chart");
		ROLEKEY_MAP.put(ROLE_DIALOG, "dialog");
		ROLEKEY_MAP.put(ROLE_BORDER, "border");
		ROLEKEY_MAP.put(ROLE_GROUPING, "grouping"); //group
		ROLEKEY_MAP.put(ROLE_SEPARATOR, "separator");
		ROLEKEY_MAP.put(ROLE_TOOLBAR, "toolbar");
		ROLEKEY_MAP.put(ROLE_STATUSBAR, "statusbar");
		ROLEKEY_MAP.put(ROLE_TABLE, "table"); //grid
		ROLEKEY_MAP.put(ROLE_COLUMNHEADER, "columnheader"); //colheader
		ROLEKEY_MAP.put(ROLE_ROWHEADER, "rowheader");
		ROLEKEY_MAP.put(ROLE_COLUMN, "column");
		ROLEKEY_MAP.put(ROLE_ROW, "row");
		ROLEKEY_MAP.put(ROLE_CELL, "cell"); //gridcell
		ROLEKEY_MAP.put(ROLE_LINK, "link");
		ROLEKEY_MAP.put(ROLE_HELPBALLOON, "helpballon");
		ROLEKEY_MAP.put(ROLE_CHARACTER, "character");
		ROLEKEY_MAP.put(ROLE_LIST, "list");
		ROLEKEY_MAP.put(ROLE_LISTITEM, "listitem");
		ROLEKEY_MAP.put(ROLE_OUTLINE, "outline"); //tree
		ROLEKEY_MAP.put(ROLE_OUTLINEITEM, "outlineitem"); //treeitem
		ROLEKEY_MAP.put(ROLE_PAGETAB, "pagetab");
		ROLEKEY_MAP.put(ROLE_PROPERTYPAGE, "propertypage"); //tabpanel
		ROLEKEY_MAP.put(ROLE_INDICATOR, "indicator");
		ROLEKEY_MAP.put(ROLE_GRAPHIC, "graphic"); //image
		ROLEKEY_MAP.put(ROLE_STATICTEXT, "statictext");
		ROLEKEY_MAP.put(ROLE_TEXT_LEAF, "textleaf");
		ROLEKEY_MAP.put(ROLE_PUSHBUTTON, "pushbutton"); //button
		ROLEKEY_MAP.put(ROLE_CHECKBUTTON, "checkbutton"); //checkbox
		ROLEKEY_MAP.put(ROLE_RADIOBUTTON,"radiobutton"); //radio
		ROLEKEY_MAP.put(ROLE_COMBOBOX, "combobox");
		ROLEKEY_MAP.put(ROLE_DROPLIST, "droplist");
		ROLEKEY_MAP.put(ROLE_PROGRESSBAR, "progressbar");
		ROLEKEY_MAP.put(ROLE_DIAL, "dial");
		ROLEKEY_MAP.put(ROLE_HOTKEYFIELD, "hotkeyfield");
		ROLEKEY_MAP.put(ROLE_SLIDER, "slider");
		ROLEKEY_MAP.put(ROLE_SPINBUTTON, "spinbutton");
		ROLEKEY_MAP.put(ROLE_DIAGRAM, "diagram");
		ROLEKEY_MAP.put(ROLE_ANIMATION, "animation");
		ROLEKEY_MAP.put(ROLE_EQUATION,"equation");
		ROLEKEY_MAP.put(ROLE_BUTTONDROPDOWN, "buttondropdown");
		ROLEKEY_MAP.put(ROLE_BUTTONMENU, "buttonmenu");
		ROLEKEY_MAP.put(ROLE_BUTTONDROPDOWNGRID, "buttondropdowngrid");
		ROLEKEY_MAP.put(ROLE_WHITESPACE, "whitespace");
		ROLEKEY_MAP.put(ROLE_PAGETABLIST, "pagetablist"); //tabs
		ROLEKEY_MAP.put(ROLE_CLOCK, "clock");
		ROLEKEY_MAP.put(ROLE_SPLITBUTTON, "splitbutton");
		ROLEKEY_MAP.put(ROLE_IPADDRESS, "ipaddress");
		ROLEKEY_MAP.put(ROLE_ACCEL_LABEL, "accellabel");
		ROLEKEY_MAP.put(ROLE_ARROW, "arrow");
		ROLEKEY_MAP.put(ROLE_CANVAS, "canvas");
		ROLEKEY_MAP.put(ROLE_CHECK_MENU_ITEM, "checkmenuitem");
		ROLEKEY_MAP.put(ROLE_COLOR_CHOOSER, "colorchooser");
		ROLEKEY_MAP.put(ROLE_DATE_EDITOR, "dateeditor");
		ROLEKEY_MAP.put(ROLE_DESKTOP_ICON, "desktopicon");
		ROLEKEY_MAP.put(ROLE_DESKTOP_FRAME, "desktopframe");
		ROLEKEY_MAP.put(ROLE_DIRECTORY_PANE, "directorypane");
		ROLEKEY_MAP.put(ROLE_FILE_CHOOSER, "filechooser");
		ROLEKEY_MAP.put(ROLE_FONT_CHOOSER, "fontchooser");
		ROLEKEY_MAP.put(ROLE_CHROME_WINDOW, "chromewindow");
		ROLEKEY_MAP.put(ROLE_GLASS_PANE, "glasspane");
		ROLEKEY_MAP.put(ROLE_HTML_CONTAINER, "htmlcontainer");
		ROLEKEY_MAP.put(ROLE_ICON, "icon");
		ROLEKEY_MAP.put(ROLE_LABEL, "label");
		ROLEKEY_MAP.put(ROLE_LAYERED_PANE, "layeredpane");
		ROLEKEY_MAP.put(ROLE_OPTION_PANE, "optionpane");
		ROLEKEY_MAP.put(ROLE_PASSWORD_TEXT, "passwordtext");
		ROLEKEY_MAP.put(ROLE_POPUP_MENU, "popupmenu");
		ROLEKEY_MAP.put(ROLE_RADIO_MENU_ITEM, "radiomenuitem");
		ROLEKEY_MAP.put(ROLE_ROOT_PANE, "rootpane");
		ROLEKEY_MAP.put(ROLE_SCROLL_PANE, "scrollpane");
		ROLEKEY_MAP.put(ROLE_SPLIT_PANE, "splitpane");
		ROLEKEY_MAP.put(ROLE_TABLE_COLUMN_HEADER, "tablecolumnheader");
		ROLEKEY_MAP.put(ROLE_TABLE_ROW_HEADER, "tablerowheader");
		ROLEKEY_MAP.put(ROLE_TEAR_OFF_MENU_ITEM, "tearoffmenuitem");
		ROLEKEY_MAP.put(ROLE_TERMINAL, "terminal");
		ROLEKEY_MAP.put(ROLE_TEXT_CONTAINER, "textcontainer");
		ROLEKEY_MAP.put(ROLE_TOGGLE_BUTTON, "togglebutton");
		ROLEKEY_MAP.put(ROLE_TREE_TABLE, "treetable");
		ROLEKEY_MAP.put(ROLE_VIEWPORT, "viewport");
		ROLEKEY_MAP.put(ROLE_HEADER, "header");
		ROLEKEY_MAP.put(ROLE_FOOTER, "footer");
		ROLEKEY_MAP.put(ROLE_PARAGRAPH, "paragraph");
		ROLEKEY_MAP.put(ROLE_RULER, "ruler");
		ROLEKEY_MAP.put(ROLE_AUTOCOMPLETE, "autocomplete");
		ROLEKEY_MAP.put(ROLE_EDITBAR, "editbar");
		ROLEKEY_MAP.put(ROLE_ENTRY, "entry");
		ROLEKEY_MAP.put(ROLE_CAPTION, "caption");
		ROLEKEY_MAP.put(ROLE_DOCUMENT_FRAME, "documentframe");
		ROLEKEY_MAP.put(ROLE_HEADING, "heading");
		ROLEKEY_MAP.put(ROLE_PAGE, "page");
		ROLEKEY_MAP.put(ROLE_SECTION, "section");
		ROLEKEY_MAP.put(ROLE_REDUNDANT_OBJECT, "redundantobject");
		ROLEKEY_MAP.put(ROLE_FORM, "form");
		ROLEKEY_MAP.put(ROLE_IME, "ime");
		ROLEKEY_MAP.put(ROLE_APP_ROOT, "approot");
		ROLEKEY_MAP.put(ROLE_PARENT_MENUITEM, "parentmenuitem");
		ROLEKEY_MAP.put(ROLE_CALENDAR, "calendar");
		ROLEKEY_MAP.put(ROLE_COMBOBOX_LIST, "comboboxlist");
		ROLEKEY_MAP.put(ROLE_COMBOBOX_OPTION, "comboboxoption");
		ROLEKEY_MAP.put(ROLE_IMAGE_MAP, "imagemap");
		ROLEKEY_MAP.put(ROLE_OPTION, "option");
		ROLEKEY_MAP.put(ROLE_RICH_OPTION, "richoption");
		ROLEKEY_MAP.put(ROLE_LISTBOX, "listbox");
		ROLEKEY_MAP.put(ROLE_FLAT_EQUATION, "flatequation");
		ROLEKEY_MAP.put(ROLE_LAST_ENTRY, "lastentry");
		ROLEKEY_MAP.put(ROLE_GRAPHIC, "graphic");
	}
	
	private static nsIAccessibleRetrieval accService;
	
	private nsIAccessible _accessible;
	private nsIDOMNode _node;

	public MozillaAccessible (Object element) {
		if (accService == null) {
			try {
				accService = (nsIAccessibleRetrieval) Mozilla.getInstance().getComponentManager()
					.createInstanceByContractID("@mozilla.org/accessibleRetrieval;1",
							null, NS_IACCESSIBLERETRIEVAL_1_9_IID);
			}catch (XPCOMException e) {
				try {
					/* try xulrunner 1.8.1.3 uuid for nsIAccessibleRetrieval */
					accService = (nsIAccessibleRetrieval) Mozilla.getInstance().getComponentManager()
					.createInstanceByContractID("@mozilla.org/accessibleRetrieval;1",
							null, nsIAccessibleRetrieval.NS_IACCESSIBLERETRIEVAL_IID);
				} catch (XPCOMException e1) {
					throw new RuntimeException("Could not retreave nsIAccessibilityService", e1);
				}
			}
		}
		
		if (accService != null && element instanceof nsISupports) {
			nsIDOMNode node = null;
			try {
				node = (nsIDOMNode) ((nsISupports) element).queryInterface(nsIDOMNode.NS_IDOMNODE_IID);
				_accessible = accService.getAccessibleFor(node);
			}catch (Exception e) {
				_accessible = null;
				// Why are we not passing the exception in the println?
				// Some nodes do not have accessible objects so we just want
				// To report this on a single line without putting a stack trace in the log
				LoggingUtil.println(LoggingUtil.WARNINGS, "Could not create accessible for " + (node != null ? node.getNodeName() : "element"));
			}
			_node = node;
		}
	}

	/**
	 * wrap the given object as a ACTF <code>IAccessibleElement</code>. The 
	 * ACTF engine will invoke this constructor using a registered adaptor factory. Clients do not 
	 * typically call this constructor.
	 * 
	 * @param acc
	 */
	public MozillaAccessible (nsIAccessible acc) {
		_accessible = acc;
	}

	/** {@inheritDoc} */
	public void checkIsValid () throws InvalidComponentException {
		if (_accessible == null) {
			throw new InvalidComponentException("Invalid nsIAccessible"); 
		}
	}

	/** {@inheritDoc} */
	public String getAccessibleAction () throws InvalidComponentException {
		checkIsValid();
		short count = _accessible.getNumActions();
		StringBuffer sb = new StringBuffer();
		for (short s = 0; s < count; ++s) {
			sb.append(_accessible.getActionName(s));
			if (s == 0) {
				sb.append("[default]");
			}
			if (s < count - 1) {
				sb.append(',');
			}
		}
		return sb.toString();
	}

	/** {@inheritDoc} */
	public int getAccessibleChildCount () throws InvalidComponentException {
		checkIsValid();
		return _accessible.getChildCount();
	}

	/** {@inheritDoc} */
	public IAccessibleElement[] getAccessibleChildren ()
		throws InvalidComponentException {
		checkIsValid();
		int count = _accessible.getChildCount();
		IAccessibleElement[] children = new MozillaAccessible[count];
		for (int c = 0; c < count; ++c) {
			children[c] = new MozillaAccessible(_accessible.getChildAt(c));
		}
		return children;
	}

	/** {@inheritDoc} */
	public IAccessibleElement getAccessibleChild (int index) throws InvalidComponentException {
		checkIsValid();
		int count = _accessible.getChildCount();
		IAccessibleElement child = null;
		if (index < count) {
			child = new MozillaAccessible(_accessible.getChildAt(index));
		}
		return child;
	}

	public String getAccessibleDescription () throws InvalidComponentException {
		checkIsValid();
		return _accessible.getDescription();
	}

	/** {@inheritDoc} */
	public String getAccessibleKeyboardShortcut ()
		throws InvalidComponentException {
		checkIsValid();
		return _accessible.getKeyboardShortcut();
	}

	/** {@inheritDoc} */
	public String getAccessibleName () throws InvalidComponentException {
		checkIsValid();
		return _accessible.getName();
	}

	/** {@inheritDoc} */
	public IAccessibleElement getAccessibleParent ()
		throws InvalidComponentException {
		checkIsValid();
		return new MozillaAccessible(_accessible.getParent());
	}

	/** {@inheritDoc} */
	public int getAccessibleIndexInParent ()
		throws InvalidComponentException {
		checkIsValid();
		return _accessible.getIndexInParent();
	}

	public Map<Long, nsIAccessible> getAccessibleRelations () throws InvalidComponentException {
		checkIsValid();
		HashMap<Long, nsIAccessible> relMap = new HashMap<Long, nsIAccessible>();
		for (Iterator<Long> iter = RELATIONKEY_MAP.keySet().iterator(); iter.hasNext();) {
			Long relType = iter.next();
			nsIAccessible acc = _accessible.getAccessibleRelated(relType.longValue());
			if (acc != null) {
				relMap.put(relType, acc);
			}
		}
		return relMap;
	}

	/** {@inheritDoc} */
	public String getAccessibleRole () throws InvalidComponentException {
		checkIsValid();
		return getActfRoleName(_accessible.getFinalRole());
	}

	/** {@inheritDoc} */
	public IAccessibleElement[] getAccessibleSelection ()
		throws InvalidComponentException {
		checkIsValid();
		nsIAccessibleSelectable accSelectable = (nsIAccessibleSelectable) _accessible.queryInterface(nsIAccessibleSelectable.NS_IACCESSIBLESELECTABLE_IID);
		int count = accSelectable.getSelectionCount();
		IAccessibleElement[] selections = new MozillaAccessible[count];
		nsIArray selectables = accSelectable.getSelectedChildren();
		int length = (int) selectables.getLength();
		for (int s = 0; s < length; ++s) {
			selections[s] = new MozillaAccessible((nsIAccessible) selectables.queryElementAt(
				s, nsIAccessible.NS_IACCESSIBLE_IID));
		}
		return selections;
	}

	/** {@inheritDoc} */
	public Set<String> getAccessibleState () throws InvalidComponentException {
		checkIsValid();
		HashSet<String> res = new HashSet<String>();
		long state = _accessible.getFinalState();
		// We must always set the visible/invisible state so that there
		// is a consistent property for the client to test against
		if ((state & nsIAccessible.STATE_INVISIBLE) != 0) {
			res.add(AccessibleConstants.STATE_INVISIBLE);
		}else {
			res.add(AccessibleConstants.STATE_VISIBLE);
		}
		if ((state & nsIAccessible.STATE_BUSY) != 0) {
			res.add(AccessibleConstants.STATE_BUSY);
		}
		if ((state & nsIAccessible.STATE_CHECKED) != 0) {
			res.add(AccessibleConstants.STATE_CHECKED);
		}
		if ((state & nsIAccessible.STATE_CHECKABLE) != 0) {
			res.add(AccessibleConstants.STATE_CHECKABLE);
		}
		if ((state & nsIAccessible.STATE_COLLAPSED) != 0) {
			res.add(AccessibleConstants.STATE_COLLAPSED);
		}
		if ((state & nsIAccessible.STATE_EXPANDED) != 0) {
			res.add(AccessibleConstants.STATE_EXPANDABLE);
		}
		if ((state & nsIAccessible.STATE_FOCUSABLE) != 0) {
			res.add(AccessibleConstants.STATE_FOCUSABLE);
		}
		if ((state & nsIAccessible.STATE_FOCUSED) != 0) {
			res.add(AccessibleConstants.STATE_FOCUSED);
		}
		if ((state & nsIAccessible.STATE_HOTTRACKED) != 0) {
			res.add(AccessibleConstants.STATE_HOT_TRACKED);
		}
		if ((state & nsIAccessible.STATE_LINKED) != 0) {
			res.add(AccessibleConstants.STATE_LINKED);
		}
		if ((state & nsIAccessible.STATE_DEFAULT) != 0) {
			res.add(AccessibleConstants.STATE_NORMAL);
		}
		if ((state & nsIAccessible.STATE_OFFSCREEN) != 0) {
			res.add(AccessibleConstants.STATE_OFF_SCREEN);
		}
		if ((state & nsIAccessible.STATE_PRESSED) != 0) {
			res.add(AccessibleConstants.STATE_PRESSED);
		}
		if ((state & nsIAccessible.STATE_INVALID) != 0) {
			res.add(AccessibleConstants.STATE_INVALID);
		}
		if ((state & nsIAccessible.STATE_REQUIRED) != 0) {
			res.add(AccessibleConstants.STATE_REQUIRED);
		}
		if ((state & nsIAccessible.STATE_READONLY) != 0) {
			res.add(AccessibleConstants.STATE_READ_ONLY);
		}
		if ((state & nsIAccessible.STATE_SELECTABLE) != 0) {
			res.add(AccessibleConstants.STATE_SELECTABLE);
		}
		if ((state & nsIAccessible.STATE_SELECTED) != 0) {
			res.add(AccessibleConstants.STATE_SELECTED);
		}
		if ((state & nsIAccessible.STATE_SIZEABLE) != 0) {
			res.add(AccessibleConstants.STATE_SIZEABLE);
		}
		return res;
	}

	/** {@inheritDoc} */
	public String getAccessibleValue () throws InvalidComponentException {
		checkIsValid();
		return _accessible.getFinalValue();
	}

	/** {@inheritDoc} */
	public Object element () {
		return _node;
	}

	private String getActfRoleName (long role) {
		//	2. attempt to retrieve the role name with the role long via Msaa and IA2
		// class' getRoleName(long) methods
		// 3. if not found, use a static map that you create mapping the long role values
		// to a role name (e.g. ROLE_GRAPHIC => 'graphic') to determine the role name for
		// the role value that you obtain from the getFinalRole call
		
		String res = null;
		//try to get a role from MSAA first
		res = Msaa.getMsaaActfRoleName(role);
		if (res == null){ // next try IA2
			res = IA2.getIA2ActfRoleName(role);
			if (res == null){ //finally get it from map
				res = ROLEKEY_MAP.get(role);
			}
		}
		return res;
	}

	public Rectangle getAccessibleLocation() throws InvalidComponentException {
		checkIsValid();
		int[] x = new int[1];
		int[] y = new int[1];
		int[] wid = new int[1];
		int[] ht = new int[1];
		_accessible.getBounds(x, y, wid, ht);
		return new Rectangle(x[1], y[1], wid[1], ht[1]);
	}

}
