/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  IBM Corporation - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.model.javapp;

import java.awt.Component;
import java.awt.EventQueue;

import org.eclipse.actf.model.AbstractRenderableModel;
import org.eclipse.actf.model.IRenderableModel;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.logging.LoggingUtil;

/**
 * This class embodies essential characteristics of the Java AWT model.
 * Documents wishing to describe an AWT GUI should use the 'awt' value in the
 * 'amodel' attribute of the root &lt;gui&gt; tag.
 * 
 * @author Mike Squillace
 */
public class AwtGuiModel extends AbstractRenderableModel
{

	public static final String AWT_MODEL = "awt";
	
	private static final String[] PACKAGE_LIST = new String[] {
			"java.lang", "java.awt", "java.awt.event"
	};
	
	static {
		// IGNORABLES.add(javax.swing.CellRendererPane.class);
	}

	/**
	 * create a new AwtGuiModel
	 */
	public AwtGuiModel () {
		super(AWT_MODEL);
		locator = null;
	}

	protected AwtGuiModel (String name) {
		super(name);
	}
	
	/**
	 * alternate method names returned include:
	 * <p>
	 * <ul>
	 * <li>'set' + rootName
	 * <li>'add' + rootName
	 * <li>'create' + rootName
	 * </ul>
	 * 
	 * <p>
	 * First char of rootName is converted to upper case
	 * 
	 * @param rootName
	 *            root name of method
	 * @return list of alternate method names based upon root name
	 */
	public String[] getAlternateMethodNames (String rootName) {
		String[] names = new String[3];
		names[0] = "set" + LoggingUtil.firstCharToUpper(rootName);
		names[1] = "add" + LoggingUtil.firstCharToUpper(rootName);
		names[2] = "create" + LoggingUtil.firstCharToUpper(rootName);
		return names;
	}

	/** @return "java.awt" */
	public String getDefaultAliasPrefix () {
		return "java.awt";
	}

	/**
	 * 
	 * <p>
	 * Includes packages:
	 * <p>
	 * <ul>
	 * <li> java.lang
	 * <li> java.awt
	 * <li> java.awt.event
	 * </ul>
	 * 
	 * @return list of packages that any code processor should import
	 */
	public String[] getPackageNames () {
		return PACKAGE_LIST;
	}

	/**
	 * return <code>false</code> since AWT GUIs are typically (and most
	 * effectively) bottom-up
	 * 
	 * @return <code>false</code>
	 */
	public boolean isTopDown () {
		return false;
	}

	/**
	 * {@inheritDoc} This implementation uses the <code>setName(String)</code>
	 * method to assign a component an id.
	 */
	public void setNodeID (Object comp, String id) {
		((Component) comp).setName(id);
	}

	/**
	 * returns whether or not the currently executing thread is the UI thread.
	 * For AWT, the UI thread is the event dispatch thread.
	 * 
	 * @return <code>true</code> if the current thread is the event dispatch
	 *         thread, <code>false</code> otherwise
	 * @see IRenderableModel#isUIThread()
	 */
	public boolean isUIThread () {
		return EventQueue.isDispatchThread();
	}

	/** {@inheritDoc} */
	public boolean isVisible (Object component) {
		return !(component instanceof Component)
				|| ((Component) component).isShowing();
	}

	/**
	 * executes the given Runnable from within the UI thread. This method will
	 * typically be called if <code>isUIThread</code> returns
	 * <code>false</code>.
	 * 
	 * @param runnable -
	 *            Runnable to be invoked in UI thread
	 * @see IRenderableModel#invokeOnUIThread(Runnable)
	 */
	public void invokeOnUIThread (Runnable runnable) {
		try {
			EventQueue.invokeAndWait(runnable);
		}catch (Exception e) {
			LoggingUtil.println(IReporter.SYSTEM_NONFATAL, e);
		}
	}

	/** {@inheritDoc} */
	public void asyncInvokeOnUIThread (Runnable runnable) {
		EventQueue.invokeLater(runnable);
	}

	/** {@inheritDoc} */
	public boolean requestFocusFor (Object comp) {
		boolean res = false;
		if (comp instanceof Component && comp != null) {
			res = ((Component) comp).requestFocusInWindow();
		}
		return res;
	}
	
} // AwtGuiModel
