/*******************************************************************************
 * Copyright (c) 2004, 2008 IBM Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.actf.model.javapp;

import java.awt.Component;
import java.awt.Window;

import javax.swing.JComponent;
import javax.swing.SwingUtilities;

import org.eclipse.actf.core.adapt.DefaultAdaptorFactory;
import org.eclipse.actf.core.adapt.IAdaptor;
import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.model.IRenderableModel;
import org.eclipse.actf.model.javapp.locate.SwingComponentLocator;
import org.eclipse.actf.model.javapp.traverse.ContainerTreeNodeWalker;
import org.eclipse.actf.model.javapp.traverse.SwingNodeItemFilter;
import org.eclipse.actf.model.locate.INodeLocator;
import org.eclipse.actf.model.traverse.INodeWalker;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.logging.LoggingUtil;
import org.eclipse.actf.util.resources.ClassLoaderCache;

/**
 * implementation for the Java Swing GUI model
 * 
 * @author Mike Squillace
 */
public class SwingGuiModel extends AwtGuiModel
{

	public static final String SWING_MODEL = "swing";	
	
	private static final String[] PACKAGE_LIST = new String[] {
			"java.lang", "java.awt", "java.awt.event", "javax.swing"
	};

	public static final String SWING_ADAPTOR_CLASS = "org.eclipse.actf.accservice.javapp.adapt.SwingAccessibilityAdaptor";
	/**
	 * create a new SwingGuiModel
	 */
	public SwingGuiModel () {
		super(SWING_MODEL);
		treeNodeWalker = null;
		Class adaptorCls = ClassLoaderCache.getDefault().classForName(SWING_ADAPTOR_CLASS);
		IAdaptor adaptor = null;
		try {
			adaptor = (IAdaptor) adaptorCls.newInstance();
		} catch (Exception e) {
			LoggingUtil.println(
					IReporter.SYSTEM_NONFATAL, "Cannot get adaptor class for Swing Model", e);
		}
		if (adaptor != null){
			DefaultAdaptorFactory.getInstance().registerAdaptor(JComponent.class,adaptor);
		}
	}

	/** {@inheritDoc} */
	public String getDefaultAliasPrefix () {
		return "javax.swing";
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * Includes packages:
	 * <p>
	 * <ul>
	 * <li> java.lang
	 * <li> java.awt
	 * <li> java.awt.event
	 * <li> javax.swing
	 * </ul>
	 */
	public String[] getPackageNames () {
		return PACKAGE_LIST;
	}

	/**
	 * returns whether or not the currently executing thread is the UI thread.
	 * For Swing, the UI thread is the thread is the event dispatch thread.
	 * 
	 * @return <code>true</code> if the current thread is the event dispatch
	 *         thread, <code>false</code> otherwise
	 * @see IRenderableModel#isUIThread()
	 */
	public boolean isUIThread () {
		return SwingUtilities.isEventDispatchThread();
	}

	/** {@inheritDoc} */
	public boolean isVisible (Object component) {
		boolean result = false;
		if (component instanceof Component) {
			Component c = (Component) component;
			Window parentWindow = SwingUtilities.getWindowAncestor(c);
			result = c.isVisible()
					&& (parentWindow == null || parentWindow.isVisible());
		}
		return result;
	}

	/**
	 * executes the given Runnable from within the UI thread. This method will
	 * typically be called if <code>isUIThread</code> returns
	 * <code>false</code>.
	 * 
	 * @param runnable -
	 *            Runnable to be invoked in UI thread
	 * @see IRenderableModel#invokeOnUIThread(Runnable)
	 */
	public void invokeOnUIThread (Runnable runnable) {
		try {
			SwingUtilities.invokeAndWait(runnable);
		}catch (Exception e) {
			LoggingUtil.println(IReporter.SYSTEM_NONFATAL, e);
		}
	}

	/** {@inheritDoc} */
	public void asyncInvokeOnUIThread (Runnable runnable) {
		SwingUtilities.invokeLater(runnable);
	}

	/** {@inheritDoc} */
	public INodeWalker getNodeWalker () {
		if ( treeNodeWalker == null ) {
			treeNodeWalker = new ContainerTreeNodeWalker();
			setFilters();
			boolean showItems = configuration.getBooleanParameter(IConfiguration.SHOW_ITEMS_KEY);
			if (!showItems){
				SwingNodeItemFilter itemsFilter = new SwingNodeItemFilter();
				treeNodeWalker.addNodeFilter(itemsFilter);
			}
		}
		return treeNodeWalker;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.actf.model.AbstractModel#getNodeLocator()
	 */
	@Override
	public INodeLocator getNodeLocator() {
		if (locator == null) {
			locator = new SwingComponentLocator();
		}
		return locator;
	}

} // SwingGuiModel
