/*******************************************************************************
 * Copyright (c) 2004, 2008 IBM Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.actf.model.javapp.traverse;

import java.lang.reflect.Method;
import java.util.ArrayList;

import org.eclipse.actf.model.DefaultModelFactory;
import org.eclipse.actf.model.InvalidComponentException;
import org.eclipse.actf.model.javapp.SwtGuiModel;
import org.eclipse.actf.model.traverse.AbstractTreeNodeWalker;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.logging.LoggingUtil;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Item;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;

/**
 *  used to walk a tree of <code>org.eclipse.swt.widgets.Composite</code> nodes. If a node is an instance of <code>org.eclipse.swt.widgets.Shell</code>,
 *  the child shells of this node will not be included in the children returned. Thus, a call to <code>hasChildren()</code> that returns <code>false</code> does not mean that
 *  the particular element has no children at all.
 *
 * <p>Note: Composite objects that have <code>org.eclipse.swt.widgets.Item</code> children
 * will have two children for each such <code>Item</code> object - one for the <code>Item</code>
 * object itself and one for the <code>Control</code> object returned by <code>getControl</code>,
 * should that item support this method.</p>
 *  
 * @author Mike Squillace
 */
public class SwtTreeNodeWalker extends AbstractTreeNodeWalker
{

	private SwtGuiModel _model;

	public SwtTreeNodeWalker () {
		super();
		_model = (SwtGuiModel) DefaultModelFactory.getInstance()
		.resolveModel(SwtGuiModel.SWT_MODEL);
	} // ctor

	/**
	 * {@inheritDoc}
	 * <p>
	 * For convenience, the menu of any control object is also counted as a child of that control. Subsequent menu items
	 * that have menus as children will also be traversed by this method.
	 * </p>
	 */
	public Object[] getChildren (final Object element) throws InvalidComponentException {
		final Object[] children = new Object[1];
		Object bridgedChild = getBridgedChild(element);
		if (bridgedChild != null) {
			children[0] = new Object[] {bridgedChild};
		} else if (element instanceof Composite
				|| element instanceof Menu
				|| element instanceof Item){
			_model.invokeOnUIThread(new Runnable() {
				public void run () {
					try {
						children[0] = getAllChildren(element);
						if (children[0] == null) {
							children[0] = new Object[0];
						}
					} catch (InvalidComponentException e) {
						LoggingUtil.println(IReporter.SYSTEM_NONFATAL, e);
					}
				}
			});
		}

		return children[0] == null ? new Object[0] : (Object[]) children[0];		
	}

	/** {@inheritDoc} */
	public Object getParent (final Object element) throws InvalidComponentException {
		Object bridgedParent = getBridgedParent(element);
		final Object[] parent = new Object[1];

		if (bridgedParent != null) {
			parent[0] = bridgedParent;
		} else {
			_model.invokeOnUIThread(new Runnable() {
				public void run () {
					if (element instanceof Control) {
						parent[0] = ((Control) element).getParent();	
					} else if (element instanceof Menu) {
						Menu menu = (Menu) element;
							parent[0] = menu.getParentItem();	
							if (parent[0] == null) {
								parent[0] = menu.getParentMenu();
							}
							if (parent[0] == null) {
								parent[0] = menu.getParent();
							}
					} else if (element instanceof MenuItem) {
						parent[0] = ((MenuItem) element).getParent();
					} else if (element instanceof Item) {
						// attempt to get a parentItem first, then the parent, which will
						// be the actual main control (e.g. Tree, Menu)
						try {
							Method parentItemMeth = element.getClass().getMethod("getParentItem", (Class[]) null);
							parent[0] = parentItemMeth.invoke(element, (Object[]) null);
							if (parent[0] == null) {
								Method parentMeth = element.getClass().getMethod("getParent", (Class[]) null);
								parent[0] = parentMeth.invoke(element, (Object[]) null);
							}
						} catch (Exception e) {
						}
					} else {
						LoggingUtil.println(LoggingUtil.WARNINGS, "No known parent for element " + element);
					}
				}
			});
		}

		return parent[0];
	}

	/** {@inheritDoc} */
	public Object[] getStartNodes () {
		return Display.getDefault().getShells();
	}

	protected Object[] getAllChildren (final Object element) throws InvalidComponentException {
		Object[] children = null;
		Object[] itemChildren = null;

		if (element instanceof Shell) {
			// we want the direct child shells, the menubar (if any), and all controls
			children = getShellChildren(element);
		} else if (element instanceof Composite || element instanceof Menu) {
			// get items first for widgets like Tree, Toolbar, etc.
			// if no items, get standard Composite children
			itemChildren = getItemChildren(element);
			if (itemChildren == null && element instanceof Composite) {
				children = ((Composite) element).getChildren();
			}

			if (itemChildren != null) {
				// if children are Items, we want both the
				// item and the control/menu corresponding to that item, if getControl/getMenu method
				// can be called on the item
				try {
					children = addAssociatedControls(itemChildren);
				}catch (Exception e) {
					LoggingUtil.println(IReporter.SYSTEM_NONFATAL, e);
				}
			}
		} else if (element instanceof Item) {
			children = getItemChildren(element);
		}

		return children == null ? new Object[0] : children;
	}

	protected Object[] addAssociatedControls (Object[] children) throws Exception {
		ArrayList controls = new ArrayList();
		Object ctrl = null, menu = null;

		for (int c = 0; c < children.length; ++c) {
			Object child = children[c];
			if (child instanceof MenuItem) {
				menu = ((MenuItem) child).getMenu();
			}
			if (menu == null && child instanceof Item) {
				try {
					Method meth = child.getClass().getMethod("getControl", (Class[]) null);
					ctrl = meth.invoke(child, (Object[]) null);
				} catch (Exception e) {
				}
			}
			if (ctrl != null) {
				controls.add(ctrl);
			} else if (menu != null) {
				controls.add(menu);
			}
		} // next child
		
		Object[] childrenx = new Object[children.length + controls.size()];
		System.arraycopy(children, 0, childrenx, 0, children.length);
		System.arraycopy(controls.toArray(), 0, childrenx, children.length, controls.size());

		return childrenx;
	}

	protected Object[] getShellChildren (Object element) {
		Shell shellElem = (Shell) element;
		Shell[] shells = shellElem.getShells();
		Menu menuBar = shellElem.getMenuBar();
		Control[] ctrlChildren = shellElem.getChildren();
		ArrayList childShellList = new ArrayList();
		Object[] children = null;

		// get only shells that are children of this element
		for (int s = 0; shells != null && s < shells.length; ++s) {
			try {
				if (getParent(shells[s]) == element) {
					childShellList.add(shells[s]);
				}
			}catch (InvalidComponentException e) {
				continue;
			}
		}
		Object[] childShells = childShellList.toArray();
		children = new Object[childShells.length + ctrlChildren.length + (menuBar != null ? 1 : 0)];
		System.arraycopy(childShells, 0, children, 0, childShells.length);
		System.arraycopy(ctrlChildren, 0, children, childShells.length, ctrlChildren.length);
		if (menuBar != null) {
			children[children.length - 1] = menuBar;
		}

		return children;
	}

	protected Object[] getItemChildren (final Object element) {
		Object[] items = null;
		try {
			Method getItemsMeth = element.getClass().getMethod("getItems", (Class[]) null);
			items = (Object[]) getItemsMeth.invoke(element, (Object[]) null);
			if (items != null && items.length == 0) {
				items = null;
			}
		} catch (Exception e) {
		}

		return items;
	}

} // SwtTreeNodeWalker
