/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.model.mozdom;

import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.List;

import org.eclipse.actf.model.AbstractRenderableModel;
import org.eclipse.actf.model.dom.traverse.DomTreeNodeWalker;
import org.eclipse.actf.util.logging.LoggingUtil;
import org.eclipse.swt.widgets.Display;


/**
 * implementation for an embedded browser widget. The widget is assumed to
 * extend <code>org.eclipse.swt.widgets.Composite</code> and the DOM exposed
 * by this widget is assumed to implement the W3C HTML 4.01 DOM specification.
 * 
 * @see <a href="http://www.w3.org/TR/DOM-Level-2-HTML/html.html">HTML DOM
 *      specification</a>
 * @author Mike Squillace
 * 
 */
public abstract class DomBrowserGuiModel extends AbstractRenderableModel {

	protected static String[] PACKAGE_LIST = new String[] { 
		"java.lang", "org.w3c.dom", "org.w3c.dom.html" 
	};
	
	protected static List BASE_ELEMENT_TAGNAMES = Arrays.asList(new String[] {
		"span", "sub", "sup",
		"tt", "i", "b", "u", "s", "strike", "big", "small",
		"em", "strong", "abbr", "code",
		"dd", "dt", "noframes", "noscript"
	});

	public DomBrowserGuiModel(String type) {
		super(type);
		treeNodeWalker = new DomTreeNodeWalker();
		setFilters();
	}

	/** {@inheritDoc} */
	public String getDefaultAliasPrefix() {
		return "org.w3c.dom.html";
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * Includes packages:
	 * <p>
	 * <ul>
	 * <li> java.lang
	 * <li> org.w3c.dom
	 * <li> org.w3c.dom.html
	 * </ul>
	 */
	public String[] getPackageNames() {
		return PACKAGE_LIST;
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * uses <code>org.eclipse.swt.widgets.Display</code> methods
	 */
	public boolean isUIThread() {
		return Display.getDefault().getThread() == Thread.currentThread();
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * uses <code>org.eclipse.swt.widgets.Display</code> methods
	 */
	public void invokeOnUIThread(Runnable runnable) {
		Display.getDefault().syncExec(runnable);
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * uses <code>org.eclipse.swt.widgets.Display</code> methods
	 */
	public void asyncInvokeOnUIThread(Runnable runnable) {
		Display.getDefault().asyncExec(runnable);
	}

	/**
	 * invoke the getter method on the specified object. The method name must
	 * name a method that has no formal parameters.
	 * 
	 * <p>
	 * Note: If the specified method is found and returns a <code>boolean</code>,
	 * then an unsuccessful invocation will result in a return value of
	 * <code>false</code>.
	 * 
	 * @param obj -
	 *            receiver
	 * @param name -
	 *            method name
	 * @return result of invocation
	 * @throws Exception
	 */
	public Object invokeGetter(Object obj, String name) throws Exception {
		return invokeGetter(obj, name, null);
	}

	/**
	 * invoke the specified getter method on the given object, passing it the
	 * single parameter.
	 * 
	 * <p>
	 * Note: If the specified method is found and returns a <code>boolean</code>,
	 * then an unsuccessful invocation will result in a return value of
	 * <code>false</code>. *
	 * 
	 * @param obj -
	 *            the receiver
	 * @param name -
	 *            the method name
	 * @param arg -
	 *            single argument to method
	 * @return result of invokation
	 * @throws Exception
	 */
	public Object invokeGetter(final Object obj, final String name,
			final Object arg) throws Exception {
		final Object[] result = new Object[1];
		final Exception[] t = new Exception[1];
		invokeOnUIThread(new Runnable() {

			public void run() {
				Method meth = null;
				if (obj != null) {
					try {
						meth = obj.getClass().getMethod(
								name,
								arg == null ? new Class[0] : new Class[] { arg
										.getClass() });
						result[0] = meth.invoke(obj,
								arg == null ? new Object[0]
										: new Object[] { arg });
					} catch (Exception e) {
						// Utils.println(Reporter.SYSTEM_NONFATAL, "Could not
						// execute method " + (meth == null ? "on object " + obj
						// : meth.toString()), e);
						t[0] = e;
					}
				}
			}
		});
		if (t[0] != null) {
			throw t[0];
		}
		return result[0];
	}

	protected String getDomTypesPackage() {
		return "org.w3c.dom.html";
	}

	protected String getTypeNamePrefix() {
		return "HTML";
	}

	protected String getTypeNameSuffix() {
		return "Element";
	}

	/**
	 * get the DOM class/interface type name for the given HTML tag name
	 * 
	 * @param tagName
	 *            HTML 4.01 tag name
	 * @return type name corresponding to given tag name
	 */
	public String tagNameToTypeName(String tagName) {
		tagName = tagName.toLowerCase();
		if (tagName.equals("a")) {
			tagName = "anchor";
		} else if (tagName.equals("br")) {
			tagName = "BR";
		} else if (tagName.equals("fieldset")) {
			tagName = "fieldSet";
		} else if (tagName.charAt(0) == 'h'
				&& Character.isDigit(tagName.charAt(1))) {
			tagName = "Heading";
		} else if (tagName.equals("hr")) {
			tagName = "HR";
		} else if (tagName.equals("img")) {
			tagName = "image";
		} else if (tagName.equals("ol")) {
			tagName = "oList";
		} else if (tagName.equals("ul")) {
			tagName = "uList";
		} else if (tagName.equals("li")) {
			tagName = "LI";
		} else if (tagName.equals("p")) {
			tagName = "paragraph";
		} else if (tagName.equals("td") || tagName.equals("th")) {
			tagName = "tableCell";
		} else if (tagName.equals("thead") || tagName.equals("tbody")
				|| tagName.equals("tfoot")) {
			tagName = "tableSection";
		} else if (tagName.equals("textarea")) {
			tagName = "textArea";
		} else if (tagName.equals("tr")) {
			tagName = "tableRow";
		} else if (tagName.equals("caption")) {
			tagName = "tableCaption";
		} else if (tagName.equals("frameset")) {
			tagName = "frameSet";
		} else if (BASE_ELEMENT_TAGNAMES.contains(tagName)) {
			tagName = "base";
}

		StringBuffer sb = new StringBuffer(getDomTypesPackage());
		sb.append('.');
		sb.append(getTypeNamePrefix());
		sb.append(LoggingUtil.firstCharToUpper(tagName));
		sb.append(getTypeNameSuffix());
		
		// Test that the class exists - if it doesnt then set it to
		// nsIDOMHTMLElement
		try {
			Class c = Class.forName(sb.toString());
		} catch (Exception e) {
			sb.setLength(0);
			sb.append(getDomTypesPackage());
			sb.append('.');
			sb.append(getTypeNamePrefix());
			sb.append(getTypeNameSuffix());
			LoggingUtil.println(LoggingUtil.WARNINGS, "Unknown tag: <" + tagName + "> using " + sb.toString());
		}
		return sb.toString();
	}
	
	/** {@inheritDoc} */
	public String getTypeName (String nodeName) {
		return tagNameToTypeName(nodeName);
	}
}
