/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  IBM Corporation - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.core;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.regex.Pattern;

import org.eclipse.actf.core.config.ConfigurationException;
import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.core.runtime.IRuntimeContext;
import org.eclipse.actf.core.runtime.RuntimeContextFactory;
import org.eclipse.actf.model.IRenderableModel;
import org.eclipse.actf.model.IModel;
import org.eclipse.actf.model.InvalidComponentException;
import org.eclipse.actf.model.locate.INodeLocator;
import org.eclipse.actf.model.traverse.ITreeNodeWalker;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.logging.PrintWriterReporter;
import org.eclipse.actf.util.logging.LoggingUtil;
import org.eclipse.actf.util.resources.ClassLoaderCache;
import org.eclipse.actf.validation.ValidationConstants;
import org.eclipse.actf.validation.valdoc.codeproc.ICodeProcessor;
import org.eclipse.actf.validation.valdoc.rulebase.ValidationDocumentRuleBase;
import org.eclipse.actf.validation.rulebase.IRuleBase;
import org.eclipse.actf.validation.rulebase.RuleBaseManager;
import org.eclipse.actf.validation.rules.IRuleSet;
import org.eclipse.actf.validation.validators.AbstractValidationContext;
import org.eclipse.actf.validation.validators.AccessibilityValidationContext;
import org.eclipse.actf.validation.validators.IValidationContext;
import org.eclipse.actf.validation.validators.IValidator;
import org.eclipse.actf.validation.validators.ValidationContextConstants;

/**
 * a validation process that walks a hierarchy of GUI components and validates
 * each component with the appropriate validator. The GUI is traversed using
 * classes in the <code>org.eclipse.actf.model.traverse </code>
 * package.
 * 
 * @see org.eclipse.actf.model.traverse.ITreeNodeWalker
 * @author John Buslawski
 */
public class AccessibilityValidationProcess extends AbstractValidationProcess {

	ValidationDocumentRuleBase valDocRB = null;

	Map validatorMap = new HashMap();

	public AccessibilityValidationProcess(IRuleBase iRuleBase,
			ExecutionPoint execPoint, IValidationContext context) {
		setRuleBase(iRuleBase);
		setExecutionPoint(execPoint);
		setValidationContext(context);
	}

	protected AccessibilityValidationProcess() {
	}

	/**
	 * {@inheritDoc} prepares the <code>IValidationContext</code> and
	 * calculates initial work
	 * 
	 * @see IValidationContext
	 */
	public void init() throws ValidationProcessException {
		if (valDocRB == null) {
			if (ruleBase != null && ruleBase.getClass().isAssignableFrom(ValidationDocumentRuleBase.class)) {
				valDocRB = (ValidationDocumentRuleBase) getRuleBase();
			} else {
				throw new ValidationProcessException("Unknown IRuleBase type in AccessibilityValidationProcess.init(). ");
			}
		}
		
		if (context.getClass().isAssignableFrom(AccessibilityValidationContext.class)) {
			// Build the remainder of the IValidationContext if necessary. The ICodeProcessor and
			// IModel are in the IRuleBase.
			IConfiguration configuration = (IConfiguration) context
					.getParameter(ValidationContextConstants.CONFIGURATION);

			if (configuration == null) {
				try {
					IRuntimeContext runtimeContext = RuntimeContextFactory.getInstance().getRuntimeContext();
					configuration = runtimeContext.getConfiguration();

					context.setParameter(
							ValidationContextConstants.CONFIGURATION, configuration);
				} catch (ConfigurationException e) {
					// Could not obtain a Configuration. We can't do much
					// here but quit.
					throw new RuntimeException("Could not obobtain configuration object in process init", e);
				}
			}
			
			if (context.getParameter(ValidationContextConstants.CODE_PROCESSOR) == null) {
				context.setParameter(
						ValidationContextConstants.CODE_PROCESSOR, valDocRB.getCodeProcessor());
			}
			if (context.getParameter(ValidationContextConstants.MODEL) == null) {
				IModel model = valDocRB.getModel();
				context.setParameter(ValidationContextConstants.MODEL, model);
				// MAS: set other model-specific context parameters
				INodeLocator locator = model.getNodeLocator();

				try {
					context.setParameter(
									ValidationContextConstants.ELEMENT_LOCATOR, locator);
				} catch (Exception e) {
					LoggingUtil.println(IReporter.SYSTEM_NONFATAL,
							"Could not find component locator for model: " + model.getName());
				}
			}
			if (context.getParameter(ValidationContextConstants.VALIDATED_ELEMENTS) == null) {
				context.setParameter(
						ValidationContextConstants.VALIDATED_ELEMENTS, new HashSet());
			}
		} else {
			throw new ValidationProcessException("Incompatible IValidationContext type in AccessibilityValidationProcess.init(). ");
		}
		
		calculateWork(execPoint, ruleBase, context);
		initProgressEstimates();
	}

	/**
	 * {@inheritDocsimply calls <code>validate</code> method}
	 * 
	 * @throws ValidationProcessException
	 */
	public void run() throws ValidationProcessException {
		validate(execPoint, ruleBase, context);
	}

	/**
	 * validate the objects in the tree with the given target as the root. All
	 * nodes should be instances of the appropriate type(s), which includes the
	 * primary type appropriate for the model and, optionally, a type
	 * that provides additional features and properties of the primary type,
	 * called its context.
	 * 
	 * @param execPoint --
	 *            the ExecutionPoint that triggered this validation
	 * @param ruleBase --
	 *            the IRuleBase that matches this target
	 * @param context --
	 *            the IValidationContext to use when evaluating rules
	 */
	protected void validate(ExecutionPoint execPoint, IRuleBase ruleBase, IValidationContext context) throws ValidationProcessException {
		if (isTerminated()) {
			return;
		}
		
		IReporter reporter = null;
		validatorMap.clear();
		Object target = execPoint.getTarget();
		if (target != null) {
			try {
				IModel model = (IModel) context.getParameter(ValidationContextConstants.MODEL);
				if (model instanceof IRenderableModel) {
					((IRenderableModel) model).requestFocusFor(target);
				}
				reporter = (IReporter) context.getParameter(ValidationContextConstants.REPORTER);
				reporter.startReport(target);
				IConfiguration config = (IConfiguration) context.getParameter(ValidationContextConstants.CONFIGURATION);
				reporter.report(IReporter.DETAIL, "Beginning validation for target: " + target.getClass().getName() + "...");
				ICodeProcessor codeProcessor = (ICodeProcessor) context.getParameter(ValidationContextConstants.CODE_PROCESSOR);
				if (codeProcessor.getScript("init") != null) {
					codeProcessor.processCode(new String[] { "init" });
				}
				codeProcessor.setLocal("rootComponent", target, "true");
				validateComponent(target, execPoint, ruleBase, context);
				reporter.report(IReporter.DETAIL, "Validation for component with id " + target.getClass().getName() + " complete.");
				if (reporter instanceof PrintWriterReporter) {
					PrintWriterReporter pwr = (PrintWriterReporter) reporter;
					reporter.report(IReporter.INFO, "Results of validation: "
							+ pwr.getErrorCount() + " errors, "
							+ pwr.getWarningCount() + " warnings");
					pwr.resetCount();
				}
			} catch (Exception e) {
				LoggingUtil.println(IReporter.SYSTEM_FATAL, e);
			} finally {
				if (reporter != null) {
					reporter.flush();
					reporter.endReport();
				}
			}
		} else {
			throw new ValidationProcessException(
					"No target object to validate.");
		}
	}

	/**
	 * used to validate single components.
	 * 
	 * @param target --
	 *            the target object to validate
	 * @param execPoint --
	 *            the ExecutionPoint that triggered this validation
	 * @param ruleBase --
	 *            the IRuleBase that matches this target
	 * @param context --
	 *            the IValidationContext to use when evaluating rules
	 */
	protected void validateComponent(Object target, ExecutionPoint execPoint,
									 IRuleBase ruleBase, IValidationContext context)
			throws ValidationProcessException {
		IModel model = (IModel) context.getParameter(ValidationContextConstants.MODEL);
		IConfiguration config = (IConfiguration) context.getParameter(ValidationContextConstants.CONFIGURATION);
		config.setSymbolPool(ValidationConstants.VALIDATION_ID);
		boolean validatingHidden = config.getBooleanParameter(ValidationConstants.VALIDATE_HIDDEN_KEY);
		boolean isHiddenComponent = model instanceof IRenderableModel && !((IRenderableModel) model).isVisible(target);
		if (validatingHidden || !isHiddenComponent) {
			String valName = execPoint.getValidatorClassName();
			IRuleSet[] ruleSets = ruleBase
					.getRuleSetsForExecutionPoint(execPoint.getName());
			IValidator validator = null;
			if (ruleSets.length > 0 && validatorMap.size() > 0) {
				if (validatorMap.containsKey(valName)) {
					validator = (IValidator) validatorMap.get(valName);
				}
			}
			// for each IRuleSet that is enabled
			// get the IValidator name from the ExecutionPoint and create it
			// invoke it with the target object from the ExecutionPoint
			for (int i = 0; i < ruleSets.length; i++) {
				if (ruleSets[i].isEnabled()) {
					if (validator == null) {
						try {
							validator = (IValidator) ClassLoaderCache.getDefault()
								.classForName(valName).getConstructor(
									new Class[] { IRuleSet.class}).newInstance(new Object[] { ruleSets[i]});
							// add it to the validator map so we don't have to
							// keep creating it
							validatorMap.put(valName, validator);
						} catch (Exception e) {
							throw new ValidationProcessException("Unable to create IValidator named:  " + valName);
						}
					} else {
						validator.setRuleSet(ruleSets[i]);
					}
					validator.validate(context, target);
					incrementProgress();
				}
			}
			// If the parent is visible then validate it's children, if not then
			// skip them.
			// Since we're validating, set the validate flag to true.
			validateChildren(target, execPoint, ruleBase, context, true);
		} else if (isHiddenComponent) {
			LoggingUtil.println(LoggingUtil.PROCESS_INFO,
					"Skipping hidden component of type "
							+ target.getClass().getName());
		}
	}

	/**
	 * Validate the child components of the given target. The TreeWalkerFactory
	 * is used to get the appropriate tree walker, which fetches the children of
	 * this component.
	 * 
	 * @param target --
	 *            the target object whose children are to be validated
	 * @param execPoint --
	 *            the ExecutionPoint that triggered this validation
	 * @param ruleBase --
	 *            the IRuleBase that matches this target
	 * @param context --
	 *            the IValidationContext to use when evaluating rules
	 * @param validate --
	 *            validate (true) or visit only
	 */
	protected void validateChildren(Object target, ExecutionPoint execPoint,
									IRuleBase ruleBase, IValidationContext context, boolean validate)
			throws ValidationProcessException {
		IModel model = (IModel) context.getParameter(ValidationContextConstants.MODEL);
		IConfiguration config = (IConfiguration) context.getParameter(ValidationContextConstants.CONFIGURATION);
		config.setSymbolPool(IConfiguration.ACTF_ID);

		if (model == null) {
			throw new ValidationProcessException(
					"IValidationContext does not have a valid IModel.");
		}

		// TODO needs to be more generic
		final ITreeNodeWalker walker = (ITreeNodeWalker) model.getNodeWalker();
		final Object[][] childrenRes = new Object[][] { new Object[0] };
		final Object targetObj = target;
		String modelType = model.getName();

		if (walker != null) {
			walker.setComponentBridgeMap((Map) context.getParameter(ValidationContextConstants.ELEMENT_BRIDGE_MAP));
			Runnable runnable = new Runnable() {
				public void run() {
					try {
						childrenRes[0] = walker.getFilteredChildren(targetObj);
					} catch (InvalidComponentException e) {
						LoggingUtil.println(IReporter.SYSTEM_FATAL, "Unable to retrieve children for component", e);
					}
				}
			};

			if (model instanceof IRenderableModel) {
				((IRenderableModel) model).invokeOnUIThread(runnable);
			} else {
				runnable.run();
			}
		} else {
			LoggingUtil.println(LoggingUtil.WARNINGS,
					"No tree walker available for component of type " + target.getClass().getName());
		}

		Object[] children = childrenRes[0];
		// The algorithm for validating the children is written below. The only difficult
		// part here is the transition to another model. For instance, Swing components
		// can be embedded in an SWT gui. In such a case, the validation process needs to transition
		// to the new model (and new IRuleBase) for that target. The ExecutionPointDescriptors
		// define the transition target regular expressions and the IRuleBase to
		// use.
		// 
		// Transition algorithm:
		//
		// for each child
		// if child is mapped to same IRuleBase as the ExecutionPoint target
		// validateComponent( child, execPoint, ruleBase, context );
		// else
		// if
		// RuleBaseManager.getExcutionPointDescriptor(name).getModelTransition() != null
		// if target object matches a transition regular expression
		// create an instance of that ExecutionPoint with component as target object
		// get the IRuleBase from RuleBaseManager
		// ??? initialize a IValidationContext ??? or ??? use the same one ???
		// validateComponent( child, newExecPoint, newRuleBase, newContext )
		// else
		// log message that there is no matching model transition
		// else
		// log message that there is no model transition
		// endfor

		RuleBaseManager rbMgr = RuleBaseManager.getInstance();
		for (int i = 0; !isTerminated() && i < children.length; ++i) {
			String childName = children[i].getClass().getName();
			IRuleBase[] ruleBases = RuleBaseManager.getInstance()
					.getRuleBasesByTarget(children[i]);
			boolean foundRuleBaseMatch = false;
			for (int j = 0; j < ruleBases.length; j++) {
				if (ruleBase.getName().equalsIgnoreCase(ruleBases[j].getName())) {
					// just visiting or validating? only the validate flag knows
					// for sure
					if (validate) {
						validateComponent(children[i], execPoint, ruleBase, context);
					} else {
						visitComponent(children[i], execPoint, ruleBase, context);
					}
					foundRuleBaseMatch = true;
					break;
				}
			}
			if (!foundRuleBaseMatch) {
				// get the execution point descriptor for this execution point.
				// The execution point descriptor has the definitions for which
				// models
				// can be transitioned to based on whether the target object
				// class name matches
				// the regular expression.
				ExecutionPointDescriptor execPointDesc = ruleBase
						.getExecutionPointDescriptor(execPoint.getName());
				if (execPointDesc == null) {
					throw new ValidationProcessException(
							"Unable to find execution point descriptor for execution point named: " + execPoint.getName());
				}

				boolean foundModelTransMatch = false;
				// get the model transitions.
				ModelTransition[] modelTrans = execPointDesc.getModelTransitions();
				// loop through each transition and test whether the regular expression matches the
				// the child class name.
				if (modelTrans != null && modelTrans.length > 0) {
					for (int k = 0; k < modelTrans.length
							&& !foundModelTransMatch; k++) {
						if (pathMatchesExpression(children[i], modelTrans[k].getRegexp())) {
							foundModelTransMatch = true;
							// get the IRuleBase from RuleBaseManager
							IRuleBase newRB = rbMgr.getRuleBase(modelTrans[k].getRuleBaseName());
							if (newRB == null) {
								throw new ValidationProcessException(
										"RuleBaseManager cannot find IRuleBase named: " + modelTrans[k].getRuleBaseName());
							}
							// get the execution point descriptor from the new rule base using
							// the name from the model transition of the
							// original execution point.
							ExecutionPointDescriptor newExecPointDesc = newRB
									.getExecutionPointDescriptor(modelTrans[k].getExecutionPointName());
							if (newExecPointDesc == null) {
								throw new ValidationProcessException(
										"Unable to find execution point descriptor for execution point named: " + modelTrans[k].getExecutionPointName());
							}
							// Create an instance of that ExecutionPoint with
							// the name and IValidator class name
							// from the ExecutionPointDescriptor. All of the
							// other fields are the same and can
							// be taken from the original execution point.
							ExecutionPoint newExecPoint = execPoint
									.cloneExecutionPoint(newExecPointDesc
											.getName(), newExecPointDesc
											.getValidatorName(), execPoint);
							IValidationContext newContext = cloneContext(newRB, context);
							// just visiting or validating? only the validate
							// flag knows for sure
							if (validate) {
								validateComponent(children[i], newExecPoint, newRB, newContext);
							} else {
								visitComponent(children[i], newExecPoint, newRB, newContext);
							}
						}
					}
					if (!foundModelTransMatch) {
						LoggingUtil
								.println(LoggingUtil.WARNINGS,
										"No matching model transitions for child target [ " + childName
												+ " ] defined for execution point descriptor named: " + execPointDesc.getName());
					}
				} else {
					LoggingUtil.println(LoggingUtil.WARNINGS,
							"No model transitions defined for execution point descriptor named: " + execPointDesc.getName());
				}
			} // if( !foundRuleBaseMatch )
		} // for (int i = 0; !isTerminated() && i < children.length; ++i)
	} // validateChildren

	/**
	 * 
	 * @param iRuleBase
	 * @param context
	 * @return a clone of the context
	 * @throws ValidationProcessException
	 */
	protected IValidationContext cloneContext(IRuleBase iRuleBase,
			IValidationContext context) throws ValidationProcessException {
		ValidationDocumentRuleBase valRB = null;
		if (iRuleBase != null && iRuleBase instanceof ValidationDocumentRuleBase) {
			valRB = (ValidationDocumentRuleBase) iRuleBase;
		} else {
			LoggingUtil.println(IReporter.SYSTEM_NONFATAL,
					"Incompatible IRuleBase type in AccessibilityValidationProcess.copyContext(). "
							+ "IRuleBase name: " + iRuleBase.getName());
			throw new ValidationProcessException(
					"Unknown IRuleBase type in AccessibilityValidationProcess.copyContext(). ");
		}
		// create a new IValidationContext from the old one
		AbstractValidationContext baseCtx = (AbstractValidationContext) context;
		IValidationContext newCtx = (IValidationContext) baseCtx.clone();
		newCtx.setParameter(ValidationContextConstants.MODEL, valRB
				.getModel());
		newCtx.setParameter(ValidationContextConstants.CODE_PROCESSOR, valRB
				.getCodeProcessor());
		newCtx.setParameter(ValidationContextConstants.VALIDATED_ELEMENTS,
				new HashSet());
		return newCtx;
	}

	/**
	 * used by child classes within the model to determine if their path
	 * matches a reqular expression that identifies a rule base. This is
	 * typically needed when a child component has a
	 * 
	 * @param target -
	 *            target object we are trying to match
	 * @param regexp -
	 *            the regular expression that identifies a rule base
	 * @return validator for this component
	 * 
	 */
	protected boolean pathMatchesExpression(Object target, String regexp) {
		String name = null;
		boolean matchFound = false;
		Pattern pattern = Pattern.compile(regexp);
		if (target != null) {
			Class cls = target.getClass();
			do {
				name = cls.getName();
				if (pattern.matcher(name).matches()) {
					matchFound = true;
					break;
				}
			} while ((cls = cls.getSuperclass()) != null);
		}
		return matchFound;
	}

	/**
	 * visit the objects in the tree with the given target as the root. This
	 * method is used primarily to walk the component tree starting with the
	 * target and estimate the amount of work (or number of components) when the
	 * tree is walked.
	 * 
	 * @param execPoint --
	 *            the ExecutionPoint that triggered this validation
	 * @param iRuleBase --
	 *            the IRuleBase that matches this target
	 * @param context --
	 *            the IValidationContext to use when evaluating rules
	 */
	protected void calculateWork(ExecutionPoint execPoint, IRuleBase iRuleBase,
			IValidationContext context) throws ValidationProcessException {
		if (isTerminated()) {
			return;
		}
		Object target = execPoint.getTarget();
		if (target != null) {
			try {
				IModel model = (IModel) context.getParameter(ValidationContextConstants.MODEL);
				if (model instanceof IRenderableModel) {
					((IRenderableModel) model).requestFocusFor(target);
				}
				workEstimate = 0;
				visitComponent(target, execPoint, iRuleBase, context);
			} catch (Exception e) {
				LoggingUtil.println(IReporter.SYSTEM_FATAL, e);
			}
		} else {
			throw new ValidationProcessException("No target object to validate.");
		}
	}

	/**
	 * used to visit a single component.
	 * 
	 * @param target --
	 *            the target object to validate
	 * @param execPoint --
	 *            the ExecutionPoint that triggered this validation
	 * @param iRuleBase --
	 *            the IRuleBase that matches this target
	 * @param context --
	 *            the IValidationContext to use when evaluating rules
	 */
	protected void visitComponent(Object target, ExecutionPoint execPoint,
			IRuleBase iRuleBase, IValidationContext context)
			throws ValidationProcessException {
		IModel model = (IModel) context.getParameter(ValidationContextConstants.MODEL);
		IConfiguration config = (IConfiguration) context.getParameter(ValidationContextConstants.CONFIGURATION);
		config.setSymbolPool(ValidationConstants.VALIDATION_ID);
		
		boolean validatingHidden = config.getBooleanParameter(ValidationConstants.VALIDATE_HIDDEN_KEY);
		boolean isHiddenComponent = model instanceof IRenderableModel && !((IRenderableModel) model).isVisible(target);
		if (validatingHidden || !isHiddenComponent) {
			workEstimate++;
			// If the parent is visible then validate it's children, if not then
			// skip them.
			// Since we're visiting only, set the validate flag to false.
			validateChildren(target, execPoint, iRuleBase, context, false);
		} else if (isHiddenComponent) {
			LoggingUtil.println(LoggingUtil.PROCESS_INFO,
					"Skipping hidden component of type "
							+ target.getClass().getName());
		}
	}
}
