/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  IBM Corporation - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.core;

/**
 * base implementation for an execution point, a point of execution of the validated application at which a 
 * validation is to be triggered. Clients will typically not create instances of this class but, 
 * rather, leave instantiation to generated classes that handle or intercept invocations or events 
 * that are to trigger validations or to the < code>ValidationProcessManager</code>. Execution points might include 
 * method invokations, a GUI component becomming focused, a top-level window becomming visible or being created, a property change, etc.
 * 
 * <p><b>Note</b>: The API for this class and its subclasses are not stable. 
 * 
 * @author Mike Squillace
 *
 */
public abstract class ExecutionPoint
{

	protected Object target;

	protected String validatorClassName;

	protected String name;

	/**
	 * create an execution point for triggering validation processes. Clients do not typically 
	 * instantiate this class but, rather, leave its instantiation to methods or event-handlers that 
	 * have been set to trigger validations in the rule base or to the <code>ValidationProcessManager</code> based upon an 
	 * <code>ExecutionPointDescriptor</code>.
	 * 
	 * @param name - name of this execution point
	 * @param validatorClassName - validator class that registered this execution point
	 * @param target - receiver at the point of execution (i.e. the object that
	 * initiated the invocation or event)
	 */
	public ExecutionPoint (String name, String validatorClassName, Object target) {
		this.name = name;
		this.validatorClassName = validatorClassName;
		this.target = target;
	}

	/**
	 * 
	 * @return target
	 */
	public Object getTarget () {
		return target;
	}

	/**
	 * 
	 * @return validator class name that caused this point to be a trigger
	 */
	public String getValidatorClassName () {
		return validatorClassName;
	}

	/**
	 * 
	 * @return name of this execution point
	 */
	public String getName () {
		return name;
	}

	/**
	 * return an ExecutionPoint of the same type but with a different name and
	 * validator class name. This is typically used in the Validation loop where
	 * we have a child class of a component that has a different model than
	 * the parent.
	 * @param name
	 * @param validator
	 * @param execPoint
	 * @return a clone of this execution point
	 */
	public abstract ExecutionPoint cloneExecutionPoint (String name,
														String validator,
														ExecutionPoint execPoint);
}
