/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  IBM Corporation - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.core;

import java.util.List;
import java.util.ListIterator;

import org.eclipse.actf.util.logging.LoggingUtil;

/**
 * embodies a description of the execution points at which validations are to be triggered.
 * Currently, three types of execution point descriptors are supported:
 * 
 * <p><ul>
 * <li>method - descriptors that describe execution points that are constructor or method 
 * invocations 
 * <li>event - descriptors that describe execution points that occur when an event is fired
 * <li>property - descriptors that describe execution points that specify that a property 
 * has changed
 * </ul>
 * 
 * <p>In ACTF, the first type of execution point descriptor is only meaningful in the context 
 * of an aspect-oriented programming environment in which validation code can be woven into 
 * the target class at the appropriate point of invocation. The other types can be implemented 
 * via listeners and proxies handled via the Java Reflection API.
 *
 * <p><b>Note</b>: The API and functionality for this class are not complete.
 * @author Mike Squillace
 *
 */
public class ExecutionPointDescriptor
{

	public static final int METHOD_EXECUTION_POINT = 0;
	public static final int EVENT_EXECUTION_POINT = METHOD_EXECUTION_POINT + 1;
	public static final int PROPERTYCHANGED_EXECUTION_POINT = EVENT_EXECUTION_POINT + 1;
	
	public static final String EXECUTION_POINT_NAME = "name";
	public static final String EXECUTION_POINT_VALIDATOR = "validator";
	
	public static final String METHOD_TYPE = "method";
	public static final String EVENT_TYPE = "event";
	
	protected String name;
	protected String ruleBaseName;
	protected String validatorName;

	protected List executionTargets;
	protected List modelTransitions;

	/**
	 * create a descriptor for a set of execution points. Each execution point has a name
	 * that is unique within a rule base definition. The <b>qualified name</b> is in the
	 * form <code>rule_base_name:execution_point_name</code>.
	 * 
	 * <p>Each execution point has an associated validator class that is to be used when 
	 * validating components. A rule base can have more than one execution point descriptor
	 * where each descriptor is differentiated by its targets and validators. 
	 * 
	 * <p>The list of execution point targets defines the condition or criteria for firing 
	 * the execution point. Execution point targets define a specific target class name and
	 * a type of either method-based,<code>raven:method</code>, or event-based,
	 * <code>raven:event</code>.
	 *  
	 * @see org.eclipse.actf.validation.core.ExecutionPointTarget
	 * @see org.eclipse.actf.validation.core.EventExecutionPointTarget
	 * @see org.eclipse.actf.validation.core.MethodExecutionPointTarget
	 * 
	 * <p>IModel transitions are used to handle cross-model validations. Java UI code
	 * libraries can support embedded GUI implementations. For instance, SWT GUIs can embed
	 * Swing GUIs. Since rule bases are specific to GUI Models and execution points are
	 * unique to a rule base, the model transition <code>raven:architectureTransition</code>
	 * provides a way for a validation in one GUI model to transition to different GUI
	 * models. The model transition defines a regular expression to match and a
	 * new execution point and rule base to use when validating the new GUI model.
	 * 
	 * @see org.eclipse.actf.validation.core.ModelTransition
	 * 
	 * @param name - name of this ExecutionPointDescriptor
	 * @param ruleBaseName - name of the IRuleBase for the contained ExecutionPoint
	 * @param validatorName - the IValidator class name to be used when generating ExecutionPoints
	 * @param executionTargets - the execution point targets for this ExecutionPointDescriptor
	 * @param modelTransitions - the architecture transitions for this ExecutionPointDescriptor
	 * 
	 */
	public ExecutionPointDescriptor (String name, String ruleBaseName, String validatorName,
									 List executionTargets, List modelTransitions) {
		setName(name);
		setRuleBaseName(ruleBaseName);
		setValidatorName(validatorName);
		setExecutionTargets(executionTargets);
		setModelTransitions(modelTransitions);
	}

	public ExecutionPointDescriptor (String name, String validatorName,
			 List executionTargets, List modelTransitions) {
		this(name, null, validatorName, executionTargets, modelTransitions);
}

	/**
	 * @param name - name of this ExecutionPointDescriptor
	 * @param ruleBaseName - name of the IRuleBase for the contained ExecutionPoint
	 * @param validatorName - the IValidator class name to be used when generating ExecutionPoints
	 * @param executionTargets - the execution point targets for this ExecutionPointDescriptor
	 * 
	 */
	public ExecutionPointDescriptor (String name, String ruleBaseName,
									 String validatorName, List executionTargets) {
		this(name, ruleBaseName, validatorName, executionTargets, null);
	}

	/**
	 * @param name - name of this ExecutionPointDescriptor
	 * @param ruleBaseName - name of the IRuleBase for the contained ExecutionPoint 
	 */
	public ExecutionPointDescriptor (String name, String ruleBaseName) {
		this(name, ruleBaseName, null, null, null);
	}

	/**
	 * 
	 * @return name of this descriptor
	 */
	public String getName () {
		return name;
	}

	/**
	 * 
	 * @param name - name of this descriptor
	 */
	public void setName (String name) {
		this.name = name;
	}

	/**
	 * 
	 * @return rule base name associated with this descriptor
	 */
	public String getRuleBaseName () {
		return ruleBaseName;
	}

	/**
	 * 
	 * @param rbName - name of this descriptor
	 */
	public void setRuleBaseName (String rbName) {
		ruleBaseName = rbName;
	}

	/**
	 * @return the ExecutionPointTargets
	 */
	public ExecutionPointTarget[] getExecutionTargets () {
		return (ExecutionPointTarget[]) executionTargets.toArray(new ExecutionPointTarget[executionTargets.size()]);
	}

	/**
	 * @param targets the executionPointTargets to set
	 */
	public void setExecutionTargets (List targets) {
		executionTargets = targets;
	}

	/**
	 * @return the validator name
	 */
	public String getValidatorName () {
		return validatorName;
	}

	/**
	 * @param name the validator name to set
	 */
	public void setValidatorName (String name) {
		validatorName = name;
	}

	/**
	 * @return the architecture transitions
	 */
	public ModelTransition[] getModelTransitions () {
		return (ModelTransition[]) modelTransitions.toArray(new ModelTransition[modelTransitions.size()]);
	}

	/**
	 * @param transitions -- the model transitions to set
	 */
	public void setModelTransitions (List transitions) {
		modelTransitions = transitions;
	}

	/**
	 * Execution point names can be reused across rule bases but they must be unique 
	 * within a rule base. The qualified name is the execution point name scoped to 
	 * the rule base name. It has the form <code>rule_base_name:execution_point_name</code>.
	 * 
	 * @return the qualified name of this execution point. 
	 * 
	 */
	public String getQualifiedName () {
		return (name != null && ruleBaseName != null) ? ruleBaseName + ":"
				+ name : "";
	}

	public void printDescriptor (int indent) {
		String singleTab = "\t";
		String tab = "";
		for (int i = 0; i < indent; i++) {
			tab += singleTab;
		}
		LoggingUtil.println(LoggingUtil.PROCESS_ALL, tab
				+ "Details for ExecutionPointDescriptor - " + getName());
		LoggingUtil.println(LoggingUtil.PROCESS_ALL, tab + "  toString() - "
				+ this.toString());
		LoggingUtil.println(LoggingUtil.PROCESS_ALL, tab + "  IRuleBase name - "
				+ getRuleBaseName());
		LoggingUtil.println(LoggingUtil.PROCESS_ALL, tab + "  Qualified name - "
				+ getQualifiedName());
		LoggingUtil.println(LoggingUtil.PROCESS_ALL, tab
				+ "  IValidator class name - " + getValidatorName());
		if (executionTargets.size() > 0) {
			LoggingUtil.println(LoggingUtil.PROCESS_ALL, tab
					+ "  ExecutionPointTargets - ");
			ListIterator iter = executionTargets.listIterator();
			while (iter.hasNext()) {
				ExecutionPointTarget ept = (ExecutionPointTarget) iter.next();
				if (ept.getType() == ExecutionPointDescriptor.METHOD_EXECUTION_POINT) {
					((MethodExecutionPointTarget) ept).printTarget(indent + 1);
				}else if (ept.getType() == ExecutionPointDescriptor.EVENT_EXECUTION_POINT) {
					((EventExecutionPointTarget) ept).printTarget(indent + 1);
				}
			}
		}else {
			LoggingUtil.println(LoggingUtil.PROCESS_ALL, tab
					+ "  No ExecutionPointTargets for this descriptor.");
		}
		if (modelTransitions.size() > 0) {
			LoggingUtil.println(LoggingUtil.PROCESS_ALL, tab
					+ "  ModelTransitions - ");
			ListIterator iter = modelTransitions.listIterator();
			while (iter.hasNext()) {
				ModelTransition modelTrans = (ModelTransition) iter.next();
				LoggingUtil.println(LoggingUtil.PROCESS_ALL, tab
						+ "Details for ModelTransition");
				LoggingUtil.println(LoggingUtil.PROCESS_ALL, tab + "  regexp - "
						+ modelTrans.getRegexp());
				LoggingUtil.println(LoggingUtil.PROCESS_ALL, tab
						+ "  IRuleBase name - " + modelTrans.getRuleBaseName());
				LoggingUtil.println(LoggingUtil.PROCESS_ALL, tab
						+ "  ExecutionPoint name - "
						+ modelTrans.getExecutionPointName());
			}
		}else {
			LoggingUtil.println(LoggingUtil.PROCESS_ALL, tab
					+ "  No ModelTransitions for this descriptor.");
		}
	}
}
