/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.core;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.net.URLStreamHandlerFactory;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * a configurable class loader. In particular, this loader can be configured
 * dynamically so as to use different classpaths during the execution of a Java
 * application.
 * 
 * @author Barry Feigenbaum
 */
public class ExternalClassLoader extends URLClassLoader {

	/**
	 * @param urls
	 */
	public ExternalClassLoader(URL[] urls) {
		super(urls);
	}

	/**
	 * @param urls
	 * @param parent
	 */
	public ExternalClassLoader(URL[] urls, ClassLoader parent) {
		super(urls, parent);
	}

	/**
	 * @param urls
	 * @param parent
	 * @param factory
	 */
	public ExternalClassLoader(URL[] urls, ClassLoader parent,
			URLStreamHandlerFactory factory) {
		super(urls, parent, factory);
	}

	/**
	 * @param paths
	 * @throws MalformedURLException
	 */
	public ExternalClassLoader(String paths) throws MalformedURLException {
		super(pathsToUrls(paths));
	}

	/**
	 * @param paths
	 * @param parent
	 * @throws MalformedURLException
	 */
	public ExternalClassLoader(String paths, ClassLoader parent)
			throws MalformedURLException {
		super(pathsToUrls(paths), parent);
	}

	/**
	 * @param paths
	 * @param parent
	 * @param factory
	 * @throws MalformedURLException
	 */
	public ExternalClassLoader(String paths, ClassLoader parent,
			URLStreamHandlerFactory factory) throws MalformedURLException {
		super(pathsToUrls(paths), parent, factory);
	}

	/**
	 * @param paths
	 * @throws MalformedURLException
	 */
	public ExternalClassLoader(String[] paths) throws MalformedURLException {
		super(pathsToUrls(paths));
	}

	/**
	 * @param paths
	 * @param parent
	 * @throws MalformedURLException
	 */
	public ExternalClassLoader(String[] paths, ClassLoader parent)
			throws MalformedURLException {
		super(pathsToUrls(paths), parent);
	}

	/**
	 * @param paths
	 * @param parent
	 * @param factory
	 * @throws MalformedURLException
	 */
	public ExternalClassLoader(String[] paths, ClassLoader parent,
			URLStreamHandlerFactory factory) throws MalformedURLException {
		super(pathsToUrls(paths), parent, factory);
	}

	public void addURL(URL url) {
		super.addURL(url);
	}

	public void addPath(String path) throws MalformedURLException {
		addURL(pathToUrl(path));
	}

	public void addPaths(String[] paths) throws MalformedURLException {
		for (int i = 0; i < paths.length; i++) {
			addURL(pathToUrl(paths[i]));
		}
	}

	public String[] getPaths() {
		List l = new ArrayList();
		URL[] urls = getURLs();
		for (int i = 0; i < urls.length; i++) {
			URL url = urls[i];
			if (url.getProtocol().equals("file")) {
				l.add(url.getPath());
			}
		}
		return (String[]) l.toArray(saType);
	}

	public List getPathsAsList() {
		return Arrays.asList(getPaths());
	}

	public List getURLsAsList() {
		return Arrays.asList(getURLs());
	}

	protected static final String[] saType = new String[0];

	protected static URL[] pathsToUrls(String paths)
			throws MalformedURLException {
		String[] xpaths = paths.split(File.pathSeparator);
		return pathsToUrls(xpaths);
	}

	protected static URL[] pathsToUrls(String[] paths)
			throws MalformedURLException {
		URL[] urls = new URL[paths.length];
		for (int i = 0; i < urls.length; i++) {
			urls[i] = pathToUrl(paths[i]);
		}
		return urls;
	}

	protected static URL pathToUrl(String path) throws MalformedURLException {
		int pos = path.indexOf("://"); // has protocol
		URL url = pos < 0 ? new File(path).toURL() : new URL(path);
		// if (pos < 0) {
		// path = "file://" + path;
		// }
		return url;
	}

	// *** following for debugging, remove ***
	protected Class findClass(String name) throws ClassNotFoundException {
		Class c = null;
		try {
			// System.out.println("ExternalClassLoader.findClass : " + name + "
			// paths: " + getURLsAsList());
			c = super.findClass(name);
		} finally {
			// System.out.println("ExternalClassLoader.findClass: " + name + "="
			// + c);
		}
		return c;
	}

	protected Class loadClass(String name, boolean resolve)
			throws ClassNotFoundException {
		Class c = null;
		try {
			// System.out.println("ExternalClassLoader.loadClass : " + name +
			// "," + resolve);
			c = super.loadClass(name, resolve);
		} finally {
			// System.out.println("ExternalClassLoader.loadClass : " + name +
			// "," + resolve + "=" + c);
		}
		return c;
	}
}
