/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  IBM Corporation - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.core;

import org.eclipse.actf.validation.rulebase.IRuleBase;
import org.eclipse.actf.validation.validators.IValidationContext;


/**
 * represents a single validation process triggered by an execution point. A
 * validation process controls the initiation of a validation of content or a
 * running application and how validation proceeds as the content or application
 * is used. The <code>ValidationProcessManager</code> is responsible for
 * launching all processes as execution points are encountered.
 * 
 * @see ExecutionPoint
 * @see ValidationProcessManager
 * @author Mike Squillace
 * 
 */
public interface IValidationProcess {

	/**
	 * perform any initialization necessary for the process. For example, some
	 * implementations of this interface try to estimate the work in this
	 * method.
	 * 
	 * @throws ValidationProcessException
	 */
	public void init() throws ValidationProcessException;

	/**
	 * launch the validation process
	 * 
	 * @throws ValidationProcessException
	 */
	public void run() throws ValidationProcessException;

	/**
	 * returns an estimate on the amount of work required by this process.
	 * Returning -1 means that no work estimate is available or that this
	 * process does not plan to support any sort of progress indication.
	 * 
	 * @return estimate of work required by this process or-1 if not supported
	 *         or not available
	 */
	public int getWorkEstimate();

	/**
	 * returns the approximate progress relative to the amount of estimated work
	 * as an integer percentage. Returning a 0 means that a work estimate is
	 * available but there is no progress as of yet. Returning -1 means that no
	 * work estimate is available or that this process does not plan to support
	 * any sort of progress indication.
	 * 
	 * @return the current progress as a percentage of the estimated work, 0 if
	 *         no progress has been make or-1 if not supported or not available
	 */
	public int getProgress();

	/**
	 * returns the IRuleBase used in this IValidationProcess.
	 * 
	 * @return the IRuleBase for this IValidationProcess
	 */
	public IRuleBase getRuleBase();

	/**
	 * returns the ExecutionPoint triggering this IValidationProcess.
	 * 
	 * @return the ExecutionPoint for this IValidationProcess
	 */
	public ExecutionPoint getExecutionPoint();

	/**
	 * returns the IValidationContext used in this IValidationProcess.
	 * 
	 * @return the IValidationContext for this IValidationProcess
	 */
	public IValidationContext getValidationContext();

	/**
	 * sets the IRuleBase used in this IValidationProcess.
	 * 
	 * @param rulebase --
	 *            the IRuleBase for this IValidationProcess
	 */
	public void setRuleBase(IRuleBase rulebase);

	/**
	 * sets the ExecutionPoint used in this IValidationProcess.
	 * 
	 * @param execPoint --
	 *            the ExecutionPoint for this IValidationProcess
	 */
	public void setExecutionPoint(ExecutionPoint execPoint);

	/**
	 * sets the IValidationContext used in this IValidationProcess.
	 * 
	 * @param context --
	 *            the IValidationContext for this IValidationProcess
	 */
	public void setValidationContext(IValidationContext context);
}
