/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  IBM Corporation - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.core;

import java.util.Map;

import org.eclipse.actf.core.config.ConfigurationException;
import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.core.runtime.IRuntimeContext;
import org.eclipse.actf.core.runtime.RuntimeContextFactory;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.logging.LoggingUtil;
import org.eclipse.actf.validation.valdoc.codeproc.EvaluationException;
import org.eclipse.actf.validation.valdoc.codeproc.ICodeProcessor;

/**
 * A Parameter object represents any attribute value or argument that is to be
 * passed to a constructor or method as it appears in the XML document. Such
 * objects include (but are certainly not limited to):
 * 
 * <p>
 * <ul>
 * <li>reference ids
 * <li>method aliases
 * <li>code to be evaluated
 * <li>literals or String literals to be treated as primitive literals
 * <li> String literals that dereference an alias
 * </ul>
 * 
 * <p>
 * Thus, this class embodies most of the special syntax used by the RGB engine
 * in reading the description of a GUI in an XML document.
 * 
 * @author Mike Squillace
 */
public class Parameter {

	/** unknown parameter type */
	public static final int UNKNOWN = -1;

	/** reference id */
	public static final int REFERENCE_ID = 0;

	/** method alias (not currently used) */
	public static final int METHOD_ALIAS = 1;

	/** script code */
	public static final int SCRIPT_CODE = 2;

	/** literal primitive */
	public static final int LITERAL_PRIMITIVE = 3;

	/** unresolved parameter */
	public static final String UNRESOLVED = "___UNRESOLVED___";

	/** empty parameter */
	public static final String EMPTY_VALUE = "__EMPTY__";

	private String unresolvedValue;
	private Object resolvedValue;
	private int parameterType;
	private ICodeProcessor codeProcessor;
	private String fieldName;

	/**
	 * create a Parameter object for later resolution. The given String argument
	 * is the parameter as it is read from the XML source. The type of this
	 * parameter object is determined by its literal form or by symbols prefixed
	 * to it. If a literal is given, a best-guess is made about the type of
	 * object to which ultimate resolution will lead.
	 * 
	 * @param param --
	 *            string representation of parameter as read from XML document
	 * @param codeProcessor --
	 *            ICodeProcessor for evaluation of expressions and ids
	 */
	public Parameter(String param, ICodeProcessor codeProcessor) {
		if (isScriptCode(param)) {
			parameterType = SCRIPT_CODE;
			unresolvedValue = param.substring(1, param.length() - 1);
		} else if (isMethodAlias(param)) {
			parameterType = METHOD_ALIAS;
			unresolvedValue = param.substring(1);
		} else if (isReferenceId(param)) {
			parameterType = REFERENCE_ID;
			unresolvedValue = param.substring(1);
		} else {
			parameterType = LITERAL_PRIMITIVE;
			if (param.startsWith("%")) {
				IConfiguration config = null;
				try {
					IRuntimeContext context = RuntimeContextFactory.getInstance().getRuntimeContext();
					config = context.getConfiguration();
				} catch (ConfigurationException e) {
					throw new RuntimeException("Could not form configuration object for rulebase construction", e);
				}
				String model = codeProcessor.getModel().getName();
				Map aliasMap = config.getSymbolPoolContents(model
						+ IConfiguration.MODEL_POOL_ID_DELIMITER
						+ IConfiguration.ALIASES_ID);
				unresolvedValue = (String) aliasMap.get(param.substring(1));
			} else {
				unresolvedValue = param;
			}
		}
		this.codeProcessor = codeProcessor;
	} // ctor

	/**
	 * create a Parameter object for later resolution. Supplying a type to this
	 * ctor overrides any default determination based upon syntax of the literal
	 * supplied
	 * 
	 * @param param --
	 *            representation of the parameter to be resolved
	 * @param type --
	 *            type of this parameter; one of the public static fields of
	 *            this class
	 * @param codeProcessor --
	 *            code processor for evaluation, which may be <code>null</code>
	 *            if the type is not SCRIPT_CODE
	 * @see #Parameter(String, ICodeProcessor)
	 */
	public Parameter(String param, int type, ICodeProcessor codeProcessor) {
		this.codeProcessor = codeProcessor;
		unresolvedValue = param;
		setParameterType(type);
	}

	/**
	 * wraps the given Object as a resolved Parameter
	 * 
	 * @param obj --
	 *            an Object that will be passed to a method or constructor
	 */
	public Parameter(Object obj) {
		resolvedValue = obj;
	}

	/**
	 * set the code processor for this parameter
	 * 
	 * @param cp --
	 *            code processor
	 */
	public void setCodeProcessor(ICodeProcessor cp) {
		codeProcessor = cp;
	}

	/**
	 * returns the code processor being used by this parameter.
	 * 
	 * @return code processor
	 */
	public ICodeProcessor getCodeProcessor() {
		return codeProcessor;
	}

	/**
	 * returns the resolved parameter value. The resolved value of a parameter
	 * is an object of the given type that best reflects the information
	 * provided to this Parameter object. This value may be <code>null</code>
	 * if one of the <code>resolve(...)</code> methods has not been invoked
	 * 
	 * @return resolved parameter value
	 * @see #resolve(Class)
	 */
	public Object getResolvedValue() {
		return resolvedValue;
	}

	/**
	 * get the unresolved value for this parameter. This will be the text of the
	 * origianal text content or attribute value of the element without special
	 * syntax of ACTF.
	 * 
	 * @return text of original text content or attribute of element
	 */
	public String getUnresolvedValue() {
		return unresolvedValue;
	}

	/**
	 * set the parameter type of this parameter. The valid parameter types are
	 * given as public static fields of this class. This method is used to
	 * override default setting based upon syntax of literal
	 * 
	 * @param type --
	 *            one of the public static fields of this class
	 */
	public void setParameterType(int type) {
		parameterType = (type < 0 || type > LITERAL_PRIMITIVE) ? UNKNOWN
				: type;
	}

	/**
	 * returns the parameter type of this unresolved parameter
	 * 
	 * @return parameter type (one of the public static fields of this class)
	 */
	public int getParameterType() {
		return parameterType;
	}

	/**
	 * attempts to resolve this unresolved Parameter. The algorithm for
	 * resolution is as follows:
	 * <p>
	 * <ul>
	 * <li>if value is in braces ({...}), attempt evaluation of expression
	 * using ICodeProcessor
	 * <li> if value begins with '@', look up reference id represented by this
	 * value in the symbol table
	 * <li> treat parameter as a fully-qualified class name and attempt to
	 * invoke default constructor
	 * <li> try treating parameter as a relative and then fully-qualified public
	 * field name
	 * <li> attempt to map value to a primitive
	 * </ul>
	 * 
	 * @param cls --
	 *            desired class of which resolution is to produce an instance
	 * @return instance of desired class or <code>null</code> if resolution
	 *         could not take place
	 */
	public Object resolve(Class cls) {
		return resolve(cls, null);
	}

	/**
	 * @see #resolve(Class)
	 * 
	 * @param cls -
	 *            desired class of which resolution is to produce an instance
	 * @param parent -
	 *            parent object (e.g. for resolving constraints or other layout
	 *            information)
	 * @return instance of desired class or <code>null</code> if resolution
	 *         could not take place
	 */
	public Object resolve(Class cls, Object parent) {
		if (!isResolved()) {
			if (isScriptCode()) {
				try {
					resolvedValue = codeProcessor.evaluateArguments(unresolvedValue, cls);
				} catch (EvaluationException e) {
					LoggingUtil.println(LoggingUtil.WARNINGS,
							"Could not evaluate argument " + unresolvedValue);
				}
			} else if (isReferenceId()) {
//				resolvedValue = codeProcessor.getSymbolTable().get(unresolvedValue);
			} else {
				// try treating parameter as name of class and invoke no-arg ctor
				try {
					resolvedValue = Class.forName(unresolvedValue).newInstance();
				} catch (Exception e) {
				}
				if (resolvedValue == null) {
					Reflector reflector = new Reflector();
					int lastDot = unresolvedValue.lastIndexOf('.');
					// treat param as the name of a relative and then
					// fully-qualified public field
					if (lastDot == -1) {
						try {
							resolvedValue = reflector.getFieldValue(
									unresolvedValue, parent);
							if (resolvedValue != null) {
								Class parentClass = parent.getClass();
								while (parentClass.getName().indexOf('$') != -1) {
									parentClass = parentClass.getSuperclass();
								}
								fieldName = parentClass.getName() + "."
										+ unresolvedValue;
							}
						} catch (Exception e) {
						}
					} else {
						try {
							Class c = Class.forName(unresolvedValue.substring(
									0, lastDot));
							resolvedValue = reflector.getFieldValue(
									unresolvedValue.substring(lastDot + 1), c);
							if (resolvedValue != null) {
								fieldName = c.getName() + "."
										+ unresolvedValue;
							}
						} catch (Exception e) {
						}
					}
				}
				if (resolvedValue == null) {
					try {
						resolvedValue = mapToPrimitive(cls);
					} catch (NumberFormatException e) {
					}
				}
			}
		}
		return resolvedValue;
	} // resolve

	/**
	 * force this Parameter to be treated as a series of statements in the
	 * scripting language of this Parameter's ICodeProcessor and return the array
	 * of objects that is the result of the evaluation of these statements.
	 * Statements are delimited via the expression delimiter, which is
	 * model-dependent or which can be set via the 'exprdelim' property
	 * in the raven.properties file.
	 * 
	 * @param paramTypes --
	 *            array of types of desired parameters
	 * @return array of results of resolution or <code>null</code> if code
	 *         could not be evaluated
	 * @see "resources/raven.properties"
	 */
	public Object[] resolve(Class[] paramTypes) {
		// TBD: handle set of uninterpreted params or literals
		Object[] objs = null;
		try {
			objs = codeProcessor.evaluateArguments(unresolvedValue,
					paramTypes);
		} catch (EvaluationException e) {
			LoggingUtil
					.println(
							IReporter.SYSTEM_NONFATAL,
							"Exception caught while attempting to resolve parameters for unresolved value - "
									+ unresolvedValue, e);
		}
		return objs;
	} // resolve

	/**
	 * returns <code>true</code> if this Parameter has been resolved via
	 * invoking one of the <code>resolve(...)</code> methods
	 * 
	 * @return <code>true</code> if this Parameter has been resolved,
	 *         <code>false</code> otherwise
	 * @see #resolve(Class)
	 */
	public boolean isResolved() {
		return unresolvedValue == null && resolvedValue != null;
	}

	/**
	 * returns <code>true</code> if this parameter is code to be evaluated by
	 * this Parameter's ICodeProcessor
	 * 
	 * @return <code>true</code> if parameter is script code,
	 *         <code>false</code> otherwise
	 */
	public boolean isScriptCode() {
		return getParameterType() == SCRIPT_CODE;
	}

	/**
	 * returns <code>true</code> if this parameter is a method alias
	 * 
	 * @return <code>true</code> if parameter is a method alias,
	 *         <code>false</code> otherwise
	 */
	public boolean isMethodAlias() {
		return getParameterType() == METHOD_ALIAS;
	}

	/**
	 * returns <code>true</code> if this parameter is a reference id
	 * 
	 * @return <code>true</code> if parameter is a reference id,
	 *         <code>false</code> otherwise
	 */
	public boolean isReferenceId() {
		return getParameterType() == REFERENCE_ID;
	}

	/**
	 * returns whether or not this parameter as an unresolved String names a
	 * public field in some class.
	 * 
	 * @return <code>true</code> if the String names a public field in some
	 *         class, <code>false</code> otherwise
	 */
	public boolean isPublicFieldName() {
		return fieldName != null;
	}

	/**
	 * gets the fully-qualified name of the field named by this parameter
	 * 
	 * @return name of field named by this parameter or <code>null</code> if
	 *         parameter does not name a public field
	 */
	public String getFieldName() {
		return fieldName;
	}

	/** {@inheritDoc} */
	public String toString() {
		StringBuffer sb = new StringBuffer();
		String[] types = { "Reference_ID", "Method_Alias", "Script_Code",
				"Literal_Primitive" };
		sb.append(types[getParameterType()]);
		sb.append(':');
		sb.append(unresolvedValue);
		sb.append("; resolved form:");
		sb.append(resolvedValue == null ? "<unresolved>" : resolvedValue);
		return sb.toString();
	} // toString

	private boolean isScriptCode(String param) {
		return param.startsWith("{");
	}

	private boolean isMethodAlias(String param) {
		return param.startsWith("!");
	}

	private boolean isReferenceId(String param) {
		return param.startsWith("@");
	}

	private Object mapToPrimitive(Class paramType) throws NumberFormatException {
		Object param = null;
		if (paramType.equals(String.class) || paramType.equals(Object.class)) {
			param = unresolvedValue;
		} else if (paramType == Character.TYPE
				&& unresolvedValue.length() == 1) {
			param = new Character(unresolvedValue.charAt(0));
		} else if (paramType == Integer.TYPE) {
			param = Integer.valueOf(unresolvedValue);
		} else if (paramType == Long.TYPE) {
			param = Long.valueOf(unresolvedValue);
		} else if (paramType == Double.TYPE) {
			param = Double.valueOf(unresolvedValue);
		} else if (paramType == Float.TYPE) {
			param = Float.valueOf(unresolvedValue);
		} else if (paramType == Boolean.TYPE) {
			param = Boolean.valueOf(unresolvedValue);
		}
		return param;
	} // mapToPrimative
} // Parameter
