/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  IBM Corporation - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.core;

import java.lang.reflect.Array;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.actf.util.logging.LoggingUtil;

/**
 * Because the IBM Reflexive GUI Builder appplication is based on the Java Reflection API, we use this class to
 * perform most of the introspection of classes required by the main RGB engine
 * to build and render GUIs.  Methods of the Reflector are used to obtain
 * appropriate constructors, methods, and fields of a given class or object.
 *
 * @author Mike Squillace
 */
public class Reflector
{

	private static final Map WRAPPER_TO_PRIMITIVE_MAP = new HashMap();
	static {
		WRAPPER_TO_PRIMITIVE_MAP.put(Integer.class, Integer.TYPE);
		WRAPPER_TO_PRIMITIVE_MAP.put(Short.class, Short.TYPE);
		WRAPPER_TO_PRIMITIVE_MAP.put(Long.class, Long.TYPE);
		WRAPPER_TO_PRIMITIVE_MAP.put(Float.class, Float.TYPE);
		WRAPPER_TO_PRIMITIVE_MAP.put(Double.class, Double.TYPE);
		WRAPPER_TO_PRIMITIVE_MAP.put(Character.class, Character.TYPE);
		WRAPPER_TO_PRIMITIVE_MAP.put(Boolean.class, Boolean.TYPE);
	}

	/**
	 * used primarily by the Reflector class to return the results
	 * of introspection
	 */
	public class Result
	{

	
		private Object member;

		private Object[] params;

		private String[] formattedParams;

		private Object value;

		/**
		 * create a new Reflector.Result
		 *
		 * @param member -- reflected member
		 * @param params -- parameters to be used upon invokation of member
		 */
		public Result (Object member, Object[] params) {
			this.member = member;
			this.params = params;
		}

		/**
		 * create a new Reflector.Result for a reflected Field member
		 *
		 * @param member -- reflected Field member
		 * @param value -- value of Field
		 */
		public Result (Object member, Object value) {
			this.member = member;
			this.value = value;
		}

		/**
		 * gets the reflected class member (e.g. Field, Constructor, or Method)
		 *
		 * @return reflected class member
		 */
		public Object getMember () {
			return member;
		}

		/**
		 * gets the parameters to be used for invokation of the
		 * reflected class member
		 *
		 * @return parameters to be used upon invokation
		 */
		public Object[] getParameters () {
			return params;
		}

		/**
		 * set the formatted parameters. Formatted parameters are those that will be used
		 * as actual parameters in source code generation. They will usually be generated
		 * with the <code>format</code> method of the code generator being used.
		 *
		 * @param params -- the formatted parameters ready for source code placement
		 */
		public void setFormattedParameters (String[] params) {
			formattedParams = params;
		}

		/**
		 * returns the formatted parameters.
		 *
		 * @return formatted parameters for source code generation
		 * @see #setFormattedParameters(String[])
		 */
		public String[] getFormattedParameters () {
			return formattedParams;
		}

		/**
		 * gets the value of the member should the member be a Field or returns the result of a method invokation or constructor call.
		 * This method will return <code>null</code> for the latter two cases until <code>invoke</code> is called.
		 *
		 * @return value of Field member
		 */
		public Object getValue () {
			return value;
		}

		/**
		 * returns <code>true</code> if member is not <code>null</code> and is an instance of
		 * <code>java.lang.reflect.Constructor</code>
		 *
		 * @return <code>true</code> if this Result holds a Constructor member, <code>false</code> otherwise
		 */
		public boolean isConstructor () {
			return member != null && member instanceof Constructor;
		}

		/**
		 * returns <code>true</code> if member is not <code>null</code> and is an instance of
		 * <code>java.lang.reflect.Method</code>
		 *
		 * @return <code>true</code> if this Result holds a Method member, <code>false</code> otherwise
		 */
		public boolean isMethod () {
			return member != null && member instanceof Method;
		}

		/**
		 * returns <code>true</code> if member is not <code>null</code> and is an instance of
		 * <code>java.lang.reflect.Field</code>
		 *
		 * @return <code>true</code> if this Result holds a Field member, <code>false</code> otherwise
		 */
		public boolean isField () {
			return member != null && member instanceof Field;
		}

		/**
		 * invoke the member Constructor or Method with the corresponding parameters
		 * on the specified object (which may be <code>null</code>)
		 *
		 * @param object -- object upon which to invoke method
		 *        (or <code>null</code> if method is static)
		 * @return result of invocation
		 * @exception Exception - any exception thrown by members of the Reflection API may be thrown
		 * @see "Java Reflection API"
		 */
		public Object invoke (Object object) throws Exception {
			if (isConstructor()) {
				value = ((Constructor) member).newInstance(params);
			}else if (isMethod()) {
				Method meth = (Method) member;
				meth.setAccessible(true);
				value = meth.invoke(object, params);
			}
			return value;
		} // invoke
	} // Result

	/**
	 * convenience method for creating a Reflector.Result object
	 * for use outside Reflector class
	 *
	 * @param member -- class member
	 * @param params -- parameters for invokation of member
	 * @return new Reflector.Result instance
	 */
	public Result createReflectorResult (Object member, Object[] params) {
		return new Result(member, params);
	}

	/**
	 * create a new Reflector for introspecting classes in order to find methods
	 * and constructors
	 */
	public Reflector () {
	}

	/**
	 * find the constructor of the given class that best fits the given parameters.
	 * notice that the constructor that is desired may not be the one returned
	 * as this method makes a best-guess at the types of the given arguments,
	 * especially if these are to be evaluated or interpreted as primitives.
	 *
	 * @param objClass -- class of object to be instantiated
	 * @param ctorArg -- either single argument for constructor or code block which, upon
	 *       evaluation, can be resolved to appropriate arguments
	 * @return Result object containing result of introspection or
	 *         <code>null</code> if no constructor could be found
	 */
	public Result findConstructor (Class objClass, Parameter ctorArg) {
		return findConstructor(objClass, new Parameter[] {ctorArg});
	}

	/**
	 * @see #findConstructor(Class, Parameter)
	 * 
	 * @param objClass -- class of object to be instantiated
	 * @param ctorArgs -- either single argument for constructor or code block which, upon
	 *       evaluation, can be resolved to appropriate arguments
	 * @return Result object containing result of introspection or
	 *         <code>null</code> if no constructor could be found
	 */
	public Result findConstructor (Class objClass, Parameter[] ctorArgs) {
		Constructor[] constructors = objClass.getConstructors();
		Object[] ctorParams = null;
		Result result = null;
		boolean matched = false;
		ctorArgs = ctorArgs == null ? new Parameter[0] : ctorArgs;
		for (int ctorNum = 0; !matched & ctorNum < constructors.length; ++ctorNum) {
			Class[] paramTypes = constructors[ctorNum].getParameterTypes();
			LoggingUtil.println(LoggingUtil.ALL, "Trying constructor:"
					+ constructors[ctorNum]);
			if (ctorArgs.length == 1 && ctorArgs[0].isScriptCode()) {
				ctorParams = ctorArgs[0].resolve(paramTypes);
			}else if (paramTypes.length == ctorArgs.length) {
				ctorParams = new Object[ctorArgs.length];
				for (int p = 0; p < ctorArgs.length; ++p) {
					ctorParams[p] = ctorArgs[p].resolve(paramTypes[p]);
					if (ctorParams[p] == null) {
						ctorParams = null;
						break;
					}
				}
			}
			matched = (ctorParams != null && paramTypes.length == ctorParams.length);
			if (matched) {
				LoggingUtil.printParams(LoggingUtil.ALL, ctorParams);
				// verify that ctor can be called with resulting params
				for (int p = 0; matched & p < ctorParams.length; ++p) {
					Class paramClass = ctorParams[p].getClass();
					matched &= (paramTypes[p].isAssignableFrom(paramClass) || isPrimitiveAssignableFrom(
						paramTypes[p], paramClass));
					if (!matched
							&& (Number.class.isAssignableFrom(paramClass) || paramClass.equals(Character.class))) {
						// try converting ctor arg to parameter type
						// if both are primitive
						if (paramClass.equals(Integer.class)
								&& (paramTypes[p].equals(Long.class) || paramTypes[p].equals(Long.TYPE))) {
							ctorParams[p] = new Long(((Integer) ctorParams[p]).intValue());
							matched = true;
						}else if (paramClass.equals(Long.class)
								&& (paramTypes[p].equals(Integer.class) || paramTypes[p].equals(Integer.TYPE))) {
							ctorParams[p] = new Integer(((Long) ctorParams[p]).intValue());
							matched = true;
						}else if (paramClass.equals(Double.class)
								&& (paramTypes[p].equals(Float.class) || paramTypes[p].equals(Float.TYPE))) {
							ctorParams[p] = new Float(((Double) ctorParams[p]).floatValue());
							matched = true;
						}else if (paramClass.equals(Float.class)
								&& (paramTypes[p].equals(Double.class) || paramTypes[p].equals(Double.TYPE))) {
							ctorParams[p] = new Double(((Float) ctorParams[p]).floatValue());
							matched = true;
						}else if (paramClass.equals(Character.class)
								&& paramTypes[p].equals(String.class)) {
							ctorParams[p] = ctorParams[p].toString();
							matched = true;
						}
					}
				} // next param
				if (matched) {
					result = new Result(constructors[ctorNum], ctorParams);
				}
			}
		} // next ctor
		return result;
	} // findConstructor

	/**
	 * find a method corresponding to the given name and parameters.
	 * This method attempts to find a method that can be invoked on the given methodOwner with the
	 * given name and that can accept the given parameters. Note that this method
	 * makes a "best guess" at the type of the given arguments, especially if these are to be evaluated
	 * or interpreted as primitives.
	 *
	 * @param methOwner -- object upon which method is to be invoked
	 * @param name -- method name
	 * @param arg -- single argument to method or code which, upon evaluation,
	 *        can be resolved to appropriate arguments for a method
	 * @return Result object containing result of introspection
	 *         or <code>null</code> if no method could be found
	 */
	public Result findMethod (Object methOwner, String name, Parameter arg) {
		return findMethod(methOwner, name, new Parameter[] {arg});
	}

	/**
	 * @see #findMethod(Object, String, Parameter)
	 * 
	 * @param methOwner -- object upon which method is to be invoked
	 * @param name -- method name
	 * @param args -- single argument to method or code which, upon evaluation,
	 *        can be resolved to appropriate arguments for a method
	 * @return Result object containing result of introspection
	 *         or <code>null</code> if no method could be found
	 */
	public Result findMethod (Object methOwner, String name, Parameter[] args) {
		List methods = findMethods(methOwner, name);
		Object[] params = null;
		Result result = null;
		boolean matched = false;
		args = args == null ? new Parameter[0] : args;
		for (int m = 0; !matched & m < methods.size(); ++m) {
			Method candidate = (Method) methods.get(m);
			Class[] paramTypes = candidate.getParameterTypes();
			LoggingUtil.println(LoggingUtil.ALL, "Trying candidate method "
					+ candidate);
			if (args.length == 1 && args[0].isScriptCode()) {
				params = args[0].resolve(paramTypes);
			}else if (paramTypes.length == args.length) {
				params = new Object[args.length];
				for (int p = 0; p < args.length; ++p) {
					params[p] = args[p].resolve(paramTypes[p], methOwner);
					if (params[p] == null) {
						params = null;
						break;
					}
				}
			}
			matched = (params != null && paramTypes.length == params.length);
			if (matched) {
				LoggingUtil.printParams(LoggingUtil.ALL, params);
				// verify that method can be called with resulting params
				for (int p = 0; matched & p < params.length; ++p) {
					Class paramClass = params[p].getClass();
					matched &= (paramTypes[p].isAssignableFrom(paramClass) || isPrimitiveAssignableFrom(
						paramTypes[p], paramClass));
					if (!matched
							&& (Number.class.isAssignableFrom(paramClass) || paramClass.equals(Character.class))) {
						// try converting method arg to parameter type
						// if both are primitive
						if (paramClass.equals(Integer.class)
								&& (paramTypes[p].equals(Long.class) || paramTypes[p].equals(Long.TYPE))) {
							params[p] = new Long(((Integer) params[p]).intValue());
							matched = true;
						}else if (paramClass.equals(Long.class)
								&& (paramTypes[p].equals(Integer.class) || paramTypes[p].equals(Integer.TYPE))) {
							params[p] = new Integer(((Long) params[p]).intValue());
							matched = true;
						}else if (paramClass.equals(Double.class)
								&& (paramTypes[p].equals(Float.class) || paramTypes[p].equals(Float.TYPE))) {
							params[p] = new Float(((Double) params[p]).floatValue());
							matched = true;
						}else if (paramClass.equals(Float.class)
								&& (paramTypes[p].equals(Double.class) || paramTypes[p].equals(Double.TYPE))) {
							params[p] = new Double(((Float) params[p]).floatValue());
							matched = true;
						}else if (paramClass.equals(Character.class)
								&& paramTypes[p].equals(String.class)) {
							params[p] = params[p].toString();
							matched = true;
						}
					}
				} // next param
				if (matched) {
					result = new Result(candidate, params);
				}
			}
		} // next candidate method
		return result;
	} // findMethod

	/**
	 * find a list of methods with the given name that can be invoked on
	 * the given object
	 *
	 * @param methodOwner -- object upon which methods returned could be invoked
	 * @param name -- method name
	 * @return list of methods with the given name that could be invoked on the given object
	 */
	public List findMethods (Object methodOwner, String name) {
		return methodOwner == null ? new LinkedList() : findMethods(
			methodOwner.getClass(), name);
	}

	/**
	 * find a list of methods with the given name that can be invoked on
	 * an object of the given class.
	 *
	 * @param cls -- class of object upon which methods returned could be invoked
	 * @param name -- method name
	 * @return list of methods with the given name that could be invoked on an object of the given class
	 */
	public List findMethods (Class cls, String name) {
		Method[] methods = cls.getMethods();
		List methList = new LinkedList();
		for (int m = 0; m < methods.length; ++m) {
			if (methods[m].getName().equals(name)) {
				methList.add(methods[m]);
			}
		}
		return methList;
	} // findMethods

	/**
	 * retreave a static field value with the given name from
	 * the given class
	 *
	 * @param name -- name of field to be queried
	 * @param cls -- class of which field is member
	 * @return value of field for that class or <code>null</code> if field does not exist
	 */
	public Object getFieldValue (String name, Class cls) {
		return getFieldValue(name, cls, null);
	}

	/**
	 * retreave a field value with the given name from
	 * the given object
	 *
	 * @param name -- name of field to be queried
	 * @param object -- object of which field is member
	 * @return value of field for that object or <code>null</code> if field does not exist
	 */
	public Object getFieldValue (String name, Object object) {
		return getFieldValue(name, object.getClass(), object);
	}

	private Object getFieldValue (String name, Class cls, Object object) {
		Field field = null;
		Object value = null;
		try {
			if (cls.isArray() && name.equals("length")) {
				value = new Integer(Array.getLength(object));
			}else {
				field = cls.getField(name);
				Class type = field.getType();
				if (type == Boolean.TYPE) {
					value = Boolean.valueOf(field.getBoolean(object));
				}else if (type == Double.TYPE) {
					value = new Double(field.getDouble(object));
				}else if (type == Float.TYPE) {
					value = new Float(field.getFloat(object));
				}else if (type == Integer.TYPE) {
					value = new Integer(field.getInt(object));
				}else if (type == Long.TYPE) {
					value = new Long(field.getLong(object));
				}else if (type.equals(String.class)) {
					value = field.get(object).toString();
				}else {
					value = field.get(object);
				}
			}
		}catch (Exception e) {
			// nothing
		}
		return value;
	} // getFieldValue

	/**
	 * set the value of the field with the given name on the given object
	 * to the value specified
	 *
	 * @param name -- name of field
	 * @param object -- object whose field is being set
	 * @param val -- value of field
	 * @return <code>true</code> if setting of field was successful, <code>false</code> otherwise
	 */
	public boolean setFieldValue (String name, Object object, Parameter val) {
		Class fType = null, oType = null;
		Field field = null;
		Object value = null;
		boolean success = true;
		try {
			field = object.getClass().getField(name);
			fType = field.getType();
			value = val.resolve(fType);
			oType = value.getClass();
			if ((fType.equals(Integer.class) || fType.equals(Integer.TYPE))
					&& oType.equals(Long.class)) {
				value = new Integer(((Long) value).intValue());
			}else if ((fType.equals(Float.class) || fType.equals(Float.TYPE))
					&& oType.equals(Double.class)) {
				value = new Float(((Double) value).floatValue());
			}
			field.set(object, value);
			LoggingUtil.println(LoggingUtil.ALL, "Set field " + name
					+ " of object " + object + " to value " + value);
		}catch (Exception e) {
			success = false;
		}
		return success;
	} // setFieldValue

	private boolean isPrimitiveAssignableFrom (Class formal, Class actual) {
		return (formal.isPrimitive() && actual.isPrimitive() && formal.equals(actual))
				|| (WRAPPER_TO_PRIMITIVE_MAP.get(actual) != null && ((Class) WRAPPER_TO_PRIMITIVE_MAP.get(actual)).equals(formal))
				|| (actual.equals(Integer.class) && formal.equals(Long.TYPE))
				|| (actual.equals(Integer.TYPE) && formal.equals(Long.TYPE))
				|| (actual.equals(Short.class) && formal.equals(Integer.TYPE))
				|| (actual.equals(Short.TYPE) && formal.equals(Integer.TYPE))
				|| (actual.equals(Float.TYPE) && formal.equals(Double.TYPE))
				|| (actual.equals(Float.class) && formal.equals(Double.TYPE));
	}
} // Reflector
