/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  IBM Corporation - initial API and implementation
*******************************************************************************/ 
package org.eclipse.actf.validation.core;

import org.eclipse.actf.model.IRenderableModel;
import org.eclipse.actf.model.InvalidComponentException;

/**
 * The ResultInvoker is used in conjunction with the
 * <code>IRenderableModel</code> interface in order to insure that all object
 * instantiations and/or method invocations effecting or depending upon the
 * state of components in GUI occur on the UI thread appropriate for the
 * model. The primary task of this class is to wrap the call to
 * <code>invoke</code> on the <code>Reflector.Result</code> specified in the
 * constructor.
 * 
 * @see IRenderableModel#invokeOnUIThread(Runnable)
 * 
 * @author Mike Squillace
 */
public class ResultInvoker implements Runnable {

	private Reflector.Result result;

	private Object receiver;

	private Object returnValue;

	private Throwable throwable;

	private IRenderableModel model;

	/**
	 * create an invoker for the specified result
	 * 
	 * @param result -
	 *            reflector result to be invoked
	 */
	public ResultInvoker(Reflector.Result result) {
		this(result, null, null);
	}

	/**
	 * create an invoker for the specified result which will use the given
	 * receiver as the receiver for the method call being invoked reflectively
	 * 
	 * @param result -
	 *            reflector result to be invoked
	 * @param receiver -
	 *            receiver for method invokation or <code>null</code> for
	 *            constructor or static method invokation
	 * 
	 * @param model -
	 *            IRenderableModel instance to which this applies
	 */
	public ResultInvoker(Reflector.Result result, Object receiver,
			IRenderableModel model) {
		this.result = result;
		this.receiver = receiver;
		this.model = model;
	}

	/**
	 * simply wraps the invocation of the <code>invoke</code> method of the
	 * result object specified at instantiation
	 * 
	 * @see org.eclipse.actf.validation.core.Reflector.Result#invoke(Object)
	 */
	public void run() {
		if (model != null && model.isValid(receiver)) {
			try {
				returnValue = result.invoke(receiver);
			} catch (Throwable t) {
				// Utils.println(Reporter.SYSTEM_NONFATAL, "Error in
				// invoker: " + result.getMember() + " on " + receiver, t);
				throwable = t;
			}
		} else {
			throwable = new InvalidComponentException("Object " + receiver
					+ " is invalid");
		}
	} // run

	/**
	 * returns the Throwable object should an exception have occured during
	 * invokation of <code>invoke</code> method
	 * 
	 * @return throwable or <code>null</code> if no exception occured
	 */
	public Throwable getThrowable() {
		return throwable;
	}

	/**
	 * return the return value from a method invokation or object instantiated
	 * by a constructor via the <code>invoke</code> method
	 * 
	 * @return return value from method or newly created object
	 */
	public Object getReturnValue() {
		return returnValue;
	}
} // ResultInvoker
