/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  IBM Corporation - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.core;

import java.lang.reflect.Member;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.actf.core.config.ConfigurationException;
import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.core.runtime.IRuntimeContext;
import org.eclipse.actf.core.runtime.RuntimeContextFactory;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.logging.LoggingUtil;
import org.eclipse.actf.validation.ValidationConstants;
import org.eclipse.actf.validation.reports.ValidationReporter;
import org.eclipse.actf.validation.rulebase.DocumentRuleBaseDescriptor;
import org.eclipse.actf.validation.rulebase.IRuleBase;
import org.eclipse.actf.validation.rulebase.RuleBaseManager;
import org.eclipse.actf.validation.validators.DefaultValidationContextFactory;
import org.eclipse.actf.validation.validators.IValidationContext;
import org.eclipse.actf.validation.validators.ValidationContextConstants;

/**
 * entry point for the ACTF validation engine and the point at which all
 * validation processes are initiated. A typical use is:
 * 
 * <p>
 * 
 * <pre>
 *  ValidationProcessManager vpm = ValidationProcessManager.getInstance();
 *  vpm.init();
 *  vpm.initiateValidation(...);
 * </pre>
 * 
 * <p>
 * During the manager's lifetime, it receives execution point notifications and
 * starts the validation processes associated with these execution points.
 * 
 * @see #init()
 * @see #initiateValidation(ExecutionPoint)
 * 
 * @author John Buslawski
 */
public class ValidationProcessManager {

	private Map processRegistry = new HashMap();
	private IConfiguration config = null;
	private ValidationReporter reporter = null;
	
	private static ValidationProcessManager instance = null;

	public static ValidationProcessManager getInstance() {
		if (instance == null) {
			instance = new ValidationProcessManager();
			instance.init();
		}
		return instance;
	}

	// default ctor
	protected ValidationProcessManager() {
	}

	/**
	 * initializes the validation process manager. This method should be called
	 * once before any validations are initiated. This method performs the
	 * following tasks:
	 * 
	 * <p>
	 * <ul>
	 * <li>creates the rulebase mappings and adds rulebases to the
	 * <code>RuleBaseManager</code>
	 * <li>creates the validation process mappings and registers them with the
	 * manager
	 * </ul>
	 * 
	 * @see RuleBaseManager
	 * @see DocumentRuleBaseDescriptor
	 * @see org.eclipse.actf.validation.core.IValidationProcess
	 */
	public void init() {
		try {
			IRuntimeContext context = RuntimeContextFactory.getInstance().getRuntimeContext();
			config = context.getConfiguration();
		} catch (ConfigurationException e) {
			throw new RuntimeException("Error initializing ValidationProcessManager", e);
		}
		
		Map valProcMap = DefaultValidationProcessMappingFactory.getInstance().getValidationProcessMappings();
		Iterator iter = valProcMap.keySet().iterator();
		while (iter.hasNext()) {
			String validatorName = (String) iter.next();
			String valProcName = (String) valProcMap.get(validatorName);
			registerProcess(validatorName, valProcName);
		}
	}

	/**
	 * Registers a IValidator/IValidationProcess pair.
	 * 
	 * @param validatorName --
	 *            the IValidator class name
	 * @param vpClassName -
	 *            the IValidationProcess class name
	 */
	public void registerProcess(String validatorName, String vpClassName) {
		if (!processRegistry.containsKey(validatorName)) {
			processRegistry.put(validatorName, vpClassName);
		}
	}

	/**
	 * Gets the IValidationProcess class name for this IValidator name.
	 * 
	 * @param validatorName --
	 *            the IValidator class name
	 * @return -- the IValidationProcess class name
	 */
	public String getValidationProcessName(String validatorName) {
		return (String) processRegistry.get(validatorName);
	}

	/**
	 * Receives an array of ExecutionPoints and starts a IValidationProcess for
	 * each one.
	 * 
	 * @param execPoints --
	 *            the array of execution points to validate
	 * @throws ValidationProcessException
	 */
	public void initiateValidation(ExecutionPoint[] execPoints)
			throws ValidationProcessException {
		for (int i = 0; i < execPoints.length; i++) {
			initiateValidation(execPoints[i], null);
		}
	}

	/**
	 * Receives an array of ExecutionPoints and a component bridge map and
	 * starts a IValidationProcess. Each ExecutionPoint uses the same component
	 * bridge map.
	 * 
	 * <p>
	 * A component bridge map is used to traverse trees of GUI components of
	 * differing models. The map contains keys that are components of one
	 * model that are to serve as the parents of components of another
	 * model. This tactic is useful when traversing types of components
	 * that do not allow access to children or parents of different
	 * models.
	 * 
	 * 
	 * @param execPoints --
	 *            the array of execution points to validate
	 * @param bridgeMap --
	 *            the component bridge map for these validations
	 * @throws ValidationProcessException
	 */
	public void initiateValidation(ExecutionPoint[] execPoints, Map bridgeMap)
			throws ValidationProcessException {
		for (int i = 0; i < execPoints.length; i++) {
			initiateValidation(execPoints[i], bridgeMap);
		}
	}

	/**
	 * Receives an ExecutionPoint and starts a IValidationProcess.
	 * 
	 * @param execPoint --
	 *            the execution point that begins a validation
	 * @throws ValidationProcessException
	 */
	public void initiateValidation(ExecutionPoint execPoint)
			throws ValidationProcessException {
		initiateValidation(execPoint, null);
	}

	/**
	 * Receives an ExecutionPoint and a component bridge map and starts a
	 * IValidationProcess. A component bridge map is used to traverse trees of
	 * GUI components of differing models. The map contains keys that are
	 * components of one model that are to serve as the parents of
	 * components of another model. This tactic is useful when traversing
	 * types of components that do not allow access to children or parents of
	 * different models.
	 * 
	 * 
	 * @param execPoint --
	 *            the execution point that begins a validation
	 * @param bridgeMap --
	 *            sets the component bridge map for this validation
	 * @throws ValidationProcessException
	 */
	public void initiateValidation(ExecutionPoint execPoint, Map bridgeMap)
			throws ValidationProcessException {
		if (execPoint == null) {
			throw new ValidationProcessException(
					"Unable to create an execution point."
							+ " No corresponding ExecutionPointDescriptor found in any IRuleBase.");
		}
		// find rulebase for the target
		String targetName = execPoint.getTarget().getClass().getName();
		IRuleBase rb = findRuleBaseForTarget(execPoint.getTarget());
		if (rb == null) {
			LoggingUtil.println(LoggingUtil.PROCESS_ALL,
					"ValidationProcessManager.initiateValidation() - unable to perform "
							+ "validation for target name " + targetName + ".");
			return;
		}
		// Find the IValidationProcess for the IValidator class of the
		// ExecutionPoint
		// in the registry and create an instance of it.
		String valProcName = getValidationProcessName(execPoint.getValidatorClassName());
		if (valProcName == null) {
			LoggingUtil.println(LoggingUtil.PROCESS_ALL,
							"ValidationProcessManager.initiateValidation() - registry does not contain a IValidationProcess entry for ExecutionPoint IValidator class named: " + execPoint.getName() + ".");
			throw new ValidationProcessException("Cannot find IValidationProcess in registry for ExecutionPoint IValidator class named: " + execPoint.getValidatorClassName());
		}
		// Create an appropriate IValidationContext for this IValidationProcess
		IValidationContext valCtx = DefaultValidationContextFactory
				.getInstance().getContextByProcessName(valProcName);
		if (valCtx == null) {
			throw new ValidationProcessException("Cannot create IValidationContext instance for process named: " + valProcName);
		}
		
		if (config == null) {
			try {
				IRuntimeContext context = RuntimeContextFactory.getInstance().getRuntimeContext();
				config = context.getConfiguration();
			} catch (ConfigurationException e) {
				throw new RuntimeException(e.getMessage(), e);
			}
		}
		
		valCtx.setParameter(ValidationContextConstants.CONFIGURATION, config);
		valCtx.setParameter(ValidationContextConstants.ELEMENT_BRIDGE_MAP, bridgeMap);
		valCtx.setParameter(ValidationContextConstants.REPORTER, createReporter());
		
		IValidationProcess valProc = null;
		try {
			valProc = (IValidationProcess) Class.forName(valProcName)
					.getConstructor(
							new Class[] { IRuleBase.class, ExecutionPoint.class,
									IValidationContext.class }).newInstance(
							new Object[] { rb, execPoint, valCtx });
		} catch (Exception e) {
			throw new ValidationProcessException("Cannot find IValidationProcess in registry for ExecutionPoint named: " + execPoint.getName());
		}
		
		// initialize the IValidationProcess
		valProc.init();
		// TODO - this is where we would queue the process to run if we were
		// multithreaded.
		valProc.run();
	}

	protected ValidationReporter createReporter() {
		if (reporter == null) {
			if (config == null) {
				try {
					IRuntimeContext context = RuntimeContextFactory
							.getInstance().getRuntimeContext();
					config = context.getConfiguration();
				} catch (ConfigurationException e) {
					// Could not obtain a Configuration. We can't do much
					// here but quit.
					throw new RuntimeException(e.getMessage(), e);
				}
			}
			config.setSymbolPool(ValidationConstants.VALIDATION_ID);
			reporter = createReporter(config
					.getIntParameter(ValidationConstants.REPORTER_LEVEL_KEY));
		} else {
			reporter.reset();
		}
		return reporter;
	}

	private ValidationReporter createReporter(int traceLevel) {
		ValidationReporter reporter = new ValidationReporter();
		reporter.setDefaultOutputLevel(traceLevel);
		reporter.setSourceID(IReporter.VALIDATION);
		reporter.open();
		return reporter;
	}

	/**
	 * get the reporter used by this manager
	 * 
	 * @return reporter used by this manager
	 */
	public ValidationReporter getReporter() {
		if (reporter == null) {
			createReporter();
		}
		return reporter;
	}

	protected IRuleBase findRuleBaseForTarget(Object target) {
		RuleBaseManager rbm = RuleBaseManager.getInstance();
		IRuleBase[] rulebases = rbm.getRuleBasesByTarget(target);
		if (rulebases.length == 0) {
			LoggingUtil.println(LoggingUtil.PROCESS_ALL,
					"ValidationProcessManager.findRuleBaseForTarget() - target name "
							+ target.getClass().getName()
							+ " is not linked to a IRuleBase.");
			return null;
		} else if (rulebases.length > 1) {
			LoggingUtil
					.println(
							LoggingUtil.PROCESS_ALL,
							"ValidationProcessManager.findRuleBaseForTarget() - target name "
									+ target.getClass().getName()
									+ " is linked to more than one IRuleBase. Returning the first IRuleBase.");
		}
		// TODO need to handle the case for multiple rule bases
		// for now just return the first IRuleBase
		return rulebases[0];
	}

	protected IRuleBase findRuleBaseForTarget(String targetName) {
		RuleBaseManager rbm = RuleBaseManager.getInstance();
		IRuleBase[] rulebases = rbm.getRuleBasesByTargetName(targetName);
		if (rulebases.length == 0) {
			LoggingUtil.println(LoggingUtil.PROCESS_ALL,
					"ValidationProcessManager.findRuleBaseForTarget() - target name "
							+ targetName + " is not linked to a IRuleBase.");
			return null;
		} else if (rulebases.length > 1) {
			LoggingUtil
					.println(
							LoggingUtil.PROCESS_ALL,
							"ValidationProcessManager.findRuleBaseForTarget() - target name "
									+ targetName
									+ " is linked to more than one IRuleBase. Returning the first IRuleBase.");
		}
		// TODO need to handle the case for multiple rule bases
		// for now just return the first IRuleBase
		return rulebases[0];
	}

	public ExecutionPoint createMethodExecutionPointForTarget(
			String execPointName, Object target, Object[] arguments,
			Member member) {
		ExecutionPoint execPoint = null;
		RuleBaseManager rbm = RuleBaseManager.getInstance();
		IRuleBase[] rulebases = rbm.getRuleBasesByTarget(target);
		if (rulebases.length == 0) {
			LoggingUtil.println(LoggingUtil.PROCESS_ALL,
					"ValidationProcessManager.findRuleBaseForTarget() - target name "
							+ target.getClass().getName()
							+ " is not linked to a IRuleBase.");
			return null;
		} else if (rulebases.length > 1) {
			LoggingUtil
					.println(
							LoggingUtil.PROCESS_ALL,
							"ValidationProcessManager.findRuleBaseForTarget() - target name "
									+ target.getClass().getName()
									+ " is linked to more than one IRuleBase. Returning the first IRuleBase.");
		}
		for (int i = 0; i < rulebases.length; i++) {
			// get the execution point descriptor from the new rule base using
			// the name from the model transition of the original
			// execution point.
			ExecutionPointDescriptor newExecPointDesc = rulebases[i]
					.getExecutionPointDescriptor(execPointName);
			if (newExecPointDesc != null) {
				// Create an instance of that ExecutionPoint with the name and
				// IValidator class name
				// from the ExecutionPointDescriptor. All of the other fields
				// are the same and can
				// be taken from the original execution point.
				execPoint = new MethodExecutionPoint(execPointName,
						newExecPointDesc.getValidatorName(), target, arguments,
						member);
				// TODO need to handle the case for multiple rule bases that
				// match the component AND
				// also define an ExecutionPoint with the same name. For now
				// just return the ExecutionPoint
				// from the first IRuleBase that matches.
				break;
			} else {
				LoggingUtil.println(LoggingUtil.PROCESS_ALL,
						"Unable to find execution point descriptor for execution point with name "
								+ execPointName + " in IRuleBase - "
								+ rulebases[i].getName());
			}
		}
		if (execPoint == null) {
			LoggingUtil
					.println(
							LoggingUtil.PROCESS_ALL,
							"Unable to create an execution point name: "
									+ execPointName
									+ ". No corresponding ExecutionPointDescriptor found in any IRuleBase.");
		}
		return execPoint;
	}

	public ExecutionPoint createEventExecutionPointForTarget(
			String execPointName, Object target, Object event) {
		ExecutionPoint execPoint = null;
		RuleBaseManager rbm = RuleBaseManager.getInstance();
		IRuleBase[] rulebases = rbm.getRuleBasesByTarget(target);
		if (rulebases.length == 0) {
			LoggingUtil.println(LoggingUtil.PROCESS_ALL,
					"ValidationProcessManager.findRuleBaseForTarget() - target name "
							+ target.getClass().getName()
							+ " is not linked to a IRuleBase.");
			return null;
		} else if (rulebases.length > 1) {
			LoggingUtil
					.println(
							LoggingUtil.PROCESS_ALL,
							"ValidationProcessManager.findRuleBaseForTarget() - target name "
									+ target.getClass().getName()
									+ " is linked to more than one IRuleBase. Returning the first IRuleBase.");
		}
		for (int i = 0; i < rulebases.length; i++) {
			// get the execution point descriptor from the new rule base using
			// the name from the model transition of the original
			// execution point.
			ExecutionPointDescriptor newExecPointDesc = rulebases[i]
					.getExecutionPointDescriptor(execPointName);
			if (newExecPointDesc != null) {
				// Create an instance of that ExecutionPoint with the name and
				// IValidator class name
				// from the ExecutionPointDescriptor. All of the other fields
				// are the same and can
				// be taken from the original execution point.
				execPoint = new EventExecutionPoint(execPointName,
						newExecPointDesc.getValidatorName(), target, event);
				// TODO need to handle the case for multiple rule bases that
				// match the component AND
				// also define an ExecutionPoint with the same name. For now
				// just return the ExecutionPoint
				// from the first IRuleBase that matches.
				break;
			} else {
				LoggingUtil.println(LoggingUtil.PROCESS_ALL,
						"Unable to find execution point descriptor for execution point with name "
								+ execPointName + " in IRuleBase - "
								+ rulebases[i].getName());
			}
		}
		if (execPoint == null) {
			LoggingUtil
					.println(
							LoggingUtil.PROCESS_ALL,
							"Unable to create an execution point name: "
									+ execPointName
									+ ". No corresponding ExecutionPointDescriptor found in any IRuleBase.");
		}
		return execPoint;
	}
}
