/*******************************************************************************
 * Copyright (c) 2004, 2008 IBM Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * IBM Corporation - initial API and implementation
 *******************************************************************************/

package org.eclipse.actf.validation.reports;

import java.sql.Time;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.validation.events.IReportModifyListener;
import org.eclipse.actf.validation.events.ReportModifyEvent;

/**
 * a validation report as generated by an AOP-based validation from a distinct
 * JVM. This class serves as a model for all views that display reports or
 * report-related information, such as creation contexts.
 * 
 * @author Mike Squillace
 */
public class ValidationReport {

	private List records;

	private Map contextMap;

	private String reportID;

	private String description, source;

	private List listeners;

	private Map propMap;

	private boolean allowDupRecords = false;

	private boolean persistent = false;

	private boolean active = true;

	private Time timestamp = null;

	private String tool = null;

	private String model = null;

	public ValidationReport() {
		this(new ValidationReportRecord[0], new CreationContext[0], "<None>");
	}

	public ValidationReport(ValidationReportRecord[] records,
			CreationContext[] contexts, String id) {
		clear();
		reportID = id;
		// Add all records, even duplicates
		if (allowDupRecords) {
			this.records.addAll(Arrays.asList(records));
		} else {
			// add only unique records
			addUniqueRecords(records);
		}
		for (int i = 0; i < contexts.length; ++i) {
			addCreationContext(contexts[i]);
		}
	}

	/**
	 * clears this report instance for a new report
	 * 
	 */
	public void clear() {
		records = new LinkedList();
		contextMap = new HashMap();
		propMap = new HashMap();
		reportID = "<none>";
		listeners = new LinkedList();
		description = source = null;
	}

	/**
	 * returns whether or not this report has content. A report is empty if it
	 * has no creation contexts and no records associated with it.
	 * 
	 * @return <code>true</code> if this report has no content
	 */
	public boolean isEmpty() {
		return contextMap.isEmpty() && records.isEmpty();
	}

	/**
	 * add a record to this report
	 * 
	 * @param record -
	 *            record to be added
	 */
	public void addRecord(ValidationReportRecord record) {
		addRecords(new ValidationReportRecord[] { record });
	}

	/**
	 * add the specified records to this report
	 * 
	 * @param records -
	 *            records to be added
	 */
	public void addRecords(ValidationReportRecord[] records) {
		// Add all records, even duplicates
		if (allowDupRecords) {
			this.records.addAll(Arrays.asList(records));
		} else {
			// add only unique records
			addUniqueRecords(records);
		}
		fireReportModifyEvent(new ReportModifyEvent(this, records));
	}

	/**
	 * add the creation context to this report
	 * 
	 * @param context -
	 *            context to be added
	 */
	public void addCreationContext(CreationContext context) {
		addCreationContexts(new CreationContext[] { context });
	}

	/**
	 * add the creation contexts to this report
	 * 
	 * @param contexts -
	 *            contexts to be added
	 */
	public void addCreationContexts(CreationContext[] contexts) {
		for (int c = 0; c < contexts.length; ++c) {
			CreationContext context = contexts[c];
			if (context != null) {
				contextMap.put(context.getComponentID(), context);
			}
		}
		fireReportModifyEvent(new ReportModifyEvent(this, contexts));
	}

	public void addCreationContexts(Map contexts) {
		if (!contexts.isEmpty()) {
			contextMap.putAll(contexts);
		}
	}

	public String getReportID() {
		return reportID;
	}

	public void setReportID(String id) {
		reportID = id;
	}

	/**
	 * return all report records in this report
	 * 
	 * @return all records in this validation report
	 */
	public ValidationReportRecord[] getRecords() {
		return (ValidationReportRecord[]) records
				.toArray(new ValidationReportRecord[records.size()]);
	}

	/**
	 * return the creation contexts added to this report. The returned map is
	 * keyed by the id of its associated creation context.
	 * 
	 * @return all creation contexts added to this report
	 */
	public Map getCreationContexts() {
		return contextMap;
	}

	/**
	 * returns the report description
	 * 
	 * @return description
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * set the description
	 * 
	 * @param desc
	 */
	public void setDescription(String desc) {
		description = desc;
	}

	/**
	 * returns the source of the report
	 * 
	 * @return source
	 */
	public String getSource() {
		return source;
	}

	/**
	 * set the source of the report
	 * 
	 * @param source
	 */
	public void setSource(String source) {
		this.source = source;
	}

	/**
	 * get the property specified by the key
	 * 
	 * @param key
	 * @return property value for the key or <code>null</code>
	 */
	public String getProperty(String key) {
		return (String) propMap.get(key);
	}

	/**
	 * set a property on this report
	 * 
	 * @param key
	 * @param val
	 */
	public void setProperty(String key, String val) {
		propMap.put(key, val);
	}

	/**
	 * add a listener to be notified when items are added to this report.
	 * Currently, additions are the only operations that constitute a
	 * modification.
	 * 
	 * @param listener
	 */
	public void addReportModifyListener(IReportModifyListener listener) {
		if (listener != null) {
			listeners.add(listener);
		}
	}

	/**
	 * remove the listener from the list of listeners to be notified when a
	 * modification to this report takes place
	 * 
	 * @param listener
	 */
	public void removeReportModifyListener(IReportModifyListener listener) {
		if (listener != null) {
			listeners.remove(listener);
		}
	}

	protected void fireReportModifyEvent(ReportModifyEvent event) {
		for (Iterator iter = listeners.iterator(); iter.hasNext();) {
			((IReportModifyListener) iter.next()).reportModified(event);
		}
	}

	public boolean isPersistent() {
		// this code moved to the report.DB package
		// added setter so that package can set the value
		/*
		 * if ((getProperty(ReportServices.REPORT_PERSISTENT_KEY))
		 * .equalsIgnoreCase("true")) { return true; } else { return false; }
		 */
		return persistent;
	}

	public boolean isActive() {
		// this code moved to the report.DB package
		// added setter so that package can set the value
		/*
		 * String a = getProperty(ReportServices.ACTIVE_KEY); if (a == null ||
		 * a.equalsIgnoreCase("F")) { return false; } else { return true; }
		 */
		return active;
	}

	public void addUniqueRecords(ValidationReportRecord[] newRecords) {
		for (int i = 0; i < newRecords.length; i++) {
			ValidationReportRecord newRecord = newRecords[i];
			Iterator recIter = records.listIterator();
			boolean foundMatch = false;
			while (recIter.hasNext()) {
				ValidationReportRecord rec = (ValidationReportRecord) recIter.next();
				if (rec != null && rec.equals(newRecord)) {
					foundMatch = true;
					break;
				}
			}
			if (!foundMatch) {
				records.add(newRecord);
			}
		}
	}

	public void setPersistent(boolean persistent) {
		this.persistent = persistent;
	}

	public void setActive(boolean active) {
		this.active = active;
	}

	public Time getTimestamp() {
		return timestamp;
	}

	public void setTimestamp(Time timestamp) {
		this.timestamp = timestamp;
	}

	public String getTool() {
		return tool;
	}

	public void setTool(String tool) {
		this.tool = tool;
	}

	public String getModel() {
		return model;
	}

	public void setModel(String model) {
		this.model = model;
	}

	public String getSummary() {
		int error = 0, warning = 0, info = 0;
		for (Iterator itr = records.iterator(); itr.hasNext();) {
			ValidationReportRecord record = (ValidationReportRecord) itr.next();
			if (record != null) {
				switch (record.getLevel()) {
				case (IReporter.ERROR):
					error += 1;
					break;
				case (IReporter.WARNING):
					warning += 1;
					break;
				case (IReporter.INFO):
					info += 1;
					break;
				}

			}

		}
		String summary = "Total errors = "+ error + ", warnings = " + warning + ", info = " + info;
		return summary;
	}
} // ValidationReport
