/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  IBM Corporation - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.reports;

import java.util.Arrays;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.actf.validation.events.IReportGroupModifyListener;
import org.eclipse.actf.validation.events.ReportGroupModifyEvent;


/**
 * a container for a set of reports
 * 
 * @author Mike Squillace
 * 
 */
public class ValidationReportGroup {

	private List reports = new LinkedList();

	private List listeners = new LinkedList();

	public ValidationReportGroup() {
		this(new ValidationReport[0]);
	}

	public ValidationReportGroup(ValidationReport[] reports) {
		if (reports != null) {
			this.reports.addAll(Arrays.asList(reports));
		}
	}

	/**
	 * clear the group of all reports
	 * 
	 */
	public void clear() {
		ValidationReport[] reports = getReports();
		listeners.clear();
		this.reports.clear();
		fireReportGroupModifyEvent(new ReportGroupModifyEvent(this, reports,
				ReportGroupModifyEvent.REPORTS_REMOVED));
	}

	/**
	 * returns whether or not this report group has reports in it
	 * 
	 * @return <code>true</code> if this group is empty
	 */
	public boolean isEmpty() {
		return reports.isEmpty();
	}

	/**
	 * add a report to this group
	 * 
	 * @param report -
	 *            report to be added
	 */
	public void addReport(ValidationReport report) {
		addReports(new ValidationReport[] { report });
	}

	/**
	 * add the specified reports to this group
	 * 
	 * @param reports -
	 *            reports to be added
	 */
	public void addReports(ValidationReport[] reports) {
		for (int r = 0; r < reports.length; ++r) {
			ValidationReport report = reports[r];
			if (report != null) {
				this.reports.add(report);
			}
		}
		fireReportGroupModifyEvent(new ReportGroupModifyEvent(this, reports,
				ReportGroupModifyEvent.REPORTS_ADDED));
	}

	/**
	 * remove a report from this group
	 * 
	 * @param report -
	 *            report to be removed
	 */
	public void removeReport(ValidationReport report) {
		removeReports(new ValidationReport[] { report });
	}

	/**
	 * remove the specified reports from this group
	 * 
	 * @param reports -
	 *            reports to be removed
	 */
	public void removeReports(ValidationReport[] reports) {
		for (int r = 0; r < reports.length; ++r) {
			ValidationReport report = reports[r];
			if (report != null && this.reports.contains(report)) {
				this.reports.remove(report);
			}
		}
		fireReportGroupModifyEvent(new ReportGroupModifyEvent(this, reports,
				ReportGroupModifyEvent.REPORTS_REMOVED));
	}

	/**
	 * get all reports in this group
	 * 
	 * @return all reports in this group
	 */
	public ValidationReport[] getReports() {
		return (ValidationReport[]) reports
				.toArray(new ValidationReport[reports.size()]);
	}

	/**
	 * retreave a validation report by its id
	 * 
	 * @param id
	 * @return report with given id or <code>null</code> if no report with the
	 *         given id exists in this group
	 */
	public ValidationReport getReport(String id) {
		ValidationReport result = null;
		for (Iterator iter = reports.iterator(); result == null
				& iter.hasNext();) {
			ValidationReport report = (ValidationReport) iter.next();
			if (id.equals(report.getReportID())) {
				result = report;
			}
		}
		return result;
	}

	/**
	 * add a listener to be notified when reports are added to this group
	 * 
	 * @param listener
	 */
	public void addReportGroupModifyListener(IReportGroupModifyListener listener) {
		if (listener != null) {
			listeners.add(listener);
		}
	}

	/**
	 * remove the listener from the list of listeners to be notified when a
	 * modification to this group takes place
	 * 
	 * @param listener
	 */
	public void removeReportGroupModifyListener(
			IReportGroupModifyListener listener) {
		if (listener != null) {
			listeners.remove(listener);
		}
	}

	protected void fireReportGroupModifyEvent(ReportGroupModifyEvent event) {
		for (Iterator iter = listeners.iterator(); iter.hasNext();) {
			((IReportGroupModifyListener) iter.next())
					.reportGroupModified(event);
		}
	}
}
