/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
* 
* Contributors:
* 	IBM Corporation - Initial  API

*******************************************************************************/ 
package org.eclipse.actf.validation.rules;

import org.eclipse.actf.validation.validators.IValidationContext;

public interface ICriterion {
	/**
	 * adds a criterion(or rule) to the list of criterion(rule parts or nested rules) 
	 * that are executed when this Criterion(rule) is evaluated.
	 * 
	 * @param criterion - criterion which encapsulates rule
	 */
	public void addPart(ICriterion criterion);
	
	/**
	 * set the criterion(rule) parts or nested criterion(rules) for this criterion(rule).
	 * 
	 * @param criterion - list of encapsulated rules(Criterion)
	 */
	public void setParts(ICriterion[] criterion);
	
	/**
	 * returns the criterion(rule) parts or nested criteria(rules) for the criterion/rule
	 * 
	 * @return list of criterion which may be an instanceof rules
	 */
	public ICriterion[] getParts();
	
	/**
	 * set the parent rule of this rule. Rules can be nested. There are some
	 * fields that are set only on the parent rule for all nested rule parts.
	 * 
	 * @param parent
	 *            rule
	 */
	public void setParent(ICriterion parent);
	
	/**
	 * return the parent rule of this rule.
	 * 
	 * @return instance of the parent rule
	 */
	public ICriterion getParent();
	
	
	/**
	 * return the IRuleSet containing this IRule.
	 * 
	 * @return instance of the containing IRuleSet
	 */
	public IRuleSet getRuleSet();
	
	
	/**
	 * Set the containing IRuleSet for this rule. A IRule is contained by only one
	 * IRuleSet.
	 * 
	 * @param iRuleSet
	 *            parent rule
	 */
	public void setRuleSet(IRuleSet iRuleSet);
	
	/**
	 * retrieves the attribute with the given name for this rule
	 * 
	 * @param name name of attribute
	 * @return attribute with given name or <code>null</code>
	 */
	public String getAttribute (String name);
	
	/**
	 * set an arbitrary attribute for this rule
	 * 
	 * @param name name of attribute
	 * @param value value of attribute
	 */
	public void setAttribute (String name, String value);
	
	/**
	 * apply this rule to the given target using the given 
	 * context. The returned value indicates whether or not a violation of
	 * some kind has taken place.
	 * 
	 * <p>Note that the context object can be used to retrieve a great deal of information about the current validation 
	 * process. In particular, it can be used to update the validation report being generated with 
	 * the violations that result from an application of this rule to the given target.
	 *
	 * @param context context under which rule is being evaluated (supplied by engine)
	 * @param target object to which rule is to be applied
	 * @return whether or not the target violates the rule
	 * @see #initialize(Object)
	 * @see org.eclipse.actf.validation.validators.IValidationContext
	 * @see org.eclipse.actf.validation.validators.ValidationContextConstants
	 * @throws Exception
	 */
	public boolean apply(IValidationContext context, Object target) throws Exception;

	/**
	 * initialize this rule with any content that might be necessary for its 
	 * application to a target during the invocation of the <code>apply</code> method.
	 * 
	 * <p>
	 * The specified content may take any form. It may be an XML element, a
	 * .properties file, a string, etc. This method may be invoked any time prior to calling 
	 * <code>apply</code>.
	 * 
	 * @param content - initialization content for the rule
	 * @see #apply(IValidationContext, Object)
	 * @throws Exception
	 */
	public void initialize(Object content) throws Exception;
	
}
