/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  IBM Corporation - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.ui;

import java.lang.reflect.Array;
import java.lang.reflect.Method;

import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.validation.ui.views.XSelectionAdapter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Widget;

public final class GuiUtils
{

	protected static ValidationUiPlugin plugin = ValidationUiPlugin.getDefault();

	public static abstract class BaseSelectionAdapter extends SelectionAdapter
	{

		public void widgetDefaultSelected (SelectionEvent e) {
			widgetSelected(e);
		}
	}

	protected GuiUtils () {
	}

	protected static boolean enableToolTips = true;

	//TODO - eliminate this?  it isn't being used anywhere
	public static boolean getEnableToolTips () {
		boolean res = enableToolTips;
		String s = System.getProperty("org.eclipse.actf.core.main.enableToolTips");
		if (s != null) {
			res = s.equalsIgnoreCase("true");
		}
		return res;
	}

	public static void setEnableToolTips (boolean enable) {
		enableToolTips = enable;
	}

	protected static IErrorHandler errorHandler;

	public static IErrorHandler getErrorHandler () {
		return errorHandler;
	}

	public static void setErrorHandler (IErrorHandler iErrorHandler) {
		errorHandler = iErrorHandler;
	}

	public static int reportError (String message) {
		int res = -1;
		if (errorHandler != null) {
			res = errorHandler.handleError(message);
		}else {
			plugin.trace(IReporter.ERROR, message);
		}
		return res;
	}

	public static int reportException (String message, Exception e) {
		int res = -1;
		if (errorHandler != null) {
			res = errorHandler.handleException(message, e);
		}else {
			plugin.logException(message, e);
		}
		return res;
	}

	protected static void setValue (Widget c, String meth, Class argType,
									Object value) {
		try {
			Method m = c.getClass().getMethod(meth, new Class[] {argType});
			Object[] oa = (Object[]) Array.newInstance(argType, 1);
			oa[0] = value;
			m.invoke(c, oa);
		}catch (NoSuchMethodException nme) {
		}catch (Exception e) {
			reportException("cannot " + meth + "(" + argType + "  " + value
					+ ")", e);
		}
	}

	protected static final Class[] selType = {SelectionListener.class};

	public static void configureCommand (final Widget c, final Object target,
											final String handler) {
		try {
			Method m = c.getClass().getMethod("addSelectionListener", selType);
			SelectionListener sl = new BaseSelectionAdapter() {

				public void widgetSelected (SelectionEvent e) {
					try {
						Method m = target.getClass().getMethod(handler, (Class[]) null);
						m.invoke(target, (Object[]) null);
					}catch (NoSuchMethodException nme) {
						plugin.trace(
							IReporter.ERROR,
							"configureCommand command not found (may not be public): "
									+ target.getClass().getName() + '.'
									+ handler);
					}catch (Exception ex) {
						reportException(
							"cannot dispatch command to "
									+ target.getClass().getName() + '.'
									+ handler, ex);
					}
				}
			};
			m.invoke(c, new Object[] {sl});
		}catch (NoSuchMethodException nme) {
		}catch (Exception e) {
			reportException("cannot register handler for " + handler, e);
		}
	}

	public static void configureCommand (final Widget c, final Runnable r) {
		SelectionListener sl = new BaseSelectionAdapter() {

			public void widgetSelected (SelectionEvent e) {
				try {
					r.run();
				}catch (Exception ex) {
					reportException("cannot dispatch command", ex);
				}
			}
		};
		configureCommand(c, sl);
	}

	public static void configureCommand (final Widget c,
											final SelectionListener sl) {
		try {
			Method m = c.getClass().getMethod("addSelectionListener", selType);
			m.invoke(c, new Object[] {sl});
		}catch (NoSuchMethodException nme) {
		}catch (Exception e) {
			reportException("cannot register handler", e);
		}
	}

	public static final String ID_KEY = "org.eclipse.actf.validation.ui.Widget_ID";

	public static String getControlId (Widget c) {
		return (String) c.getData(ID_KEY);
	}

	public static void setControlId (Widget c, String id) {
		c.setData(ID_KEY, id);
	}

	/**
	 * Configures a Widget
	 * @param c Widget to configure
	 * @param id id value to give to widget via setData; shoudl be unique per GUI (at a minimum, per Composite). Highly recommended.
	 * Uses key "org.eclipse.actf.validation.ui.Widget_ID"
	 * @param ld LayoutData sets the layout data the the control. Optional
	 * @param tooltip Text for the tool tip; defaults to text if missing. Optional
	 * @param image Image for the control. Optional.
	 * @param text Text for the control. Optional.
	 * @param enabled Sets the enabled state. Default true.
	 *
	 */
	public static void configureControl (Widget c, String id, Object ld,
											String tooltip, Image image,
											String text, boolean enabled) {
		if (id != null) {
			setControlId(c, id);
		}
		if (ld != null) {
			if (c instanceof Control) {
				((Control) c).setLayoutData(ld);
			}else {
				throw new IllegalArgumentException("only controls support layout data");
			}
		}
		if (text != null) {
			setValue(c, "setText", String.class, text);
			if (tooltip == null) {
				tooltip = text;
			}
		}
		if (image != null) {
			setValue(c, "setImage", Image.class, image);
		}
		if (enableToolTips) {
			if (tooltip != null) {
				while (tooltip.endsWith(" ")) {
					tooltip = tooltip.substring(0, tooltip.length() - 1);
				}
				if (tooltip.endsWith(":")) {
					tooltip = tooltip.substring(0, tooltip.length() - 1);
				}
				setValue(c, "setToolTipText", String.class, tooltip);
			}
		}
		setValue(c, "setEnabled", Boolean.class, enabled ? Boolean.TRUE
				: Boolean.FALSE);
	}

	public static void configureControl (Widget c, String id, Object ld,
											String tooltip, Image image,
											String text) {
		configureControl(c, id, ld, tooltip, image, text, true);
	}

	public static void configureControl (Widget c, String id, Object ld,
											String tooltip, String text) {
		configureControl(c, id, ld, tooltip, null, text);
	}

	public static void configureControl (Widget c, String id, Object ld,
											String tooltip, String text,
											boolean enabled) {
		configureControl(c, id, ld, tooltip, null, text, enabled);
	}

	public static void configureControl (Widget c, String id, Object ld,
											String tooltip, Image image) {
		configureControl(c, id, ld, tooltip, image, null);
	}

	public static void configureControl (Widget c, String id, Object ld,
											String tooltip, Image image,
											boolean enabled) {
		configureControl(c, id, ld, tooltip, image, null, enabled);
	}

	public static void configureControl (Widget c, String id, Object ld,
											String tooltip) {
		configureControl(c, id, ld, tooltip, null, null);
	}

	public static void configureControl (Widget c, String id, Object ld,
											String tooltip, boolean enabled) {
		configureControl(c, id, ld, tooltip, null, null, enabled);
	}

	public static void configureControl (Widget c, String id, String tooltip,
											Image image, String text) {
		configureControl(c, id, null, tooltip, image, text, true);
	}

	public static void configureControl (Widget c, String id, String tooltip,
											String text) {
		configureControl(c, id, null, tooltip, null, text);
	}

	public static void configureControl (Widget c, String id, String tooltip,
											String text, boolean enabled) {
		configureControl(c, id, null, tooltip, null, text, enabled);
	}

	public static void configureControl (Widget c, String id, String tooltip,
											Image image) {
		configureControl(c, id, null, tooltip, image, null);
	}

	public static void configureControl (Widget c, String id, String tooltip,
											Image image, boolean enabled) {
		configureControl(c, id, null, tooltip, image, null, enabled);
	}

	public static void configureControl (Widget c, String id, String tooltip) {
		configureControl(c, id, null, tooltip, null, null);
	}

	public static void configureControl (Widget c, String id, String tooltip,
											boolean enabled) {
		configureControl(c, id, null, tooltip, null, null, enabled);
	}

	public static void configureControl (Widget c, String text) {
		configureControl(c, (String) null, (String) null, text, true);
	}

	public static Button createPushButton (Composite parent, String name) {
		return createPushButton(parent, name, null, null, null);
	}

	public static Button createPushButton (Composite parent, String name,
											Object layoutData) {
		return createPushButton(parent, name, null, null, layoutData);
	}

	public static Button createPushButton (Composite parent, String name,
											final Object receiver,
											final String handler) {
		return createPushButton(parent, name, receiver, handler, null);
	}

	public static Button createPushButton (Composite parent, String name,
											final Object receiver,
											final String handler,
											Object layoutData) {
		Button b = createButton(parent, SWT.PUSH, name, layoutData);
		if (receiver != null && handler != null) {
			b.addSelectionListener(new XSelectionAdapter() {

				public void widgetSelected (SelectionEvent e) {
					try {
						Method m = receiver.getClass().getMethod(handler, (Class[]) null);
						m.invoke(receiver, (Object[]) null);
					}catch (Exception ex) {
						// ex.printStackTrace();
						plugin.logException(
							"exception processing event handler " + handler, ex);
						e.widget.getDisplay().beep();
					}
				}
			});
		}
		return b;
	}

	public static Button createCheckButton (Composite parent, String name) {
		return createCheckButton(parent, name, null);
	}

	public static Button createCheckButton (Composite parent, String name,
											Object layoutData) {
		return createButton(parent, SWT.CHECK, name, layoutData);
	}

	public static Button createRadioButton (Composite parent, String name) {
		return createRadioButton(parent, name, null);
	}

	public static Button createRadioButton (Composite parent, String name,
											Object layoutData) {
		return createButton(parent, SWT.RADIO, name, layoutData);
	}

	public static Button createToggleButton (Composite parent, String name) {
		return createToggleButton(parent, name, null);
	}

	public static Button createToggleButton (Composite parent, String name,
												Object layoutData) {
		return createButton(parent, SWT.TOGGLE, name, layoutData);
	}

	public static Button createButton (Composite parent, int style,
										String name, Object layoutData) {
		Button b = new Button(parent, style);
		if (name != null) {
			b.setText(name);
		}
		if (layoutData != null) {
			b.setLayoutData(layoutData);
		}
		return b;
	}

	public static void configureLayout (Control c, FormAttachment left,
										FormAttachment top,
										FormAttachment right,
										FormAttachment bottom) {
		FormData fd = new FormData();
		if (left != null) {
			fd.left = left;
		}
		if (top != null) {
			fd.top = top;
		}
		if (right != null) {
			fd.right = right;
		}
		if (bottom != null) {
			fd.bottom = bottom;
		}
		c.setLayoutData(fd);
	}
}
