/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  IBM Corporation - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.ui;

import java.io.File;
import java.io.FileInputStream;
import java.io.ObjectInputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.eclipse.actf.core.ActfCorePlugin;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.logging.LoggingUtil;
import org.eclipse.actf.util.resources.ClassLoaderCache;
import org.eclipse.actf.validation.rulebase.DocumentRuleBaseDescriptor;
import org.eclipse.actf.validation.rulebase.RuleBaseManager;
import org.eclipse.actf.validation.ui.preferences.ValidationPreferenceInitializer;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchWindow;
import org.osgi.framework.BundleContext;

/**
 * The main plugin class to be used in the desktop.
 */
public class ValidationUiPlugin extends AbstractValidationUIPlugin {

	public static final String VALIDATION_UI_PLUGIN_ID = "org.eclipse.actf.validation.ui";
	
	public static final String VALIDATION_UI_PLUGIN_CLASS = "org.eclipse.actf.validation.ui.AbstractValidationUIPlugin"; 

	public static final String VALIDATION_PLUGIN_RESOURCE_ID = VALIDATION_UI_PLUGIN_ID + "Resources";

	public static final String DEFAULT_IMAGE_NAME = "actf.gif";

	public static final String VALIDATION_UI_TRACE_FILE = "ui_trace.out";
	
	private static ValidationUiPlugin plugin;

	public String getPluginId() {
		return VALIDATION_UI_PLUGIN_ID;
	}

	public String getResourceBundleId() {
		return VALIDATION_PLUGIN_RESOURCE_ID;
	}
	protected String getTraceOptionId () {
		return VALIDATION_UI_PLUGIN_ID + "/" + TRACE_OPTION;
	}

	protected String getDebugOptionId () {
		return VALIDATION_UI_PLUGIN_ID + "/" + DEBUG_OPTION;
	}

	protected String getLogOptionId () {
		return VALIDATION_UI_PLUGIN_ID + "/" + LOG_OPTION;
	}
	
	protected String getDefaultImageName() {
		return DEFAULT_IMAGE_NAME;
	}
	
	protected String getValidationTraceFile() {
		return VALIDATION_UI_TRACE_FILE;
	}
	
	public ValidationUiPlugin() {
		super();
		plugin = this;
	}
	
	public static ValidationUiPlugin getDefault() {
		return plugin;
	}
	
	/**
	 * This method is called upon plug-in activation
	 */
	public void start (BundleContext context) throws Exception {
		// Be sure to register bundles and load plugins before starting
		// to be sure resources can be found
		
		super.start(context);
		registerAddedRulebases(getPreferenceStore());
	}

	/**
	 * This method is called when the plug-in is stopped
	 */
	public void stop (BundleContext context) throws Exception {
		super.stop(context);
	}	
	
	public Shell getShell() {
		Shell result = null;
		IWorkbenchWindow window = getDefault().getWorkbench()
				.getActiveWorkbenchWindow();
		if (window != null) {
			result = window.getShell();
		}
		return result;
	}

	private void registerAddedRulebases(IPreferenceStore store) {

		String rulebases = store
				.getString(ValidationPreferenceInitializer.P_NEW_RULEBASES);
		DocumentRuleBaseDescriptor desc = null;
		Map rulebaseMap;
		if (rulebases != null) {
			rulebaseMap = ValidationPreferenceInitializer.toMap(rulebases);
			Set keys = rulebaseMap.keySet();
			for (Iterator iter = rulebaseMap.keySet().iterator(); iter
					.hasNext();) {
				String rulebaseName = (String) iter.next();
				String descriptorFilename = (String) rulebaseMap
						.get(rulebaseName);
				FileInputStream fis = null;
				ObjectInputStream in = null;
				try {
					fis = new FileInputStream(descriptorFilename);
					in = new ObjectInputStream(fis);
					desc = (DocumentRuleBaseDescriptor) in.readObject();
					in.close();
				} catch (Exception ex) {
					LoggingUtil
							.println(IReporter.SYSTEM_NONFATAL,
									"registerAddedRulebases() - Failed to reconstruct descriptor object");
				}
				URL[] urls = new URL[1];
				URLClassLoader loader = null;
				int i = descriptorFilename.lastIndexOf("\\");
				String saveLocation = descriptorFilename.substring(0, i);
				File dir = new File(saveLocation);

				try {
					urls[0] = dir.toURL();
					loader = new URLClassLoader(urls, getClass()
							.getClassLoader());

				} catch (MalformedURLException e) {
					LoggingUtil
							.println(IReporter.SYSTEM_NONFATAL,
									"registerAddedRulebases() - Error adding the rulebase - URLClassLoader.");
				}
				// Need to check for its existence first
				// causes failure if there are duplicates
				if (loader != null) {
					try {
						ClassLoaderCache.getDefault().put("URLClassLoader",
								loader);
					} catch (IllegalArgumentException e) {
						LoggingUtil
								.println(IReporter.SYSTEM_NONFATAL,
										"registerAddedRulebases() - URLClassloader already registered");
					}
				}
				RuleBaseManager rbMgr = RuleBaseManager.getInstance();
				if (rbMgr != null && desc != null) {
					rbMgr.addRuleBase(desc);
				} else {
					LoggingUtil
							.println(IReporter.SYSTEM_NONFATAL,
									"registerAddedRulebases() - Failed to add rulebase");
				}

			}

		}

	}
}