/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  IBM Corporation - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.ui.dialogs;

import org.eclipse.actf.validation.reports.ValidationReportRecord;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;


public abstract class BaseRecordDetailsDialog extends Dialog
{

	public static final String REPORT_TITLE = "Validation Record Details";

	public static final String REPORT_CLASS_NAME = "Class &name:";

	public static final String REPORT_COMPONENT_LABEL = "&Label:";

	public static final String REPORT_COMPONENT = "&Component:";

	public static final String REPORT_COMPONENT_ID = "Component &ID:";

	public static final String REPORT_COMPONENT_PATH = "&Path to component";

	public static final String REPORT_SEVERITY = "&Severity:";

	public static final String REPORT_MESSAGE = "&Message:";

	public static final String REPORT_DETAIL_MESSAGE = "&Detailed message:";

	public static final String REPORT_TIME_STAMP = "&Time stamp:";

	public static final String REPORT_CONTEXT_TRACE = "Component creation &trace";

	private final String title;

	private final Image image;

	protected final ValidationReportRecord record;

	private Button detailsButton;

	private Control recordDetails;

	private Point savedShellSize;

	public BaseRecordDetailsDialog (Shell parent, String title, Image image,
									ValidationReportRecord record) {
		super(parent);
		this.title = title;
		this.image = image;
		this.record = record;
		setShellStyle(SWT.DIALOG_TRIM | SWT.RESIZE);
	}

	protected void buttonPressed (int id) {
		if (id == IDialogConstants.DETAILS_ID) {
			toggleRecordDetails();
		}else {
			super.buttonPressed(id);
		}
	}

	protected void configureShell (Shell shell) {
		super.configureShell(shell);
		if (title != null) {
			shell.setText(title);
		}
	}

	protected void createButtonsForButtonBar (Composite parent) {
		createButton(
			parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, false);
		detailsButton = createButton(
			parent, IDialogConstants.DETAILS_ID,
			IDialogConstants.SHOW_DETAILS_LABEL, false);
	}

	protected Control createDialogArea (Composite parent) {
		Composite composite = (Composite) super.createDialogArea(parent);
		composite.setLayoutData(new GridData());
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		layout.marginHeight = convertHorizontalDLUsToPixels(IDialogConstants.VERTICAL_MARGIN);
		layout.marginWidth = convertHorizontalDLUsToPixels(IDialogConstants.HORIZONTAL_MARGIN);
		composite.setLayout(layout);
		if (image != null) {
			Label label = new Label(composite, 0);
			image.setBackground(label.getBackground());
			label.setImage(image);
			label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_CENTER
					| GridData.VERTICAL_ALIGN_BEGINNING));
		}
		String component = "";
		String compName = record.getComponentName();
		String className = record.getNodeName();
		if (compName != null && compName.trim().length() > 0
				&& !compName.equals("<None>")) {
			component = className + ":" + compName;
		}else {
			component = className;
		}
		new Label(composite, SWT.NONE).setText(REPORT_COMPONENT);
		Text compText = new Text(composite, SWT.NONE | SWT.READ_ONLY);
		compText.setText(component);
		new Label(composite, SWT.NONE).setText(REPORT_SEVERITY);
		String level = record.getLevelAsString();
		new Text(composite, SWT.NONE | SWT.READ_ONLY).setText(level);
		new Label(composite, SWT.NONE).setText(REPORT_MESSAGE);
		new Text(composite, SWT.NONE | SWT.WRAP | SWT.READ_ONLY).setText(record.getMessage());
		GridData data = new GridData(GridData.FILL_HORIZONTAL
				| GridData.VERTICAL_ALIGN_CENTER);
		data.widthHint = 500;
		composite.setLayoutData(data);
		return composite;
	}

	protected void toggleRecordDetails () {
		Point priorShellSize = getShell().getSize();
		Point shellSize = savedShellSize;
		savedShellSize = priorShellSize;
		// get the top level control for this window. The parent of this control is the shell.
		Control topLevelControl = getContents();
		// Expand or contract the record details area
		if (recordDetails == null) {
			recordDetails = createRecordDetails((Composite) topLevelControl);
			// Set the button to contract the record details
			detailsButton.setText(IDialogConstants.HIDE_DETAILS_LABEL);
		}else {
			recordDetails.dispose();
			recordDetails = null;
			// Set the button to expand the record details
			detailsButton.setText(IDialogConstants.SHOW_DETAILS_LABEL);
		}
		// Compute the size of the top level control. The first time
		// the dialog is launched the shellSize will be null.
		if (shellSize == null) {
			Point oldSize = topLevelControl.getSize();
			Point newSize = topLevelControl.computeSize(
				SWT.DEFAULT, SWT.DEFAULT);
			shellSize = new Point(priorShellSize.x, priorShellSize.y
					+ (newSize.y - oldSize.y));
		}
		// Crop it to the screen.
		Point location = getShell().getLocation();
		Rectangle screenArea = getContents().getDisplay().getClientArea();
		if (shellSize.y > screenArea.height - (location.y - screenArea.y)) {
			shellSize.y = screenArea.height - (location.y - screenArea.y);
		}
		// Change the shell size and tell the control to layout the children.
		getShell().setSize(shellSize);
		((Composite) topLevelControl).layout();
	}

	protected abstract Control createRecordDetails (Composite parent);
}
