/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  IBM Corporation - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.ui.dialogs;

import org.eclipse.actf.validation.ui.GuiUtils;
import org.eclipse.actf.validation.ui.ValidationUIConstants;
import org.eclipse.actf.validation.ui.ValidationUiPlugin;
import org.eclipse.actf.validation.ui.filters.ReportTableViewerSorter;
import org.eclipse.actf.validation.ui.views.ReportView;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.dialogs.IMessageProvider;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Shell;


public class ReportViewSortDialog extends TitleAreaDialog
{

	ReportView view = null;

	private int sortBy;

	private boolean ascending = false;

	//	 controls
	private Group sortGroup = null;

	private Button severityButton = null;

	private Button idButton = null;

	private Button messageButton = null;

	private Button compButton = null;

	private Group orderGroup = null;

	private Button ascendingOrderButton = null;

	private Button descendingOrderButton = null;

	private Button restoreDefaultsButton = null;

	public ReportViewSortDialog (Shell parentShell, ReportView rv) {
		super(parentShell);
		view = rv;
	}

	protected Control createContents (Composite parent) {
		Control contents = super.createContents(parent);
		setMessage(
			"Select the sort criteria for the view.",
			IMessageProvider.INFORMATION);
		setTitle("Report Record Sorting");
		return contents;
	}

	protected Control createDialogArea (Composite parent) {
		initializeDialogUnits(parent);
		Composite area = (Composite) super.createDialogArea(parent);
		GridLayout layout = new GridLayout();
		area.setLayout(layout);
		createSortGroup(area);
		createOrderGroup(area);
		createDefaultsButton(area);
		loadSettings();
		return area;
	}

	private void createSortGroup (Composite parent) {
		sortGroup = new Group(parent, SWT.NULL);
		sortGroup.setText("Sort by:");
		GridData gd = new GridData(GridData.FILL_BOTH);
		sortGroup.setLayoutData(gd);
		GridLayout layout = new GridLayout(1, true);
		sortGroup.setLayout(layout);
		severityButton = new Button(sortGroup, SWT.RADIO);
		GuiUtils.configureControl(
			severityButton, "sevRadio", "Sort by severity", "Severity");
		GuiUtils.configureCommand(severityButton, new Runnable() {

			public void run () {
				sortBy = ReportTableViewerSorter.SEVERITY;
			}
		});
		messageButton = new Button(sortGroup, SWT.RADIO);
		GuiUtils.configureControl(
			messageButton, "messageRadio", "Sort by message", "Message");
		GuiUtils.configureCommand(messageButton, new Runnable() {

			public void run () {
				sortBy = ReportTableViewerSorter.DESCRIPTION;
			}
		});
		compButton = new Button(sortGroup, SWT.RADIO);
		GuiUtils.configureControl(
			compButton, "compRadio", "Sort by component", "Component");
		GuiUtils.configureCommand(compButton, new Runnable() {

			public void run () {
				sortBy = ReportTableViewerSorter.TYPE;
			}
		});
		idButton = new Button(sortGroup, SWT.RADIO);
		GuiUtils.configureControl(
			idButton, "idRadio", "Sort by the object ID of the component", "ID");
		GuiUtils.configureCommand(idButton, new Runnable() {

			public void run () {
				sortBy = ReportTableViewerSorter.ID;
			}
		});
	}

	private void createOrderGroup (Composite parent) {
		orderGroup = new Group(parent, SWT.NULL);
		orderGroup.setText("Sort order is:");
		GridData gd = new GridData(GridData.FILL_BOTH);
		orderGroup.setLayoutData(gd);
		GridLayout layout = new GridLayout(2, true);
		orderGroup.setLayout(layout);
		ascendingOrderButton = new Button(orderGroup, SWT.RADIO);
		GuiUtils.configureControl(
			ascendingOrderButton, "ascendingOrderRadio",
			"Sort in ascending order", "Ascending");
		GuiUtils.configureCommand(ascendingOrderButton, new Runnable() {

			public void run () {
				ascending = true;
			}
		});
		descendingOrderButton = new Button(orderGroup, SWT.RADIO);
		GuiUtils.configureControl(
			descendingOrderButton, "descendingOrderRadio",
			"Sort in descending order", "Descending");
		GuiUtils.configureCommand(descendingOrderButton, new Runnable() {

			public void run () {
				ascending = false;
			}
		});
	}

	protected void createDefaultsButton (Composite parent) {
		restoreDefaultsButton = new Button(parent, SWT.PUSH);
		GridData gd = new GridData();
		gd.horizontalAlignment = SWT.RIGHT;
		restoreDefaultsButton.setLayoutData(gd);
		GuiUtils.configureControl(
			restoreDefaultsButton, "restoreDefaults",
			"Restore default selections", "Restore Defaults");
		SelectionListener listener = new SelectionAdapter() {

			public void widgetSelected (SelectionEvent e) {
				setDefaults();
			}
		};
		restoreDefaultsButton.addSelectionListener(listener);
	}

	protected void okPressed () {
		saveSettings();
		view.adjustSortCriteria(sortBy, ascending);
		super.okPressed();
	}

	private void saveSettings () {
		IDialogSettings settings = ValidationUiPlugin.getDefault().getDialogSettings();
		try {
			settings.put(ValidationUIConstants.KEY_REPORT_RECORD_SORT_BY, sortBy);
			settings.put(
				ValidationUIConstants.KEY_REPORT_RECORD_SORT_ORDER, ascending);
		}catch (Exception e) {
			GuiUtils.reportException("Error saving dialog settings", e);
		}
	}

	protected void loadSettings () {
		IDialogSettings settings = ValidationUiPlugin.getDefault().getDialogSettings();
		try {
			sortBy = settings.getInt(ValidationUIConstants.KEY_REPORT_RECORD_SORT_BY);
			ascending = settings.getBoolean(ValidationUIConstants.KEY_REPORT_RECORD_SORT_ORDER);
		}catch (Exception e) { //if any of these come back with no setting, use the default for all
			setDefaults();
		}
		setControls();
	}

	protected void setDefaults () {
		// sortBy default is severity
		sortBy = ReportTableViewerSorter.SEVERITY;
		//sort order default is descending
		ascending = false;
		setControls();
	}

	private void setControls () {
		switch (sortBy) {
		case ReportTableViewerSorter.ID:
			idButton.setSelection(true);
			break;
		case ReportTableViewerSorter.TYPE:
			compButton.setSelection(true);
			break;
		case ReportTableViewerSorter.DESCRIPTION:
			messageButton.setSelection(true);
			break;
		case ReportTableViewerSorter.SEVERITY:
			severityButton.setSelection(true);
			break;
		}
		if (ascending) {
			ascendingOrderButton.setSelection(true);
		}else {
			descendingOrderButton.setSelection(true);
		}
	}
}
