/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  IBM Corporation - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.ui.preferences;

import org.eclipse.actf.ui.preferences.AbstractBasePreferencePage;
import org.eclipse.actf.validation.ui.ValidationUiPlugin;
import org.eclipse.jface.preference.DirectoryFieldEditor;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;


/**
 * 
 * @author John Buslawski
 * 
 */
public class PathPreferencePage extends AbstractBasePreferencePage implements
		IWorkbenchPreferencePage {

	private DirectoryFieldEditor workingDirEditor;

	private PathEditor interpEditor;

	private PathEditor libraryEditor;

	private PathEditor osEditor;

	public PathPreferencePage() {
		super();
		setPreferenceStore(ValidationUiPlugin.getDefault().getPreferenceStore());
	}

	protected Control createContents(Composite parent) {
		TabFolder folder = new TabFolder(parent, SWT.NONE);
		folder.setLayout(new TabFolderLayout());
		folder.setLayoutData(new GridData(GridData.FILL_BOTH));
		TabItem item = new TabItem(folder, SWT.NONE);
		item.setText("Environment");
		item.setControl(createEnvironmentTab(folder));
		item = new TabItem(folder, SWT.NONE);
		item.setText("Interpreters");
		item.setControl(createInterpreterTab(folder));
		item = new TabItem(folder, SWT.NONE);
		item.setText("Libraries");
		item.setControl(createLibraryTab(folder));
		item = new TabItem(folder, SWT.NONE);
		item.setText("Open Source");
		item.setControl(createOpenSourceTab(folder));
		initialize();
		return folder;
	}

	public void init(IWorkbench workbench) {
		// TODO Auto-generated method stub
	}

	private Control createEnvironmentTab(Composite parent) {
		GridData gd = null;
		Composite envComposite = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		envComposite.setLayout(layout);
		gd = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		envComposite.setLayoutData(gd);

		final AbstractBasePreferencePage pathPage = this;

		workingDirEditor = new DirectoryFieldEditor(
				ValidationPreferenceInitializer.P_WORKINGDIR,
				"ACTF Working Directory:", envComposite);
		// set this field editor so that only way to enter a value into this
		// field is thru the browse button
		Text workingDir = workingDirEditor.getTextControl(envComposite);
		workingDir.setEditable(false);
		workingDirEditor
				.setPropertyChangeListener(new IPropertyChangeListener() {

					public void propertyChange(PropertyChangeEvent propEvent) {
						if (!workingDirEditor.isValid()) {
							// set the error message
							workingDirEditor
									.setErrorMessage("Invalid working directory.  Please select another.");
							pathPage
									.setErrorMessage("Invalid working directory.  Please select another.");
							pathPage.setValid(false);
						} else {
							// reset the error message
							workingDirEditor.setErrorMessage(null);
						}
					}
				});
		return envComposite;
	}

	private Control createInterpreterTab(Composite parent) {
		Composite interpComposite = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		interpComposite.setLayout(layout);
		interpEditor = new PathEditor(
				ValidationPreferenceInitializer.P_SCRIPTCLASSPATH,
				"Script Interpreter JAR Paths", interpComposite);
		return interpComposite;
	}

	private Control createLibraryTab(Composite parent) {
		Composite libraryComposite = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		libraryComposite.setLayout(layout);
		libraryEditor = new PathEditor(ValidationPreferenceInitializer.P_LIBPATH,
				"Library Path", libraryComposite);
		return libraryComposite;
	} // end--createLibraryTab

	private Control createOpenSourceTab(Composite parent) {
		Composite osComposite = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		osComposite.setLayout(layout);
		osEditor = new PathEditor(ValidationPreferenceInitializer.P_OSCLASSPATH,
				"Open Source JAR Paths", osComposite);
		return osComposite;
	} // end--createOpenSourceTab

	private void initialize() {
		initializeEnvironment();
		// Add the initial content to the interpretor editor
		initializeEditor(this, interpEditor, getPreferenceStore());
		// Add the initial content to the library editor
		initializeEditor(this, libraryEditor, getPreferenceStore());
		// Add the initial content to the open source editor
		initializeEditor(this, osEditor, getPreferenceStore());
	} // end--initialize

	private void initializeEnvironment() {
//		 TODO
		// The following line is a Eclipse 3.1 API only. It is commented out because it breaks Eclipse 3.0.
		// When we no longer support Eclipse 3.0 we should uncomment this line.
		//editor.setPage(prefPage);
		//workingDirEditor.setPage(this);
		workingDirEditor.setPreferenceStore(getPreferenceStore());
		workingDirEditor.load();
		workingDirEditor.setEmptyStringAllowed(false);
		workingDirEditor
				.setValidateStrategy(StringFieldEditor.VALIDATE_ON_FOCUS_LOST);
		if (!workingDirEditor.isValid()) {
			// reset the error message
			workingDirEditor.setErrorMessage("Invalid working directory.");
			setErrorMessage("Invalid working directory.");
			setValid(false);
		} else {
			workingDirEditor.setErrorMessage(null);
		}

	}

	protected void performDefaults() {
		performEnvironmentDefaults();
		performInterpreterDefaults();
		performLibraryDefaults();
		performOpenSourceDefaults();
		super.performDefaults();
	} // end--performDefaults

	protected void performEnvironmentDefaults() {
		workingDirEditor.loadDefault();
	} // end--performEnrionmentDefaults

	protected void performInterpreterDefaults() {
		interpEditor.loadDefault();
	}

	protected void performLibraryDefaults() {
		libraryEditor.loadDefault();
	}

	protected void performOpenSourceDefaults() {
		osEditor.loadDefault();
	}

	public boolean performOk() {
		boolean validFields = true;
		performEnvironmentOk();
		performInterpreterOk();
		performLibraryOk();
		performOpenSourceOk();
		// if the fields are valid then check the super class
		// otherwise return not valid
		if (validFields) {
			validFields = super.performOk();
		}
		return validFields;
	} // end--peformOk

	protected void performEnvironmentOk() {
		workingDirEditor.store();
	}

	protected void performInterpreterOk() {
		interpEditor.store();
	}

	protected void performLibraryOk() {
		libraryEditor.store();
	}

	protected void performOpenSourceOk() {
		osEditor.store();
	}
} // end--PathPreferencesPage
