/*******************************************************************************
 * Copyright (c) 2004, 2008 IBM Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  IBM Corporation - initial API and implementation
 *  Kentarou FUKUDA - [232278]
 *******************************************************************************/

package org.eclipse.actf.validation.ui.preferences;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;

import org.eclipse.actf.core.ActfCorePlugin;
import org.eclipse.actf.core.config.ConfigurationException;
import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.core.runtime.IRuntimeContext;
import org.eclipse.actf.core.runtime.RuntimeContextFactory;
import org.eclipse.actf.model.ModelPlugin;
import org.eclipse.actf.util.FileUtils;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.resources.EclipseResourceLocator;
import org.eclipse.actf.validation.ValidationConstants;
import org.eclipse.actf.validation.ValidationCorePlugin;
import org.eclipse.actf.validation.rulebase.IDocumentRuleBase;
import org.eclipse.actf.validation.ui.ValidationUiPlugin;
import org.eclipse.actf.validation.valdoc.codeproc.ICodeProcessor;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.preferences.AbstractPreferenceInitializer;
import org.eclipse.core.runtime.preferences.DefaultScope;
import org.eclipse.core.runtime.preferences.IEclipsePreferences;
import org.eclipse.jface.resource.StringConverter;
import org.eclipse.swt.graphics.RGB;
import org.osgi.framework.Bundle;

public class ValidationPreferenceInitializer extends AbstractPreferenceInitializer {

	private static final String[] BUNDLE_NAMES = {
			ActfCorePlugin.ACTFCORE_PLUGIN_ID, ModelPlugin.MODEL_PLUGIN_ID,
			ValidationCorePlugin.VALIDATION_CORE_PLUGIN_ID,

			// TODO should be added dynamically
			"org.eclipse.actf.accservice",
			"org.eclipse.actf.model.javapp",
			"org.eclipse.actf.javaco.core",

			// TODO move to web validation
			"org.mozilla.xpcom" };

	// keys should be prefaced with their appropriate pool id
	// Configuration.ACTF_ID is default
	public static final String POOL_ID_DELIMITER = "$";

	/* keys for this preference page */
	public static final String P_WORKINGDIR = IConfiguration.WORKING_DIR_KEY;
	public static final String P_COLOR_INFO = "colorInfoPreference";
	public static final String P_COLOR_WARNING = "colorWarningPreference";
	public static final String P_COLOR_ERROR = "colorErrorPreference";
	public static final String P_COLORS_NORMAL = "colorsNormalPreference";
	public static final String P_USE_MARKER_COLOR = "colorsUseMarkerPreference";
	public static final String P_MARKER_BG_COLOR = "colorsMarkerBGPreference";
	public static final String P_MARKER_OUTLINE_COLOR = "colorsMarkerOLPreference";

	public static final String P_VALIDATION_LEVEL = ValidationConstants.VALIDATION_ID
			+ POOL_ID_DELIMITER + ValidationConstants.REPORTER_LEVEL_KEY;

	/* validation error level keys */

	public static final String V_NONE = "SEVERE";
	public static final String V_ERROR = "ERROR";
	public static final String V_WARNING = "WARNING";
	public static final String V_INFO = "INFO";
	public static final String V_ALL = "DETAIL";

	public static final int NONE = IReporter.SEVERE;
	public static final int ERRORS = IReporter.ERROR;
	public static final int WARNINGS = IReporter.WARNING;
	public static final int PROCESS_INFO = IReporter.INFO;
	public static final int PROCESS_ALL = IReporter.CONFIG;

	public static final String P_MODELS = IConfiguration.MODEL_ID;

	/* validation report-related keys */

	public static final String P_VAL_DOC = "valDocPreference";

	public static final String P_VAL_DOC_ENABLE = "valDocEnablePreference";

	/* path-related keys */
	public static final String P_LIBPATH = ValidationConstants.ACTF_LIB_PATH_KEY;

	public static final String P_ACTFPATH = ValidationConstants.ACTF_CLASSPATH_KEY;

	public static final String P_OSCLASSPATH = ValidationConstants.OS_CLASSPATH_KEY;

	public static final String P_SCRIPTCLASSPATH = ValidationConstants.SCRIPTLANG_CLASSPATH_KEY;

	public static final String P_JAVACLASSPATH = ValidationConstants.JAVA_CLASSPATH_KEY;

	/* keys for model-related preference pages */
	public static final String P_ALIASES = IDocumentRuleBase.ALIASES_ID;

	/* keys for <code>LanguagesPreferencePage</code> */
	public static final String P_SCRIPTLANGS = ICodeProcessor.SCRIPTLANG_ID;

	/* keys for <code>JythonLanguagePreferencePage</code> */
	public static final String P_CODE_PROCESSOR_CLASSPATH = ValidationConstants.CODE_PROCESSOR_CLASSPATH_KEY;

	public static final String OSGI_CLASSLOADER_DEFAULT = "app";
	public static final String OSGI_LOGFILE_DEFAULT = "osgi.log";

	public static final String[][] traceChoices = new String[][] {
			{ "None", V_NONE }, { "Errors Only", V_ERROR },
			{ "Errors and Warnings", V_WARNING },
			{ "Errors, Warnings and Information", V_INFO },
			{ "Debug Information", V_ALL } };

	public static String DELIMITER = "|";
	public static String PROPDELIMITER = " = ";

	protected static IConfiguration config = null;
	protected static IEclipsePreferences preferencesNode = null;
	protected static String tmpdir = null;
	protected static boolean basePreferencesLoaded = false;

	public static final String P_FORM_CONTEXTS = "formCreationContexts";

	public static final String P_NEW_RULEBASES = "addedRulebases";
	public ValidationPreferenceInitializer() {
		super();
	}

	public void initializeDefaultPreferences() {

		initalizeStaticMembers();

		if (!basePreferencesLoaded) {
			initializeValidationDefaultPreferences();
			basePreferencesLoaded = true;
		}
	}

	private void initalizeStaticMembers() {
		if (config == null) {
			try {

				IRuntimeContext context = RuntimeContextFactory.getInstance()
						.getRuntimeContext();
				config = context.getConfiguration();
			} catch (ConfigurationException e) {
				throw new RuntimeException(e.getMessage(), e);
			}
		}

		if (preferencesNode == null) {
			ValidationUiPlugin plugin = ValidationUiPlugin.getDefault();
			preferencesNode = new DefaultScope().getNode(plugin.getPluginId());
		}

		if (tmpdir == null) {
			tmpdir = FileUtils.getActfTempDir();
			File actfTmpDir = new File(tmpdir);
			if (!actfTmpDir.exists()) {
				try {
					actfTmpDir.mkdir();
				} catch (Exception e) {
					tmpdir = System.getProperty("java.io.tmpdir");
				}
			}
		}
	}

	private void initializeValidationDefaultPreferences() {

		preferencesNode.put(P_COLOR_INFO, StringConverter.asString(
				new RGB(255, 255, 240))); //ivory = 255 255 240
							// white = 255, 255, 255 
							//light grey = 211, 211, 211
		preferencesNode.put(P_COLOR_WARNING, StringConverter.asString(new RGB(
				255, 255, 175))); //yellow
		preferencesNode.put(P_COLOR_ERROR, StringConverter.asString(new RGB(
				240, 200, 200))); //red
		preferencesNode.putBoolean(P_COLORS_NORMAL, false);
		preferencesNode.put(P_MARKER_BG_COLOR, StringConverter
				.asString(new RGB(255, 200, 220)));
		preferencesNode.put(P_MARKER_OUTLINE_COLOR, StringConverter
				.asString(new RGB(255, 0, 0)));
		preferencesNode.putBoolean(P_USE_MARKER_COLOR, true);
		config.setSymbolPool(IConfiguration.ACTF_ID);
		// TODO replace with actf classpath
		String actfJar = getActfCoreJarPath();
		preferencesNode.put(P_ACTFPATH, actfJar);
		config.setParameter(ValidationConstants.ACTF_CLASSPATH_KEY,
				preferencesNode.get(P_ACTFPATH, ""));

		String paths = getOpenSourcePaths();
		StringBuffer openSourcePaths = new StringBuffer(paths);
		preferencesNode.put(P_OSCLASSPATH, openSourcePaths.toString());
		config.setParameter(ValidationConstants.OS_CLASSPATH_KEY,
				preferencesNode.get(P_OSCLASSPATH, ""));
		preferencesNode.put(P_SCRIPTCLASSPATH, getDefaultScriptLangPath());
		config.setParameter(ValidationConstants.SCRIPTLANG_CLASSPATH_KEY,
				preferencesNode.get(P_SCRIPTCLASSPATH, ""));
		preferencesNode.put(P_WORKINGDIR, tmpdir == null ? "/aop_workdir"
				: tmpdir);
		config.setParameter(IConfiguration.WORKING_DIR_KEY, preferencesNode
				.get(P_WORKINGDIR, ""));
		preferencesNode.put(P_LIBPATH, getDefaultLibraryPath());
		config.setParameter(ValidationConstants.ACTF_LIB_PATH_KEY,
				preferencesNode.get(P_LIBPATH, ""));
		preferencesNode.put(P_JAVACLASSPATH, "");
		preferencesNode.put(P_CODE_PROCESSOR_CLASSPATH, actfJar);
		config.setParameter(ValidationConstants.CODE_PROCESSOR_CLASSPATH_KEY,
				actfJar);

		config.setParameter(IConfiguration.SHOW_ITEMS_KEY, true);
		config.setParameter(IConfiguration.SHOW_INVISIBLE_KEY, true);
		config.setParameter(IConfiguration.TRACE_LEVEL_KEY, IReporter.CONFIG);

		config.setSymbolPool(ValidationConstants.VALIDATION_ID);

		preferencesNode.put(P_VALIDATION_LEVEL, V_INFO);
		preferencesNode.put(P_MODELS, ValidationPreferenceInitializer
				.toStringList(config.getModelTypes()));
		preferencesNode.put(P_SCRIPTLANGS, ValidationPreferenceInitializer
				.toStringList(config.getSymbolPoolContents(P_SCRIPTLANGS)));
		preferencesNode.put(P_NEW_RULEBASES, "");
	}

	public static int mapToLevel(String levelKey) {
		int level = WARNINGS;
		if (levelKey == null) {
			level = WARNINGS;
		} else if (levelKey.equals(V_ALL)) {
			level = PROCESS_ALL;
		} else if (levelKey.equals(V_INFO)) {
			level = PROCESS_INFO;
		} else if (levelKey.equals(V_WARNING)) {
			level = WARNINGS;
		} else if (levelKey.equals(V_ERROR)) {
			level = ERRORS;
		} else if (levelKey.equals(V_NONE)) {
			level = NONE;
		}
		return level;
	}

	private String getDefaultScriptLangPath() {
		return "";
	}

	public String getOpenSourcePaths() {
		final String[] osBundleNames = { "org.apache.bsf",
				"org.apache.commons.logging", "org.apache.commons.beanutils",
				"org.apache.commons.jxpath", "org.mozilla.javascript" };

		IRuntimeContext context = RuntimeContextFactory.getInstance().getRuntimeContext();
		StringBuffer buffer = new StringBuffer();
		
		for (int i = 0; i < osBundleNames.length; i++) {
			Bundle osBundle = Platform.getBundle(osBundleNames[i]);
			if (osBundle != null) {
				Enumeration en = osBundle.findEntries("/", "*.jar", true);
				while (en != null && en.hasMoreElements()) {
					try {
						String jarPath = Platform.resolve(
								(URL) en.nextElement()).getPath();
						if (buffer.length() > 0) {
							buffer.append(File.pathSeparator);
						}
						buffer.append(jarPath);
					} catch (IOException e) {
					}
				}

				if (en == null && osBundleNames[i] != null) {
					if (buffer.length() > 0) {
						buffer.append(File.pathSeparator);
					}
					buffer.append(((EclipseResourceLocator) context.getResourceLocator()).getPathToBundle(osBundleNames[i]));
				}
			}
		}

		return buffer.toString();
	}

	/*
	 * private String getActfCoreJarPath () {
	 * 
	 * StringBuffer buffer = new StringBuffer();
	 * 
	 * PathGenerationUtil gen = new PathGenerationUtil(); for (int b = 0; b <
	 * BUNDLE_NAMES.length; ++b) {
	 * 
	 * String path = gen.getJarPathsFromBundle(BUNDLE_NAMES[b]); if (path ==
	 * null) { continue; } if ( buffer.length() > 0 ) { buffer.append(
	 * File.pathSeparator ); } buffer.append( path ); }
	 * 
	 * return buffer.toString(); }
	 */

	private String getActfCoreJarPath() {
		IRuntimeContext context = RuntimeContextFactory.getInstance().getRuntimeContext();
		StringBuffer buffer = new StringBuffer();
		
		for (int b = 0; b < BUNDLE_NAMES.length; ++b) {
			Bundle bundle = Platform.getBundle(BUNDLE_NAMES[b]);
			if (bundle == null) {
				continue;
			}

			Enumeration en = bundle.findEntries("/", "*.jar", true);
			while (en != null && en.hasMoreElements()) {
				try {
					URL url = FileLocator.resolve((URL) en.nextElement());
					String urlPath = url.getPath();
					String jarPath = null;

					// TODO nested jars
					int bang = urlPath.indexOf('!');
					if (bang > 0) {
						// jar is nested so just use containing jar
						jarPath = urlPath.substring(1, bang);
					} else {
						jarPath = urlPath.substring(1);
					}
					if (buffer.length() > 0) {
						buffer.append(File.pathSeparator);
					}
					buffer.append(jarPath);
				} catch (IOException e) {
				}
			}

			// TODO hack from MAS since accservice main bundle is being skipped
			// by above algorithm
			if (bundle != null
					&& (BUNDLE_NAMES[b].equals("org.eclipse.actf.accservice") || en == null)) {
				if (buffer.length() > 0) {
					buffer.append(File.pathSeparator);
				}
				buffer.append(((EclipseResourceLocator) context.getResourceLocator()).getPathToBundle(BUNDLE_NAMES[b]));
			}
		}

		return buffer.toString();
	}

	private String getDefaultLibraryPath() {
		// TODO remove dependency on accservice; should not be hard-coded
		Bundle accBundle = Platform.getBundle("org.eclipse.actf.accservice");
		Enumeration en = accBundle.findEntries("/", "*.dll", true);
		StringBuffer buffer = new StringBuffer();

		while (en != null && en.hasMoreElements()) {
			try {
				String dllFilePath = Platform.resolve((URL) en.nextElement())
						.getPath().substring(1);
				String parentDir = dllFilePath.substring(0, dllFilePath
						.lastIndexOf('/'));
				if (buffer.length() > 0) {
					buffer.append(File.pathSeparator);
				}
				buffer.append(parentDir);
			} catch (IOException e) {
			}
		}
		return buffer.toString();
	}

	public static Map toMap(String list) {
		Map props = new HashMap();
		String propDelim = PROPDELIMITER;
		StringTokenizer st = new StringTokenizer(list, DELIMITER);
		while (st.hasMoreTokens()) {
			String prop = st.nextToken();
			int pos = prop.indexOf(propDelim);
			if (prop.length() > 0 && pos > 0) {
				props.put(prop.substring(0, pos), prop.substring(pos
						+ propDelim.length()));
			}
		}
		return props;
	}

	public static String toStringList(Map propMap) {
		StringBuffer sb = new StringBuffer();
		String[] entries = (String[]) propMap.keySet().toArray(
				new String[propMap.size()]);
		String propDelim = PROPDELIMITER;
		String delim = DELIMITER;
		Arrays.sort(entries);
		for (int e = 0; e < entries.length; ++e) {
			String key = entries[e].trim();
			String val = ((String) propMap.get(key)).trim();
			sb.append(key);
			sb.append(propDelim);
			sb.append(val);
			if (e < entries.length - 1) {
				sb.append(delim);
			}
		}
		return sb.toString();
	}

	public static String toStringList(String[] array) {
		StringBuffer sb = new StringBuffer();
		String delim = DELIMITER;
		Arrays.sort(array);
		for (int e = 0; e < array.length; ++e) {
			String key = array[e].trim();
			sb.append(key);
			if (e < array.length - 1) {
				sb.append(delim);
			}
		}
		return sb.toString();
	}

	protected void updateModelList() {
		config.setSymbolPool(ValidationConstants.VALIDATION_ID);
		preferencesNode.put(P_MODELS, ValidationPreferenceInitializer
				.toStringList(config.getModelTypes()));
	}
}
