/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  IBM Corporation - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.ui.views;

import java.sql.Time;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.eclipse.actf.validation.reports.ValidationReport;
import org.eclipse.actf.validation.reports.ValidationReportGroup;
import org.eclipse.actf.validation.reports.ValidationReportRecord;
import org.eclipse.actf.validation.ui.ValidationUIConstants;
import org.eclipse.actf.validation.ui.filters.ReportManagerViewerSorter;
import org.eclipse.actf.validation.ui.filters.ReportScopeFilter;
import org.eclipse.actf.validation.ui.providers.ReportManagerContentProvider;
import org.eclipse.actf.validation.ui.providers.ReportManagerLabelProvider;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;


public class ReportManagerView extends AbstractViewPart
{

	protected IStructuredSelection currentSelection;

	private ValidationReportGroup group = null;

	private TreeViewer viewer = null;

	private ReportManagerViewerSorter sorter = new ReportManagerViewerSorter();

	private ReportScopeFilter sFilter = new ReportScopeFilter();

	/**
	 * Create the  view
	 * @Override
	 */
	public void createPartControl (Composite parent) {
		super.createPartControl(parent);
		/** name format for ImageRegistry is  "key:imageName" */
		registerImages(new String[] {
				ValidationUIConstants.USER_ICON_ID + ":icons/userid_TB",
				ValidationUIConstants.REPORT_GROUP_ICON + ":icons/history_rep",
				ValidationUIConstants.REPORT_ICON_ID + ":icons/prop_ps",
				ValidationUIConstants.RECORD_ICON_ID + ":icons/record_details",
				ValidationUIConstants.DB_REPORT_ICON_ID + ":icons/memory_view"});
		
		viewer = new TreeViewer(parent, SWT.H_SCROLL | SWT.V_SCROLL | SWT.MULTI);
		viewer.setLabelProvider(new ReportManagerLabelProvider(this));
		viewer.setContentProvider(new ReportManagerContentProvider());
		sorter.setSortingField(ReportManagerViewerSorter.SOURCE, true);
		viewer.setSorter(sorter);
		setFiltersFromSettings();
		viewer.setInput(group = new ValidationReportGroup());
		initContextMenu(viewer);
		ISelectionProvider provider = viewer;
		provider.setSelection(null);
		provider.addSelectionChangedListener(new ISelectionChangedListener() {

			public void selectionChanged (SelectionChangedEvent event) {
				currentSelection = (IStructuredSelection) event.getSelection();
				if (currentSelection.size() == 1) {
					Object selected = currentSelection.getFirstElement();
					if (selected instanceof ValidationReport) {
						setReportViewInput((ValidationReport) selected);
					}else if (selected instanceof ValidationReportRecord) {
						// probably won't show report records here
						// report records of the selected report will be shown in
						// the Report View
					}
				}else if (currentSelection.size() == 0) {
					setReportViewInput(null);
				}
			}
		});
		
	}

	
	private void setFiltersFromSettings () {
		viewer.addFilter(sFilter);
	}


	/**
	 * return the title for the view
	 * @Override
	 */
	protected String getMessageTitle () {
		return "Report Manager View";
	}

	/**
	 * set the focus to the tree
	 * @Override
	 */
	public void setFocus () {
		viewer.getControl().setFocus();
	}

	/**
	 * Add a report to the view
	 * @param report   the report to add
	 * @param description  	a longer description of what the report represents 
	 * @param component the component (Class) upon report was run (i.e. SwingSet2)
	 * @param tool 	the tool which created the report (i.e. the Java Gui View)
	 * @param model	the GUI architechture inspected (i.e. SWT, Swing, or AWT)
	 */
	public void addReport (ValidationReport report, String description,
							String component, String tool, String model) {
		report.setReportID("none");
		report.setSource(component);
		report.setDescription(description);
		report.setTimestamp(new Time(System.currentTimeMillis()));
		report.setPersistent(false);
		report.setTool("unknown");
		report.setModel("unknown");
		group.addReport(report);
	}

	/**
	 * add a set of reports to the view
	 * @param reports   an array of Validation Reports
	 * @param description  	a longer description of what the report represents 
	 * @param source the component (Class) upon report was run (i.e. SwingSet2)
	 * @param toolId 	the tool which created the report (i.e. the Java Gui View)
	 * @param architecture	the GUI architechture inspected (i.e. SWT, Swing, or AWT)
	 * @return identifiers of created reports or null if no reports created
	 */
	public Object addReports (ValidationReport[] reports, String description,
								String source, String toolId,
								String architecture) {
		java.util.List result = null;
		// Consolidate the reports into one report
		ValidationReport consolidatedReport = new ValidationReport();
		for (int i = 0; i < reports.length; i++) {
			if (!reports[i].isEmpty()) {
				consolidatedReport.addRecords(reports[i].getRecords());
				Map creationCtxs = reports[i].getCreationContexts();
				consolidatedReport.addCreationContexts(creationCtxs);
			}
		}
		addReport(consolidatedReport, description, source, toolId, architecture);
		return result;
	}



	/*
	 * This method is called from an object contribution on a ValidationReport
	 * so it expects that the current selection consists of one or more report records
	 * and no other object types, though it is written as if there could be other selections.
	 * 
	 */
	public void deleteReport () {
		if (currentSelection != null && currentSelection.size() > 0) {
			ArrayList reports = new ArrayList();
			for (Iterator iter = currentSelection.iterator(); iter.hasNext();) {
				Object next = iter.next();
				if (next instanceof ValidationReport) {
					reports.add(next);
				}
			}
			if (reports.size() > 0) {
				boolean rc = MessageDialog.openQuestion(
					null,
					"Confirm Delete",
					reports.size() == 1 ? "Are you sure you want to delete the report?"
							: "Are you sure you want to delete "
									+ reports.size() + " reports");
				if (rc) {
					for (Iterator it = reports.iterator(); it.hasNext();) {
						ValidationReport report = (ValidationReport) it.next();
						int id = Integer.parseInt(report.getReportID());
						try {
							group.removeReport(report);
						}catch (Exception e) {
							reportError("Error occured while deleting report "
									+ id, e.getMessage());
						}
					}
					viewer.setInput(group);
					setReportViewInput(null);
				}
			}else { //instance of a report record
				plugin.logError("No report is selected");
			}
		}else { //no selection
			plugin.logError("No items are selected");
		}
	}

	public void refresh () {
		viewer.setInput(viewer.getInput());
	}


	public void setReportFilters (boolean inactive, int scope, String scopeValue) {
		sFilter.setScope(scope, scopeValue);
	}

	public void adjustSortCriteria (int sortBy, boolean direction) {
		sorter.setSortingField(sortBy, direction);
	}

	public Set queryTools () {
		Set s = new HashSet();
		ValidationReport[] reports = group.getReports();
		for (int i = 0; i < reports.length; i++) {
			ValidationReport rep = reports[i];
			String tool = rep.getTool();
			s.add(tool);
		}
		return s;
	}

	public Set querySources () {
		Set s = new HashSet();
		ValidationReport[] reports = group.getReports();
		for (int i = 0; i < reports.length; i++) {
			ValidationReport rep = reports[i];
			String source = rep.getSource();
			s.add(source);
		}
		return s;
	}

	private void toggleToolbarButton (String toolItemId, boolean selected) {
		IToolBarManager tbMgr = getViewSite().getActionBars().getToolBarManager();
		IContributionItem item = tbMgr.find(toolItemId);
		if (item instanceof ActionContributionItem) {
			ActionContributionItem aci = (ActionContributionItem) item;
			aci.getAction().setChecked(selected);
		}
	}

	
}
