/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
* IBM Corporation - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.ui.views;

import java.io.FileWriter;
import java.io.IOException;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;

import org.eclipse.actf.util.logging.LoggingUtil;
import org.eclipse.actf.validation.reports.CreationContext;
import org.eclipse.actf.validation.reports.ValidationReport;
import org.eclipse.actf.validation.reports.ValidationReportRecord;
import org.eclipse.actf.validation.ui.GuiUtils;
import org.eclipse.actf.validation.ui.ValidationUIConstants;
import org.eclipse.actf.validation.ui.filters.ReportTableViewerSorter;
import org.eclipse.actf.validation.ui.filters.SeverityFilter;
import org.eclipse.actf.validation.ui.providers.MessageTableContentProvider;
import org.eclipse.actf.validation.ui.providers.MessageTableLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IWorkbenchPart;

/**
 *  Used to display a Validation Report in a tabular format
 * @author annford
 *
 */
public class ReportView extends AbstractViewPart
{

	private TableViewer viewer = null;

	private ReportTableViewerSorter sorter = new ReportTableViewerSorter(this);

	private Label reportTitle = null;

	private ValidationReport currentReport = null;

	private final String noReport = "No report is selected or executing";

	// constants for exporting reports 
	private final String XML_HEADER_TEXT = "<?xml version=\"1.0\"?>\n\n<!-- Generated on "
			+ (new Date())
			+ " by "
			+ System.getProperty("user.name")
			+ " -->\n\n";

	private final String BEGIN_REPORTS_TAG = "<reports>";

	private final String END_REPORTS_TAG = "</reports>";

	private final String INDENTATION = "  ";

	// partListener to enable/disable toolbar buttons
	private IPartListener partListener = new PartAdapter() {

		public void partActivated (IWorkbenchPart part) {
			boolean enable = (viewer.getInput() != null);
			enableToolbarButton(
				ValidationUIConstants.REPORT_VIEW_CLIPBOARD_ACTION, enable);
			enableToolbarButton(
				ValidationUIConstants.REPORT_VIEW_EXPORT_ACTION, enable);
		}
	};

	public ReportView () {
		super();
	}

	/**
	 * Transform the report data to a plain text format
	 * @param records the ReportRecords contained in the current validation report.
	 * @return the report data as plain text
	 */
	private String asText (ValidationReportRecord[] records) {
		StringBuffer sb = new StringBuffer();
		sb.append("report timestamp=");
		sb.append(getReportTimestamp(currentReport));
		sb.append("\n");
		for (int r = 0; r < records.length; ++r) {
			String line = records[r].toString();
			int dot = line.indexOf('.');
			/*if (dot >= 0) {
				line = line.substring(dot + 1);
			}*/
			sb.append(line);
			sb.append("\n");
		}
		return sb.toString();
	}

	/**
	 * Transform the report data to an XML form report
	 * @param records the ReportRecords contained in the current validation report.
	 * @return the report in XML format
	 */
	private String asXml (ValidationReportRecord[] records) {
		StringBuffer sb = new StringBuffer();
		int indent = 1;
		sb.append(XML_HEADER_TEXT);
		sb.append(BEGIN_REPORTS_TAG + "\n");
		sb.append(indent(indent));
		sb.append("<report timestamp=");
		sb.append('\"');
		sb.append(getReportTimestamp(currentReport));
		sb.append("\">\n");
		++indent;
		for (int r = 0; r < records.length; ++r) {
			sb.append(indent(indent));
			sb.append("<violation class=");
			sb.append('\"');
			sb.append(records[r].getNodeName());
			sb.append('\"');
			sb.append(" severity=");
			sb.append('\"');
			sb.append(records[r].getLevelAsString());
			sb.append('\"');
			String compName = LoggingUtil.escape(records[r].getComponentName());
			if (compName != null && compName.trim().length() > 0
					&& !compName.equals("<None>")) {
				sb.append(" compname=");
				sb.append('\"');
				sb.append(compName);
				sb.append('\"');
			}
			sb.append(">\n");
			sb.append(indent(++indent));
			sb.append("<text>");
			// need to parse this for special symbols like <  
			// and substitute entities like &lt;
			String message = LoggingUtil.escape(records[r].getMessage());
			sb.append(message);
			sb.append("</text>\n");
			sb.append(indent(--indent));
			sb.append("</violation>\n");
		}
		sb.append(indent(--indent));
		sb.append("</report>\n");
		sb.append(END_REPORTS_TAG + "\n");
		return sb.toString();
	}

	/**
	 * Test to determine if the report is empty or not
	 * @return true if the validation report has data
	 */
	public boolean checkForContent () {
		ValidationReport report = (ValidationReport) viewer.getInput();
		if (report != null && !report.isEmpty()) {
			return true;
		}else {
			return false;
		}
	}

	/**
	 * this method constructs the view
	 */
	public void createPartControl (Composite parent) {
		super.createPartControl(parent);
		try {
			viewer = createReportTableView(parent);
			initContextMenu(viewer);
			getSite().getPage().addPartListener(partListener);
		}catch (Exception e) {
			plugin.logException("Cannot create report view", e);
		}
	}

	private void sizeReportTableViewerTable (Table t, Composite parent) {
		Point p = t.computeSize(SWT.DEFAULT, SWT.DEFAULT, true);
		Display d = parent.getDisplay();
		Rectangle r = d.getBounds();
		Point s = new Point(Math.min(r.width - 400, Math.max(p.x, 600)), Math.min(
			r.height - 200, Math.max(p.y, 200)));
		t.setSize(s);
		parent.layout();
	}

	private void updateReportTable (TableViewer tv, Object input) {
		if (tv != null) {
			tv.setInput(input);
			tv.setSorter(sorter);
			// tv.refresh();
		}
	}

	private TableViewer createReportTableView (Composite parent) {
		Composite comp = new Composite(parent, SWT.NONE);
		GridLayout gl = new GridLayout(1, false);
		comp.setLayout(gl);
		reportTitle = new Label(comp, SWT.NONE);
		reportTitle.setText(noReport);
		GridData gd1 = new GridData(GridData.FILL_HORIZONTAL);
		reportTitle.setLayoutData(gd1);
		Composite tcomp = new Composite(comp, SWT.NONE);
		GridData gd2 = new GridData(GridData.FILL_BOTH);
		gd2.grabExcessHorizontalSpace = true;
		gd2.grabExcessVerticalSpace = true;
		tcomp.setLayoutData(gd2);
		TableViewer tv = createReportTableViewer(tcomp);
		updateReportTable(tv, tv.getInput());
		Table t = tv.getTable();
		sizeReportTableViewerTable(t, parent);
		return tv;
	}

	private TableViewer createReportTableViewer (Composite parent) {
		final Table t = createReportTable(parent);
		parent.addControlListener(new TableViewerControlAdapter(t));
		createReportTableColumns(t);
		TableViewer tv = new TableViewer(t);
		configureReportTableViewer(tv);
		createReportTableViewerProviders(tv);
		tv.setSorter(new ViewerSorter());
		return tv;
	}

	private Table createReportTable (Composite parent) {
		GridData ttgd = new GridData(GridData.FILL_BOTH
				| GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL);
		Table t = new Table(parent, SWT.V_SCROLL | SWT.H_SCROLL | SWT.SINGLE
				| SWT.FULL_SELECTION);
		GuiUtils.configureControl(
			t, "reportTable", ttgd,
			"Displays report records in a table.  Select a record for more information.");
		t.setHeaderVisible(true);
		t.setLinesVisible(true);
		return t;
	}

	private void configureReportTableViewer (TableViewer tv) {
		tv.setColumnProperties(new String[] {
				"severity", "message", "component", "id"});
		tv.addFilter(new SeverityFilter());
	}

	private void createReportTableViewerProviders (TableViewer tv) {
		tv.setContentProvider(new MessageTableContentProvider());
		tv.setLabelProvider(new MessageTableLabelProvider(tv.getTable(), this, this));
	}

	private void createReportTableColumns (Table t) {
		TableColumn tscol = new TableColumn(t, SWT.LEFT);
		tscol.setText("Severity");
		tscol.setWidth(100); //150
		tscol.setResizable(true);
		tscol.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected (SelectionEvent e) {
				sorter.setSortingField(ReportTableViewerSorter.SEVERITY);
				viewer.refresh();
			}
		});
		TableColumn desccol = new TableColumn(t, SWT.LEFT);
		desccol.setText("Message");
		desccol.setWidth(250); //was 300
		desccol.setResizable(true);
		desccol.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected (SelectionEvent e) {
				sorter.setSortingField(ReportTableViewerSorter.DESCRIPTION);
				viewer.refresh();
			}
		});
		TableColumn typecol = new TableColumn(t, SWT.LEFT);
		typecol.setText("Component");
		typecol.setWidth(150); //100
		typecol.setResizable(true);
		typecol.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected (SelectionEvent e) {
				sorter.setSortingField(ReportTableViewerSorter.TYPE);
				viewer.refresh();
			}
		});
		TableColumn idcol = new TableColumn(t, SWT.LEFT);
		idcol.setText("Rule Id");
		idcol.setWidth(150); //100
		idcol.setResizable(true);
		idcol.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected (SelectionEvent e) {
				sorter.setSortingField(ReportTableViewerSorter.ID);
				viewer.refresh();
			}
		});
	}

	/**
	 * Export to a file the report data as plain text
	 * @param file to create
	 */
	public void exportText (String file) {
		ValidationReport report = (ValidationReport) viewer.getInput();
		if (report != null && !report.isEmpty()) {
			ValidationReportRecord[] records = report.getRecords();
			if (records.length > 0) {
				String output = asText(records);
				try {
					FileWriter out = new FileWriter(file);
					out.write(output);
					out.close();
					plugin.logInfo("Export successful");
				}catch (IOException e) {
					System.err.println(e);
					plugin.logError("Export failed");
				}
			}
		}else {
			plugin.logError("No report data to export");
		}
	}

	/**
	 * Export the report data to the clipboard as plain text
	 * @return the report data as plain text
	 */
	public String exportTextToClipboard () {
		ValidationReport report = (ValidationReport) viewer.getInput();
		String output = null;
		if (report != null && !report.isEmpty()) {
			ValidationReportRecord[] records = report.getRecords();
			if (records.length > 0) {
				output = asText(records);
			}else {
				plugin.logError("No report data to export");
			}
		}
		return output;
	}

	/**
	 * Export the report data to a file as XML
	 * @param file the filename to create
	 */
	public void exportXML (String file) {
		ValidationReport report = (ValidationReport) viewer.getInput();
		if (report != null && !report.isEmpty()) {
			ValidationReportRecord[] records = report.getRecords();
			if (records.length > 0) {
				String output = asXml(records);
				try {
					FileWriter out = new FileWriter(file);
					out.write(output);
					out.close();
					plugin.logInfo("Export successful");
				}catch (IOException e) {
					plugin.logException("Cannot export report", e);
					plugin.logError("Export failed");
				}
			}
		}else {
			plugin.logError("No report data to export");
		}
	}

	protected String getMessageTitle () {
		return "Validation Report View";
	}

	/**
	 *  Indentation utility for the XML report format
	 * @param level nesting level
	 * @return a string with the appropriate number of spaces  
	 */
	private String indent (int level) {
		StringBuffer sb = new StringBuffer();
		for (int l = 0; l < level; ++l) {
			sb.append(INDENTATION);
		}
		return sb.toString();
	}

	public void setFocus () {
		viewer.getControl().setFocus();
	}

	public void setInput (ValidationReport report) {
		if (viewer != null) {
			viewer.setInput(report);
		}
		if (reportTitle != null) {
			if (report != null) {
				String description = report.getDescription();
				String ts = getReportTimestamp(report);
				String summary = report.getSummary();
				reportTitle.setText(description + " - " + ts + " - " + summary);
				enableToolbarButton(
					ValidationUIConstants.REPORT_VIEW_CLIPBOARD_ACTION, true);
				enableToolbarButton(
					ValidationUIConstants.REPORT_VIEW_EXPORT_ACTION, true);
			}else {
				reportTitle.setText(noReport);
			}
		}
	}

	public CreationContext getCreationContext (String recordId) {
		CreationContext context = null;
		ValidationReport report = (ValidationReport) viewer.getInput();
		if (report != null && !report.isEmpty()) {
			HashMap contextMap = (HashMap) report.getCreationContexts();
			context = (CreationContext) contextMap.get(recordId);
		}
		return context;
	}

	public void updateView () {
		ViewerFilter[] filters = viewer.getFilters();
		for (int i = 0; i < filters.length; i++) {
			if (filters[i] instanceof SeverityFilter) {
				SeverityFilter filter = (SeverityFilter) filters[i];
				filter.getCurrentFilterSettings();
			}
		}
		viewer.setInput(viewer.getInput());
	}

	public void adjustSortCriteria (int sortBy, boolean direction) {
		sorter.setSortingField(sortBy, direction);
	}

	private String getReportTimestamp (ValidationReport report) {
		String ts = null;
		if (report != null) {
			if(report.getTimestamp()!=null){
				ts = report.getTimestamp().toString();
			}
		}
		if (ts == null) {
			ts = Calendar.getInstance().getTime().toString();
		}
		return ts;
	}

	public void dispose () {
		getSite().getPage().removePartListener(partListener);
		super.dispose();
	}
}
