/*******************************************************************************
 * Copyright (c) 2004, 2008 IBM Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html

* Contributors: Initial API - Ann Ford
*	
*******************************************************************************/
package org.eclipse.actf.validation.ui.wizards;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Map;
import java.util.Set;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;

import org.eclipse.actf.util.FileUtils;
import org.eclipse.actf.util.jar.ZipExploder;
import org.eclipse.actf.util.resources.ClassLoaderCache;
import org.eclipse.actf.validation.rulebase.DocumentRuleBaseDescriptor;
import org.eclipse.actf.validation.rulebase.RuleBaseManager;
import org.eclipse.actf.validation.ui.ValidationUiPlugin;
import org.eclipse.actf.validation.ui.preferences.ValidationPreferenceInitializer;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;

public class LoadRulebaseWizard extends Wizard {
	private RulebaseSourcePage sourcePage = null;
	private RulebaseLocalFilePage localFilePage = null;
	private RulebaseRemoteFilePage remoteFilePage = null;
	private RulebaseNameAndTypePage namePage = null;
	private TargetLocationPage targetLocationPage = null;
	private File sourceFile = null;
	private URL sourceURL = null;
	private File metadataDir = null;
	private File saveLocation = null;
	private String rulebaseName = null;
	private Set targetTypeSet = null;
	private boolean isFileLocal = false;

	private static final int BLOCK_SIZE = 512;

	private static final int BLOCK_COUNT = 20;

	// *** must be a multiple of BLOCK_SIZE ***
	public static int bufferSize = 128 * (2 * BLOCK_SIZE);

	public static final String COMMON_SCRIPT_NAME = "common_validation.js";

	private static ValidationUiPlugin plugin = (ValidationUiPlugin.getDefault());

	public LoadRulebaseWizard() {
		super();
		setWindowTitle("Add Rulebase Wizard");

	}

	@Override
	public void addPages() {
		// Note - order is important here. The order in which these are added
		// will be the order they appear in the wizard and determine navigation
		sourcePage = new RulebaseSourcePage();
		addPage(sourcePage);
		localFilePage = new RulebaseLocalFilePage();
		addPage(localFilePage);
		remoteFilePage = new RulebaseRemoteFilePage();
		addPage(remoteFilePage);
		targetLocationPage = new TargetLocationPage();
		addPage(targetLocationPage);
		namePage = new RulebaseNameAndTypePage();
		addPage(namePage);

	}

	@Override
	public boolean performFinish() {
		// TODO AKF - advise is to run this on another thread
		// getContainer().run(forkable, canceleable, runnable);

		// steps:
		// 1 - Copy the file and metadata (script files or whatever) to the
		// target location

		if (!fetchNewRulebaseFile()) {
			namePage.setErrorMessage("Unable to fetch the file.");
			return false;
		} else {

			// 2 - create a rulebaseDescriptor with the information from the
			// wizard pages

			DocumentRuleBaseDescriptor desc = new DocumentRuleBaseDescriptor();
			if (rulebaseName != null) {
				desc.setRuleBaseName(rulebaseName);
			} else {
				namePage.setErrorMessage("Rulebase name is null.");
				return false;
			}
			// Document is the only type supported for now but eventually needs
			// to be a user choice
			desc.setType(DocumentRuleBaseDescriptor.DOCUMENT);
			
			// The RuleBaseFactory should also be a user specifyable option but
			// for now we only have this one
			desc
					.setRuleBaseFactory("org.eclipse.actf.validation.valdoc.rulebase.DocumentRuleBaseBuilder");

			if (targetTypeSet != null) {
				desc.setTargets(targetTypeSet);
			} else {
				namePage.setErrorMessage("Target type is null.");
				return false;
			}

			if (sourceFile != null) {
				desc.setSourceLocation(sourceFile.getName());
			} else{
				namePage.setErrorMessage("Save location is null");
				return false;
			}

			// 3 - set a classloader so that the ResourceLocator can find it
			URL[] urls = new URL[1];
			URLClassLoader loader = null;
			try {
				urls[0] = saveLocation.toURL();
				loader = new URLClassLoader(urls, getClass().getClassLoader());
				ClassLoaderCache.getDefault().put("URLClassLoader", loader);
			} catch (MalformedURLException e) {
				namePage
						.setErrorMessage("Error adding the rulebase - URLClassLoader.");
				return false;
			}

			// 4 - Add it to the RuleBaseManager
			RuleBaseManager rbMgr = RuleBaseManager.getInstance();
			if (rbMgr != null) {
				rbMgr.addRuleBase(desc);
			} else {
				namePage.setErrorMessage("Error adding the rulebase.");
				return false;
			}

			// 5 - Make it persistent

			
			// a - Serialize descriptor out to a file in the save location

			FileOutputStream fos = null;
			ObjectOutputStream out = null;
			String descriptorFilename = saveLocation.getPath() + "\\"
					+ rulebaseName + "_descriptor.obj";
			try {
				fos = new FileOutputStream(descriptorFilename);
				out = new ObjectOutputStream(fos);
				out.writeObject(desc);
				out.close();
			} catch (IOException ex) {
				namePage.setErrorMessage("Unable to save rulebase descriptor");
				return false;
			}

			// b - get the Preference store for new rulebases

			IPreferenceStore store = plugin.getPreferenceStore();
			String rulebases = store
					.getString(ValidationPreferenceInitializer.P_NEW_RULEBASES);
			
			// c - add this new rulebase to the map
			// key = rulebaseName and value = saveLocation\+descriptor-file-name
			Map rulebaseMap;
			if (rulebases != null) {
				rulebaseMap = ValidationPreferenceInitializer.toMap(rulebases);
				if (rulebaseMap != null) {
					rulebaseMap.put(rulebaseName, descriptorFilename);
				}
			} else {
				namePage
						.setErrorMessage("Unable to get rulebases from preferences");
				return false;
			}
			// d - write Map back out to preference store
			rulebases = ValidationPreferenceInitializer
					.toStringList(rulebaseMap);
			store.setValue(ValidationPreferenceInitializer.P_NEW_RULEBASES,
					rulebases);
			plugin.savePluginPreferences();


			// 6 - refresh the view - happens in RulesMasterDetailBlock, Add
			// Button action if the dialog returns OK
		}
		return true;
	}

	@Override
	public boolean canFinish() {
		if (namePage.isPageComplete()) {
			return true;
		}
		return false;
	}

	public void setSourceFile(File file) {
		this.sourceFile = file;
	}

	public void setMetadataDir(File dir) {
		this.metadataDir = dir;
	}

	public void setSaveLocation(File saveDir) {
		this.saveLocation = saveDir;
	}

	public File getSourceFile() {
		return sourceFile;
	}

	public File getTargetLocation() {
		return saveLocation;
	}

	public boolean isFileLocal() {
		return isFileLocal;
	}

	public void setIsFileLocal(boolean isFileLocal) {
		this.isFileLocal = isFileLocal;
	}

	public boolean fetchNewRulebaseFile() {
		boolean fileCopied = false;
		if (isFileLocal) {
			if (sourceFile != null && saveLocation != null) {
				String targetPath = saveLocation.getPath() + "\\"
						+ sourceFile.getName();
				MessageBox mb = new MessageBox(getShell(), SWT.YES | SWT.NO);
				mb.setText("Copy files warning");
				mb
						.setMessage("Files in the target location will be overwritten.  Are you sure you wish to proceed?");
				int result = mb.open();
				if (result == SWT.YES) {
					fileCopied = FileUtils.copyFile(sourceFile, targetPath,
							true);
					if (metadataDir != null) {
						FileUtils.copyFiles(metadataDir, saveLocation, "*.js");
					}
				} else {
					return fileCopied = false;
				}
			}
		} else { // file is remote
			if (sourceFile == null) {
				sourceFile =  getRemoteRulebaseFile();
				if (sourceFile != null){
					fileCopied = true;
				}	
			} 
		}
		return fileCopied;
	}

	public void setRulebaseName(String text) {
		this.rulebaseName = text;
	}

	public void setTargetTypeExpression(Set regexSet) {
		this.targetTypeSet = regexSet;

	}

	public void setSourceURL(URL url) {
		this.sourceURL = url;

	}

	public URL getSourceURL() {
		return sourceURL;
	}

	public File getRemoteRulebaseFile() {
		File outfile = null;
		if (sourceURL != null && saveLocation != null) {
			File urlfile = getRemoteFile(sourceURL, saveLocation);
			if (urlfile.isFile()){
				String filename = urlfile.getName().trim();
				if (filename.endsWith(".xml")){
					outfile = urlfile;
				} else if (filename.endsWith(".zip")){
					try {
						ZipFile zipfile = new ZipFile(urlfile);
						ZipExploder exploder = new ZipExploder();
						exploder.processZips(new String [] {zipfile.getName()}, saveLocation.getPath());
						File [] ruleFiles = FileUtils.findFiles("*.xml", saveLocation);
						if (ruleFiles.length == 1){
							outfile = ruleFiles[0];
						} else {
							//TODO AKF more than one - what to do?
							// for now - return first one
							outfile = ruleFiles[0];
						}
						//TODO AKF should we delete the zip file?
					} catch (ZipException e) {
						return outfile = null;
					} catch (IOException e) {
						return outfile = null;
					}
				} 
			}
		}
		return outfile;

	}

	

	public File getRemoteFile(URL url, File saveDir) {
		File outfile = null;
		if (url != null && saveDir != null) {
			BufferedInputStream is = null;
			String fullpath = url.getFile();
			String path = null;
			String filename = null;
			if (fullpath != null && fullpath.length() > 0) {
				// /actf/CI162_mozdom_accessibility.xml
				int idx = fullpath.lastIndexOf("/");
				filename = fullpath.substring(idx + 1);
				outfile = new File(saveDir + "\\" + filename);
			}
			if (outfile != null) {
				try {
					is = new BufferedInputStream(url.openStream(), BLOCK_SIZE
							* BLOCK_COUNT);
					if (is != null) {
						BufferedOutputStream os = new BufferedOutputStream(
								new FileOutputStream(outfile), BLOCK_SIZE
										* BLOCK_COUNT);
						byte[] buf = new byte[bufferSize];
						for (int size = is.read(buf), count = 0; size >= 0; size = is
								.read(buf), count++) {
							os.write(buf, 0, size);
						}
						os.flush();
						os.close();
					}
					is.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
		return outfile;
	}
}
