/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html

* Contributors: Initial API - Ann Ford
*	
*******************************************************************************/
package org.eclipse.actf.validation.ui.wizards;

import java.io.File;

import org.eclipse.actf.validation.ui.GuiUtils;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

public class RulebaseLocalFilePage extends WizardPage {

	public static final String PAGE_NAME = "Local File";
	public static final String INVALID_FILE = "This is not a valid file. Please enter a valid filename.";
	public static final String INVALID_DIR = "This is not a valid directory.  Please enter a valid directory path";
	private Text sourceText = null;
	private File sourceFile = null;
	private Text metadataText = null;
	private File metadataDir = null;
	private Label metadataLabel = null;
	private Button metadataButton = null;
	private Button metadataInSamePlace;
	private boolean sourceLocationComplete = false;
	private boolean metadataLocationComplete = false;

	protected RulebaseLocalFilePage() {
		super(PAGE_NAME);
		setTitle("File Located on Local File System");
		setDescription("Choose the new rulebase file from somewhere on the local file system.");
	}

	public void createControl(Composite parent) {
		Composite composite = new Composite(parent, SWT.NONE);
		composite.setLayoutData(new GridData(GridData.FILL_BOTH));
		GridLayout layout = new GridLayout(3, false);
		composite.setLayout(layout);

		// Source File
		Label sourceLabel = new Label(composite, SWT.LEFT);
		sourceLabel.setText("Source File");
		sourceText = new Text(composite, SWT.SINGLE);
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		sourceText.setLayoutData(data);
		sourceText
				.setToolTipText("Enter a fully qualified or relative path to the new rulebase file or use the Browse button to select one");
		sourceText.addModifyListener(new ModifyListener() {

			public void modifyText(ModifyEvent e) {
				if (sourceText != null) {
					String text = sourceText.getText();
					if (text != null && text.trim().length() > 0) {
						sourceLocationComplete = true;
						getWizard().getContainer().updateButtons();
					}
				}

			}

		});
		Button sourceButton = new Button(composite, SWT.PUSH);
		GuiUtils.configureControl(sourceButton, "sourceFile",
				"Choose a file to import", "Browse...");
		SelectionListener sourceListener = new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				sourceFile = getRulebaseFile();
				sourceText.setText(sourceFile.getPath());
			}
		};
		sourceButton.addSelectionListener(sourceListener);

		metadataInSamePlace = new Button(composite, SWT.CHECK);
		metadataInSamePlace.setText("Rulebase metadata (script files) are in same location");
		metadataInSamePlace
				.setToolTipText("If this is selected, all resources in the same dirctory will be considered metadata for this rulebase and will be copied. If there are many files, it could take a long time.");
		metadataInSamePlace.setSelection(true);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 3;
		metadataInSamePlace.setLayoutData(data);
		metadataInSamePlace.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				enableMetadataLocation(!metadataInSamePlace.getSelection());
				getWizard().getContainer().updateButtons();
			}
		});

		metadataLabel = new Label(composite, SWT.LEFT);
		metadataLabel.setText("Metadata Location");
		metadataText = new Text(composite, SWT.SINGLE);
		data = new GridData(GridData.FILL_HORIZONTAL);
		metadataText.setLayoutData(data);
		metadataText
				.setToolTipText("Enter the directory where this rulebase's metadata resides or use the Browse button to select one");
		metadataText.addModifyListener(new ModifyListener() {

			public void modifyText(ModifyEvent e) {
				if (metadataText != null) {
					String text = metadataText.getText();
					if (text != null && text.trim().length() > 0) {
						metadataLocationComplete = true;
					} else
						metadataLocationComplete = false;
					getWizard().getContainer().updateButtons();
				}

			}

		});
		metadataButton = new Button(composite, SWT.PUSH);
		GuiUtils.configureControl(metadataButton, "metadata",
				"Choose a file to import", "Browse...");
		SelectionListener metadataListener = new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				metadataDir = getMetadataDir();
				metadataText.setText(metadataDir.getPath());
			}
		};
		metadataButton.addSelectionListener(metadataListener);
		enableMetadataLocation(false);
		setControl(composite);
	}

	private void enableMetadataLocation(boolean b) {
		metadataLabel.setEnabled(b);
		metadataText.setEnabled(b);
		metadataButton.setEnabled(b);
	}

	@Override
	public boolean canFlipToNextPage() {
		return isPageComplete();
	}

	@Override
	public IWizardPage getNextPage() {
		return getWizard().getPage(TargetLocationPage.PAGE_NAME);
	}

	private boolean saveData() {
		LoadRulebaseWizard wizard = (LoadRulebaseWizard) getWizard();

		sourceFile = new File(sourceText.getText());
		if (sourceFile != null && !sourceFile.canRead()) {
			setErrorMessage(INVALID_FILE);
			return false;
		}
		if (metadataInSamePlace.getSelection()) {
			metadataDir = sourceFile.getParentFile();
		} else {
			metadataDir = new File(metadataText.getText());
		}
		if (metadataDir != null && !metadataDir.isDirectory()) {
			setErrorMessage(INVALID_DIR);
			return false;
		}
		wizard.setSourceFile(sourceFile);
		wizard.setMetadataDir(metadataDir);
		setErrorMessage(null);
		return true;

	}

	private File getRulebaseFile() {
		FileDialog fd = new FileDialog(getShell(), SWT.OPEN);
		fd.setFilterExtensions(new String[] { "*.xml" });
		fd.setFilterNames(new String[] { "Rulebase Files (*.xml)" });
		File file = null;
		String path = fd.open();
		if (path != null && path.length() > 0) {
			file = new File(path);
		}
		return file;
	}

	private File getMetadataDir() {
		DirectoryDialog fd = new DirectoryDialog(getShell(), SWT.OPEN);
		File dir = null;
		String path = fd.open();
		if (path != null && path.length() > 0) {
			dir = new File(path);
		}
		return dir;

	}

	@Override
	public boolean isPageComplete() {
		if (sourceLocationComplete && metadataInSamePlace.getSelection()
				|| sourceLocationComplete && metadataLocationComplete) {
			return saveData();
		} else {
			return false;
		}

	}

}
