/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  IBM Corporation - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.valdoc.rulebase;

import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.logging.LoggingUtil;
import org.eclipse.actf.util.resources.ClassLoaderCache;
import org.eclipse.actf.validation.rulebase.IRuleBase;
import org.eclipse.actf.validation.rules.AndRule;
import org.eclipse.actf.validation.rules.EqRule;
import org.eclipse.actf.validation.rules.FocusRule;
import org.eclipse.actf.validation.rules.ICriterion;
import org.eclipse.actf.validation.rules.ICriterionFactory;
import org.eclipse.actf.validation.rules.ICriterionMapping;
import org.eclipse.actf.validation.rules.IRule;
import org.eclipse.actf.validation.rules.IRuleSet;
import org.eclipse.actf.validation.rules.InstanceofRule;
import org.eclipse.actf.validation.rules.MaxRule;
import org.eclipse.actf.validation.rules.MinRule;
import org.eclipse.actf.validation.rules.NeqRule;
import org.eclipse.actf.validation.rules.NodeNameCriterion;
import org.eclipse.actf.validation.rules.OrRule;
import org.eclipse.actf.validation.rules.PropertyCriterion;
import org.eclipse.actf.validation.rules.RangeRule;
import org.eclipse.actf.validation.rules.ReflexiveCriterion;
import org.eclipse.actf.validation.rules.ReflexiveRule;
import org.eclipse.actf.validation.rules.RegexpRule;
import org.eclipse.actf.validation.rules.RuleConstants;
import org.eclipse.actf.validation.rules.ScriptRule;
import org.eclipse.actf.validation.rules.TypeCriterion;
import org.eclipse.actf.validation.valdoc.codeproc.ICodeProcessor;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;

/**
 * The IRuleFactory for ACTF rules.
 * 
 * @author John Buslawski
 * 
 */
public class ReflectionDocumentCriterionFactory implements ICriterionFactory {

	protected static final Map ruleMap = new HashMap();
	static {
		ruleMap.put(RuleConstants.AND, AndRule.class.getName());
		ruleMap.put(RuleConstants.CRITERION, ReflexiveCriterion.class.getName());
		ruleMap.put(RuleConstants.EQ, EqRule.class.getName());
		ruleMap.put(RuleConstants.FOCUS, FocusRule.class.getName());
		ruleMap.put(RuleConstants.INSTANCEOF, InstanceofRule.class.getName());
		ruleMap.put(RuleConstants.MIN, MinRule.class.getName());
		ruleMap.put(RuleConstants.MAX, MaxRule.class.getName());
		ruleMap.put(RuleConstants.NEQ, NeqRule.class.getName());
		ruleMap.put(RuleConstants.OR, OrRule.class.getName());
		ruleMap.put(RuleConstants.RANGE, RangeRule.class.getName());
		ruleMap.put(RuleConstants.REGEXP, RegexpRule.class.getName());
		ruleMap.put(RuleConstants.REFLEXIVE, ReflexiveRule.class.getName());
		ruleMap.put(RuleConstants.SCRIPT, ScriptRule.class.getName());
		ruleMap.put(RuleConstants.SCRIPTTEST, ScriptRule.class.getName());
	}

	private ClassLoaderCache clCache = ClassLoaderCache.getDefault();
	private IRuleBase ruleBase;
	private ICodeProcessor codeProcessor;
	private Set polymorphicTypes = new HashSet();
	private Set nonPolymorphicTypes = new HashSet();
	private IValidationDocument validationDoc;
	private IRuleSet ruleSet;

	public ICodeProcessor getCodeProcessor () {
		return codeProcessor;
	}
	
	public void setCodeProcessor (ICodeProcessor codeProc) {
		codeProcessor = codeProc;
	}
	
	public IRuleBase getRuleBase () {
		return ruleBase;
	}
	
	public void setRuleBase (IRuleBase ruleBase) {
		this.ruleBase = ruleBase;
	}
	
	public IRuleSet getRuleSet () {
		return ruleSet;
	}
	
	public void setRuleSet (IRuleSet ruleSet) {
		this.ruleSet = ruleSet;
	}
	
	/** 
	 * {@inheritDoc}
	 * <p>For a <code>ReflectionDocument</code>,the mapping will consist of a Java type 
	 * mapped to a set of rules to be applied to instances of that type. The type is 
	 * typically aliased and is the name of the element defining a particular set of rules 
	 * within a ruleset.
	 * 
	 * <p>Hence, the 'rulesetToken' should be an instance of <code>org.w3c.dom.Element</code>, 
	 * having a name that specifies the type for which rules represented by its child elements 
	 * are to be applied.
	 *  
	 */
	public ICriterionMapping getCriterionMapping(Object rulesetToken) {
		// Map containing list of rules for each criterion
		Map<ICriterion, List<IRule>> criteriaRuleMap = new HashMap<ICriterion, List<IRule>>();
		
		if (rulesetToken instanceof Element) {
			Element ruleSetElem = (Element) rulesetToken;
			String ruleSetName = 
				ruleSetElem.getAttributeNS(IValidationDocument.ACTF_NAMESPACE_URI, RuleConstants.NAME);
			Element[] ruleElements = ReflectionValidationDocument.getChildElements(ruleSetElem);
			for (Element child : ruleElements) {
				processRuleElement(child, criteriaRuleMap);
			}
		} else {
			throw new IllegalArgumentException(
				"Ruleset token for a ReflectionDocument must be of type org.w3c.dom.Element");
		}
		return new ValidationDocumentCriterionMapping(
				criteriaRuleMap, polymorphicTypes, nonPolymorphicTypes);
	}

	protected void processRuleElement (Element ruleElem, Map<ICriterion, List<IRule>> criteriaRuleMap) {
		// short-hand in valaidation documents may use rule element tag name
		// to point to class or nodeName as defined by model
		String classOrNodeName = getAliasValue(ruleElem.getTagName());
		ICriterion criterion = null;
		
		if (classOrNodeName.indexOf('.') > 0) {
			// assume a class name and use TypeCriterion
			criterion = new TypeCriterion();
		} else {
			// using node name as defined by model
			criterion = new NodeNameCriterion();
		}
		
		try {
			criterion.setRuleSet(ruleSet);
			criterion.initialize(ruleElem);

			List<IRule> rulesList = createRulesForCriterion(ruleElem);
			criteriaRuleMap.put(criterion, rulesList);
			if (criterion instanceof TypeCriterion) {
				Class cls = ((TypeCriterion) criterion).getTargetClass();
				if (((TypeCriterion) criterion).isPolymorphic()) {
					polymorphicTypes.add(cls);
					if (nonPolymorphicTypes.contains(cls)) {
						nonPolymorphicTypes.remove(cls);
					}
				} else {
					nonPolymorphicTypes.add(cls);
				}
			}
		} catch (Exception e) {
			LoggingUtil.println(IReporter.WARNING, "Could not initialize criterion for rule element " + ruleElem.getTagName());
		}
	}

	protected List<IRule> createRulesForCriterion(Element ruleElem) {
		List<IRule> rulesList = new LinkedList<IRule>();
		Element[] rules = ReflectionValidationDocument.getChildElements(ruleElem);
		for (Element child : rules) {
			IRule rule = createRule(child);
			if (rule != null) {
				rulesList.add(rule);
				LoggingUtil.println(LoggingUtil.PROCESS_ALL,
						"Adding rule: " + child.getTagName() +
						" to rule list for component: " + ruleElem.getTagName());
				//((AbstractRule) rule).printRule(1);
			} else {
				LoggingUtil.println(LoggingUtil.PROCESS_ALL,
						"Unable to create rule: " + child.getTagName() + 
						" for component: " + ruleElem.getTagName());
			}
		}
		return rulesList;
	}

	protected IRule createRule (Element ruleElement, ICriterion parent) {
		ICriterion vRule = makeCriterionInstance(ruleElement, parent);
		if (vRule != null && vRule instanceof IRule) {
			try {
				LoggingUtil.println(LoggingUtil.PROCESS_ALL,
						"Initializing rule for element: " + ruleElement.getTagName());
				vRule.setRuleSet(ruleSet);
				vRule.initialize(ruleElement);
				LoggingUtil.println(LoggingUtil.PROCESS_ALL,
						"IRule: " + ((IRule)vRule).getName() + " initialized.");
				
				// Get the children of this rule.
				Element[] childRuleElems = ReflectionValidationDocument.getChildElements(ruleElement);
				for (Element childRuleElem : childRuleElems) {
					IRule childRule = createRule(childRuleElem, vRule);
					if (childRule != null) {
						LoggingUtil.println(LoggingUtil.PROCESS_ALL,
								"Adding rule part: " + ((IRule)childRule).getName() + 
								" to parent: " + ((IRule)vRule).getName());
						vRule.addPart(childRule);
					}
				}
			} catch (Exception e) {
				LoggingUtil.println(IReporter.SYSTEM_NONFATAL,
						"Unable to initialize rule: " + ((IRule)vRule).getName(), e);
			}
		}
		
		return vRule instanceof IRule ? (IRule) vRule : null;
	}

	protected IRule createRule (Element ruleElem) {
		return createRule(ruleElem, null);
	}

	protected String getAliasValue(String name) {
		// check aliases defined in validation document
		String value = ((ValidationDocumentRuleBaseMetadata) ruleBase.getMetadata()).getAliasValue(name);
		if (value == null || value.trim().length() == 0) {
			value = name;
		}
		return value;
	}

	public ReflectionDocumentCriterionFactory() {

	}

	public void registerRuleClassName(String name, String ruleClassName) {
		if (name != null && ruleClassName != null) {
			if (!ruleMap.containsKey(name)) {
				ruleMap.put(name, ruleClassName);
			}
		}
	}

	public IRule getRuleInstance(String name) {
		IRule rule = null;
		if (name != null && name.length() > 0) {
			// Try looking up the rule by name.
			String ruleClassName = (String) ruleMap.get(name);
			if (ruleClassName != null && ruleClassName.length() > 0) {
				try {
					LoggingUtil.println(LoggingUtil.PROCESS_ALL,
							"ReflectionDocumentCriterionFactory.getRuleInstance(String), " +
							"Creating rule for element: " + name + 
							", instantiating class: " + ruleClassName);
					rule = (IRule) Class.forName(ruleClassName).newInstance();
				} catch (Exception e) {
					LoggingUtil.println(IReporter.SYSTEM_NONFATAL,
							"ReflectionDocumentCriterionFactory.getRuleInstance(String), " +
							"unable to create rule: " + ruleClassName, e);
				}
			}
		}
		
		return rule;
	}

	public IRule getRuleInstance(String name, IRule parent) {
		IRule rule = null;
		if (name != null && name.length() > 0) {
			rule = getRuleInstance(name);
		}
		if (rule != null && parent != null) {
			LoggingUtil.println(LoggingUtil.PROCESS_ALL,
					"ReflectionDocumentCriterionFactory.getRuleInstance(Element, IRule), " +
					"Setting parent:" + parent.getName());
			rule.setParent(parent);
		}
		return rule;
	}

	public ICriterion makeCriterionInstance(Object obj, ICriterion parent) {
		ICriterion criterion = null;
		Element elem = null;
		Attr attr = null;
		String name = null;
		boolean isActfNamespace = false;
		
		if (obj instanceof Element) {
			elem = (Element) obj;
			name = elem.getTagName();
			isActfNamespace = elem.getNamespaceURI() != null && 
				elem.getNamespaceURI().equals(IValidationDocument.ACTF_NAMESPACE_URI);
		} else if (obj instanceof Attr) {
			// TODO MAS->Phani
			// currently, the only time this is invoked is during processing of validation
			// documents for MSAA and IA2; attribute of immediate child of <actf:ruleset>
			// indicates that a property is to be tested
			attr = (Attr) obj;
			name = attr.getLocalName();
			isActfNamespace = attr.getNamespaceURI() != null && 
				attr.getNamespaceURI().equals(IValidationDocument.ACTF_NAMESPACE_URI);
		}
		
		if (elem != null) {
			String localName = elem.getLocalName();
			if (isActfNamespace && !localName.equals(RuleConstants.CRITERION)) {
				criterion = getRuleInstance(localName);
			} else if (isActfNamespace && localName.equals(RuleConstants.CRITERION)) {
				// TODO: Phani Later once markup is finalized
				// fill in when actf:criterion is introduced				
			}
			
			/*  
			 	getRuleInstance(String) will try to instantiate a rule by name and,
			 	if not successful, will try to look the name up in the ruleNames Map
				and if a name is found, will try to instantiate that class.
				If a rule can't be created and the element is not in the ACTF namespace, then 
				the default is to create a Reflexive rule 
			*/
			if (criterion == null && !isActfNamespace) {
				String ruleClassName = (String) ruleMap.get(RuleConstants.REFLEXIVE);
				if (ruleClassName != null && ruleClassName.length() > 0) {
					try {
						LoggingUtil.println(LoggingUtil.PROCESS_ALL,
								"ReflectionDocumentCriterionFactory.makeRuleInstance(), " +
								"Creating rule for element: " + name +
								", instantiating class: " + ruleClassName);
						criterion = (IRule) Class.forName(ruleClassName).newInstance();
					} catch (Exception e) {
						LoggingUtil.println(IReporter.SYSTEM_NONFATAL,
								"ReflectionDocumentCriterionFactory.makeRuleInstance(), " +
								"unable to create rule: " + ruleClassName, e);
					}
				}
			}
			if (criterion != null && parent != null) {
				LoggingUtil.println(LoggingUtil.PROCESS_ALL,
					"ReflectionDocumentCroterionFactory.getRuleInstance(Element, ComponentRule), " +
					"Setting parent:" + ((IRule)parent).getName());
				criterion.setParent(parent);
			}
		} else if (attr != null) {
			// TODO needs work
			if (!isActfNamespace) {
				// TODO Phani - Test once markup and PropertyCriterion is finalized
				// creating a PropertyCriterion
				try {
					LoggingUtil.println(LoggingUtil.PROCESS_ALL,
							"ReflectionDocumentCriterionFactory.makeRuleInstance(), " +
							"Creating PropertyCriterion for attribute: " + name + ", " +
							"instantiating propertyCriterion");
					criterion = new PropertyCriterion();
				} catch(Exception e) {
					LoggingUtil.println(IReporter.SYSTEM_NONFATAL, 
							"ReflectionDocumentCriterionFactory.makeRuleInstance(), " +
							"unable to create PropertyCriterion " +  e);
				}
			}
		}
		return criterion;
	}

	public ICriterion makeCriterionInstance(Object obj) {
		return makeCriterionInstance(obj, null);
	}

	/**
	 * Checks to see the parameter is a rule Attribute
	 * @param name
	 * @return boolean
	 */
	public boolean isRuleAttribute(String name) {
		boolean value = false;
		value =  name.equals(RuleConstants.ID)
					|| name.equals(RuleConstants.NAME)
					|| name.equals(RuleConstants.SEVERITY)
					|| name.equals(RuleConstants.POLYMORPHIC)
					|| name.equals(RuleConstants.MESSAGE)
					|| name.equals(RuleConstants.DESCRIPTION)
					|| name.equals(RuleConstants.HELPURL)
					|| name.equals(RuleConstants.PARAM)
					|| name.equals(RuleConstants.ITERATOR)
					|| name.equals(RuleConstants.IGNOREIFNULL)
					|| name.equals(RuleConstants.ENABLE);
		
		return value;
	}
}
