/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  IBM Corporation - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.valdoc.rulebase;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.eclipse.actf.validation.core.ExecutionPointDescriptor;
import org.eclipse.actf.validation.core.MethodExecutionPointTarget;
import org.eclipse.actf.validation.core.ModelTransition;
import org.eclipse.actf.validation.core.TargetArgument;
import org.eclipse.actf.validation.rulebase.MalformedRuleBaseException;
import org.eclipse.actf.validation.rules.RuleConstants;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.xml.sax.SAXException;


/**
 * embodies a validation document that uses knowledge of the Java API of the model being validated and the validation engine's use 
 * of the Java Reflection API to process validation rules. This was the original implementation of a valaidation document in the 
 * IBM IRule-based Accessibility Validation Environment (RAVEN).
 * 
 *  @see <a href="http://www.alphaworks.ibm.com/tech/raven">IBM IRule-based Accessibility Validation Environment</a>
 * @author <a href="mailto:masquill@us.ibm.com">Mike Squillace</a>
 */
public class ReflectionValidationDocument implements IValidationDocument
{

	private Element root;
	
	/**
	 * create a ReflectionValidationDocument from the given XML source stream. The XML document
	 * is parsed and may be traversed using the methods of this class.
	 * 
	 * @param xmlSource --
	 *            source stream for document
	 * @throws SAXException
	 *             in case of parsing error
	 * @throws IOException
	 *             in case of I/O error
	 * @throws ParserConfigurationException
	 */
	public ReflectionValidationDocument(InputStream xmlSource)
			throws ParserConfigurationException, SAXException, IOException {
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setNamespaceAware(true);
		factory.setIgnoringComments(true);
		factory.setCoalescing(true);
		
		DocumentBuilder builder = factory.newDocumentBuilder();
		Document document = builder.parse(xmlSource);
		root = document.getDocumentElement();
	} // ctor

	/** {@inheritDoc} */
	public String getDocumentName() {
		return root.getAttributeNS(ACTF_NAMESPACE_URI, DOCUMENT_NAME);
	}

	/** {@inheritDoc} */
	public String getModelType() {
		return root.getAttributeNS(ACTF_NAMESPACE_URI, MODEL);
	}

	/** {@inheritDoc} */
	public String getDocumentVersion() {
		return root.getAttributeNS(ACTF_NAMESPACE_URI, DOCUMENT_VERSION);
	}

	/** {@inheritDoc} */
	public String getScriptLanguage() {
		return root.getAttributeNS(ACTF_NAMESPACE_URI, SCRIPTLANG);
	}

	/** {@inheritDoc} */
	public String getEnable () {
		return root.getAttributeNS(ACTF_NAMESPACE_URI, ENABLE);
	}

	/** {@inheritDoc} */
	public ScriptElement[] getScriptElements() throws MalformedRuleBaseException {
		Element scriptsElem = getMetaDataElement("scripts");
		NodeList nodes = scriptsElem.getElementsByTagNameNS(ACTF_NAMESPACE_URI, "script");
		int length = nodes.getLength();
		ScriptElement[] scripts = new ScriptElement[length];
		
		for (int i = 0; i < length; ++i) {
			Element elem = (Element) nodes.item(i);
			Node first = elem.getFirstChild();
			scripts[i] = new ScriptElement(
					elem.getAttributeNS(ACTF_NAMESPACE_URI, RuleConstants.NAME),
					first != null && first.getNodeType() == Node.TEXT_NODE ? ((Text) first).getWholeText() : null,
							elem.getAttributeNS(ACTF_NAMESPACE_URI, "src"),
							elem.getAttributeNS(ACTF_NAMESPACE_URI, "imports")
			);
		}
		
		return scripts;
	}

	/** {@inheritDoc} */
	public AliasElement[] getAliasElements()  throws MalformedRuleBaseException {
		Element aliasesElem = getMetaDataElement("aliases");
		NodeList nodes = aliasesElem.getElementsByTagNameNS(ACTF_NAMESPACE_URI, "alias");
		int length = nodes.getLength();
		AliasElement[] aliases = new AliasElement[length];
		
		for (int i = 0; i < length; ++i) {
			Element elem = (Element) nodes.item(i);
			Node first = elem.getFirstChild();
			aliases[i] = new AliasElement(
					elem.getAttributeNS(ACTF_NAMESPACE_URI, RuleConstants.NAME),
					first != null && first.getNodeType() == Node.TEXT_NODE ? ((Text) first).getWholeText() : null,
							elem.getAttributeNS(ACTF_NAMESPACE_URI, "value")
			);
		}
		
		return aliases;
	}

	/** {@inheritDoc} */
	public Element[] getRuleSetElements() {
		return getChildElements(root, "ruleset");
	}

	/** {@inheritDoc} */
	public ExecutionPointDescriptor[] getExecutionPointDescriptors() {
		NodeList nodes = root.getElementsByTagNameNS(ACTF_NAMESPACE_URI, "executionPoint");
		int length = nodes.getLength();
		LinkedList descriptors = new LinkedList();
		
		for (int i = 0; i < length; ++i) {
			Element elem = (Element) nodes.item(i);
			ExecutionPointDescriptor descriptor = parseExecutionPointDescriptor(elem);
			if (descriptor != null) {
				descriptors.add(descriptor);
			}
		}
		
		return (ExecutionPointDescriptor[]) descriptors.toArray(new ExecutionPointDescriptor[descriptors.size()]);
	}

	protected ExecutionPointDescriptor parseExecutionPointDescriptor(Element execPointElem) {
		List executionTargets = new LinkedList();
		String name = execPointElem.getAttributeNS(ACTF_NAMESPACE_URI, ExecutionPointDescriptor.EXECUTION_POINT_NAME);
		String validator = execPointElem.getAttributeNS(ACTF_NAMESPACE_URI, ExecutionPointDescriptor.EXECUTION_POINT_VALIDATOR);
		
		Element[] targetElems = getChildElements(execPointElem, "target");
		for (int t = 0; t < targetElems.length; ++t) {
			Element targetElem = targetElems[t];
			String targetTypeName = targetElem.getAttributeNS(ACTF_NAMESPACE_URI, ExecutionPointDescriptor.EXECUTION_POINT_NAME);
			String polyStr = targetElem.getAttributeNS(ACTF_NAMESPACE_URI, RuleConstants.POLYMORPHIC);
			boolean polymorphic = polyStr != null && polyStr.equalsIgnoreCase("true");
			
			// Build the List of method/event targets.
			Element[] targetTypeElems = getChildElements(targetElem);
			for (int tt = 0; tt < targetTypeElems.length; ++tt) {
				Element targetTypeElem = targetTypeElems[tt];
				String typeName = targetTypeElem.getLocalName();
				if (typeName.equals(ExecutionPointDescriptor.METHOD_TYPE)) {
					executionTargets.add(createMethodExecutionTarget(targetTypeElem, targetTypeName, polymorphic));
				} else if (typeName.equals(ExecutionPointDescriptor.EVENT_TYPE)) {
//					String listenerName = targetTypeElem
//							.getActfAttributeValue(ValidationDocumentConstants.LISTENER);
//					String methodName = targetTypeElem
//							.getActfAttributeValue(ValidationDocumentConstants.METHOD_NAME);
//					EventExecutionPointTarget eepTarget = new EventExecutionPointTarget(
//							targetName, listenerName, methodName, polymorphic);
//					executionTargets.add(eepTarget);
				}
			}
		}
		
		List modelTrans = new LinkedList();
		Element[] modelTransElems = getChildElements(execPointElem, "modelTransition");
		for (int i = 0; i < modelTransElems.length; ++i) {
			Element archTransElem = modelTransElems[i];
			String regExpStr = archTransElem.getAttributeNS(ACTF_NAMESPACE_URI, "regexp");
			String ruleBaseName = archTransElem.getAttributeNS(ACTF_NAMESPACE_URI, "rulebase");
			String execPointName = archTransElem.getAttributeNS(ACTF_NAMESPACE_URI, "executionPoint");
			modelTrans.add(new ModelTransition(regExpStr, ruleBaseName, execPointName));
		}
		
		return new ExecutionPointDescriptor(name, validator, executionTargets, modelTrans);
	}

	protected MethodExecutionPointTarget createMethodExecutionTarget (Element targetTypeElem, String targetTypeName, boolean polymorphic) {
		String methodName = targetTypeElem.getAttributeNS(ACTF_NAMESPACE_URI, ExecutionPointDescriptor.EXECUTION_POINT_NAME);
		
		// build the arguments for a method invocation
		List argumentList = new LinkedList();
		Element[] argElems = getChildElements(targetTypeElem);
		for (int a = 0; a < argElems.length; ++a) {
			Element argElem = argElems[a];
			String type = argElem.getAttributeNS(ACTF_NAMESPACE_URI, "type");
			String value = argElem.getAttributeNS(ACTF_NAMESPACE_URI, "value");
			argumentList.add(new TargetArgument(type, value));
		}
		
		TargetArgument[] targetArgs = (TargetArgument[]) argumentList.toArray(new TargetArgument[argumentList.size()]);
		return new MethodExecutionPointTarget(targetTypeName, methodName, targetArgs, polymorphic);
	}
	
	public static Element[] getChildElements (Element element) {
		NodeList childNodes = element.getChildNodes();
		ArrayList children = new ArrayList();
		for (int child = 0; child < childNodes.getLength(); ++child) {
			Node childNode = childNodes.item(child);
			if (childNode.getNodeType() == Node.ELEMENT_NODE) {
				children.add(childNode);
			}
		}
		
		return (Element[]) children.toArray(new Element[children.size()]);
	}
	
	public static Element[] getChildElements (Element element, String name) {
		NodeList childNodes = element.getChildNodes();
		ArrayList children = new ArrayList();
		for (int child = 0; child < childNodes.getLength(); ++child) {
			Node childNode = childNodes.item(child);
			if (childNode.getNodeType() == Node.ELEMENT_NODE && childNode.getLocalName().equals(name)) {
				children.add(childNode);
			}
		}
		
		return (Element[]) children.toArray(new Element[children.size()]);
	}
	
	protected Element getMetaDataElement (String name) throws MalformedRuleBaseException {
		Element result = null;
		NodeList nodes = root.getElementsByTagNameNS(ACTF_NAMESPACE_URI, "metadata");
		if (nodes.getLength() == 1 && nodes.item(0).getNodeType() == Node.ELEMENT_NODE) {
			Element metadata = (Element) nodes.item(0);
			nodes = metadata.getElementsByTagNameNS(ACTF_NAMESPACE_URI, name);
			if (nodes.getLength() == 1 && nodes.item(0).getNodeType() == Node.ELEMENT_NODE) {
				result = (Element) nodes.item(0);
			} else {
				throw new MalformedRuleBaseException("Only one actf:" + name + " element per <actf:metadata> element in a document");
			}
		} else {
			throw new MalformedRuleBaseException("Only one actf:metadata element per <actf:metadata> element in a document");
		}
		
		return result; 
	}
/*	
	public static void main(String[] args) throws Exception {
		FileInputStream fis = new FileInputStream("c:/eclipse-constellation-321/eclipse/actf-0.1.0/com.ibm.haac.raven.model.java.core/resources/swing_accessibility.xml");
		ReflectionValidationDocument doc = new ReflectionValidationDocument(fis);
		System.out.println("version:"+doc.getDocumentVersion());
		System.out.println("Name:"+doc.getDocumentName());
		System.out.println("language:"+doc.getScriptLanguage());
		System.out.println("model:"+doc.getModelType());
		ScriptElement[] s = doc.getScriptElements();
		AliasElement[] a = doc.getAliasElements();
		RuleSetElement[] r = doc.getRuleSetElements();
		ExecutionPointDescriptor[] d = doc.getExecutionPointDescriptors();
		System.out.println("Done");
	}
	*/
} // ReflectionValidationDocument
