/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  IBM Corporation - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.validators;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.eclipse.actf.core.config.ConfigurationException;
import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.core.runtime.IRuntimeContext;
import org.eclipse.actf.core.runtime.RuntimeContextFactory;
import org.eclipse.actf.model.IModel;
import org.eclipse.actf.model.locate.INodeLocator;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.logging.LoggingUtil;
import org.eclipse.actf.validation.valdoc.codeproc.ICodeProcessor;

/**
 * a wrapper class for the IRuleBase properties and selected global properties
 * that are necessary to validate a component.
 * 
 * @author John Buslawski
 */
public class AccessibilityValidationContext extends AbstractValidationContext {

	// empty ctor
	public AccessibilityValidationContext() {
		super();
	}

	// ctor
	public AccessibilityValidationContext(IModel arch, ICodeProcessor codeProc,
										  IReporter reporter, INodeLocator locator,
										  Set validatedComps, Map componentMap) {
		super();
		setArchitecture(arch);
		setCodeProcessor(codeProc);
		try {
			IRuntimeContext context = RuntimeContextFactory.getInstance()
					.getRuntimeContext();
			IConfiguration config = context.getConfiguration();
			setActfConfiguration(config);
		} catch (ConfigurationException e) {
			throw new RuntimeException(e.getMessage(), e);
		}
		setReporter(reporter);
		setElementLocator(locator);
		setValidatedComponents(validatedComps == null ? new HashSet() : validatedComps);
		setElementBridgeMap(componentMap);
	}

	/**
	 * convenience method for obtaining the model associated with this
	 * context. This method is equivalent to
	 * <code>(IModel) getParameter(ValidationContextConstants.MODEL)</code>
	 * 
	 * @return model of this context
	 */
	public IModel getModel() {
		return (IModel) getParameter(ValidationContextConstants.MODEL);
	}

	/**
	 * convenience method for obtaining the code processor associated with this
	 * context. This method is equivalent to
	 * <code>(IModel) getParameter(ValidationContextConstants.CODE_PROCESSOR)</code>
	 * 
	 * @return code processor of this context
	 */
	public ICodeProcessor getCodeProcessor() {
		return (ICodeProcessor) getParameter(ValidationContextConstants.CODE_PROCESSOR);
	}

	/**
	 * convenience method for obtaining the locator associated with this
	 * context. This method is equivalent to
	 * <code>(IModel) getParameter(ValidationContextConstants.ELEMENT_LOCATOR)</code>
	 * 
	 * @return locator of this context
	 */
	public INodeLocator getNodeLocator() {
		return (INodeLocator) getParameter(ValidationContextConstants.ELEMENT_LOCATOR);
	}

	/**
	 * convenience method for obtaining the component bridge map associated with
	 * this context. This method is equivalent to
	 * <code>(IModel) getParameter(ValidationContextConstants.ELEMENT_BRIDGE_MAP)</code>
	 * 
	 * @return component bridge map of this context
	 */
	public Map getElementBridgeMap() {
		return (Map) getParameter(ValidationContextConstants.ELEMENT_BRIDGE_MAP);
	}

	/**
	 * convenience method for obtaining the configuration associated with this
	 * context. This method is equivalent to
	 * <code>(IModel) getParameter(ValidationContextConstants.CONFIGURATION</code>
	 * 
	 * @return configuration of this context
	 */
	public IConfiguration getConfiguration() {
		return (IConfiguration) getParameter(ValidationContextConstants.CONFIGURATION);
	}

	/**
	 * convenience method for obtaining the reporter associated with this
	 * context. This method is equivalent to
	 * <code>(IModel) getParameter(ValidationContextConstants.REPORTER)</code>
	 * 
	 * @return reporter of this context
	 */
	public IReporter getReporter() {
		return (IReporter) getParameter(ValidationContextConstants.REPORTER);
	}

	/**
	 * convenience method for obtaining the set of previously validated
	 * components from this context. This method is equivalent to
	 * <code>(IModel) getParameter(ValidationContextConstants.VALIDATED_ELEMENTS)</code>
	 * 
	 * @return model of this context
	 */
	public Set getValidatedComponents() {
		return (Set) getParameter(ValidationContextConstants.VALIDATED_ELEMENTS);
	}

	/**
	 * set the model for this context. This method is equivalent to
	 * <code>setParameter(ValidationContextConstants.MODEL, arch)</code>
	 * so long as the model is not <code>null</code>
	 * 
	 * @param model -
	 *            must not be <code>null</code>
	 */
	public void setArchitecture(IModel model) {
		if (model != null) {
			setParameter(ValidationContextConstants.MODEL, model);
		} else {
			LoggingUtil
					.println(IReporter.SYSTEM_NONFATAL,
							"Null IRenderableModel in AccessibilityValidationContext.setArchitecture().");
		}
	}

	/**
	 * set the code processor for this context. This method is equivalent to
	 * <code>setParameter(ValidationContextConstants.CODE_PROCESSOR, cp)</code>
	 * so long as the code processor is not <code>null</code>
	 * 
	 * @param cp -
	 *            must not be <code>null</code>
	 */
	public void setCodeProcessor(ICodeProcessor cp) {
		if (cp != null) {
			contextMap.put(ValidationContextConstants.CODE_PROCESSOR, cp);
		} else {
			LoggingUtil
					.println(IReporter.SYSTEM_NONFATAL,
							"Null ICodeProcessor in AccessibilityValidationContext.setCodeProcessor().");
		}
	}

	/**
	 * set the component bridge map for this context. This method is equivalent
	 * to
	 * <code>setParameter(ValidationContextConstants.ELEMENT_BRIDGE_MAP, compMap)</code>
	 * 
	 * @param compMap
	 */
	public void setElementBridgeMap(Map compMap) {
		// component map can be null
		setParameter(ValidationContextConstants.ELEMENT_BRIDGE_MAP, compMap);
	}

	/**
	 * set the locator for this context. This method is equivalent to
	 * <code>setParameter(ValidationContextConstants.ELEMENT_LOCATOR, locator)</code>
	 * so long as the locator is not <code>null</code>
	 * 
	 * @param locator -
	 *            must not be <code>null</code>
	 */
	public void setElementLocator(INodeLocator locator) {
		if (locator != null) {
			setParameter(ValidationContextConstants.ELEMENT_LOCATOR, locator);
		} else {
			LoggingUtil
					.println(IReporter.SYSTEM_NONFATAL,
							"Null locator in AccessibilityValidationContext.setElementLocator.");
		}
	}

	/**
	 * set the configuration for this context. This method is equivalent to
	 * <code>setParameter(ValidationContextConstants.CONFIGURATION, config)</code>
	 * so long as the configuration is not <code>null</code>
	 * 
	 * @param config -
	 *            must not be <code>null</code>
	 */
	public void setActfConfiguration(IConfiguration config) {
		if (config != null) {
			setParameter(ValidationContextConstants.CONFIGURATION, config);
		} else {
			LoggingUtil
					.println(
							IReporter.SYSTEM_NONFATAL,
							"Null configuration in AccessibilityValidationContext.setActfConfiguration(), setting default configuration.");
		}
	}

	/**
	 * set the reporter for this context. This method is equivalent to
	 * <code>setParameter(ValidationContextConstants.REPORTER, reporter)</code>
	 * so long as the reporter is not <code>null</code>
	 * 
	 * @param reporter -
	 *            must not be <code>null</code>
	 */
	public void setReporter(IReporter reporter) {
		if (reporter != null) {
			setParameter(ValidationContextConstants.REPORTER, reporter);
		} else {
			LoggingUtil
					.println(IReporter.SYSTEM_NONFATAL,
							"Null Reporter in AccessibilityValidationContext.setReporter().");
		}
	}

	/**
	 * set the set of previously validated components for this context. This
	 * method is equivalent to
	 * <code>setParameter(ValidationContextConstants.VALIDATED_ELEMENTS, arch)</code>
	 * 
	 * @param valComp
	 */
	public void setValidatedComponents(Set valComp) {
		if (valComp != null) {
			setParameter(ValidationContextConstants.VALIDATED_ELEMENTS, valComp);
		} else {
			setParameter(ValidationContextConstants.VALIDATED_ELEMENTS,
					new HashSet());
		}
	}
}
