/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  IBM Corporation - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.validators;

import java.text.MessageFormat;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.model.IModel;
import org.eclipse.actf.model.InvalidComponentException;
import org.eclipse.actf.model.locate.AbstractNodeLocator;
import org.eclipse.actf.model.locate.INodeLocator;
import org.eclipse.actf.util.logging.AbstractReporter;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.logging.LoggingUtil;
import org.eclipse.actf.util.logging.PrintWriterReporter;
import org.eclipse.actf.validation.ValidationConstants;
import org.eclipse.actf.validation.rules.ICriterion;
import org.eclipse.actf.validation.rules.ICriterionMapping;
import org.eclipse.actf.validation.rules.IRule;
import org.eclipse.actf.validation.rules.IRuleSet;
import org.eclipse.actf.validation.valdoc.codeproc.ICodeProcessor;
import org.eclipse.actf.validation.valdoc.rulebase.ValidationDocumentRuleSet;

/**
 * base implementation for all accessibility validator classes.
 * 
 * @author John Buslawski
 * 
 */
public class GenericAccessibilityValidator implements IValidator
{

	protected IRuleSet ruleSet;
	
	/**
	 * Create an AccesibilityValidator for the given rule set.
	 * 
	 * @param ruleSet - the rule set to use when validating the components
	 */
	public GenericAccessibilityValidator(IRuleSet ruleSet) {
		if (ruleSet != null) {
			this.ruleSet = ruleSet;
		} else {
			LoggingUtil.println(IReporter.SYSTEM_NONFATAL,"Null ruleSet in AccessibilityValidator().");
		}
	}

	public IRuleSet getRuleSet() {
		return ruleSet;
	}

	public void setRuleSet(IRuleSet ruleSet) {
		if (ruleSet != null) {
			this.ruleSet = ruleSet;
		} else {
			LoggingUtil.println(IReporter.SYSTEM_NONFATAL,
					"Null ruleSet in AccessibilityValidator.setRuleSet().");
		}
	}

	/**
	 * validate the specified component according to the list of lists of rules.
	 * This list is typically generated by a call to
	 * <code>createRulesList</code>.
	 *
	 * @param context - supplied by engine
	 * @param rulesListList - list of rule lists for this component
	 */
	protected void validateRules(IValidationContext context, List<List<IRule>> rulesListList) {
		Object component = context.getParameter(ValidationContextConstants.CURRENT_TARGET);
		LoggingUtil.println(LoggingUtil.ALL, "Evaluating rules for component : " + component);
		
		OUTER: for (List<IRule> rulesList : rulesListList) {
			for (IRule rule : rulesList) {
				try {
					rule.setComponentCategory(((IReporter) context.getParameter(
							ValidationContextConstants.REPORTER)).getCategory());
					rule.apply(context, component);
					LoggingUtil.println(LoggingUtil.ALL,
						"Component : " + component.getClass().getName() +
						", rule name : " +rule.getName());
				} catch (InvalidComponentException e) {
					break OUTER;
				} catch (Exception e) {
					LoggingUtil.println(IReporter.SYSTEM_NONFATAL,
							"Exception caught while processing rule "
									+ rule.getName() + ", for component "
									+ component.getClass().getName(), e);
				}
			}
		}
	}

	/**
	 * used to validate single components.
	 *
	 * @param context supplied by the engine
	 * @param component - control to be validated
	 */
	public void validate(IValidationContext context, Object component) {
		List<List<IRule>> ruleList = new LinkedList<List<IRule>>();
		INodeLocator locator = (INodeLocator) context.getParameter(ValidationContextConstants.ELEMENT_LOCATOR);
		IReporter reporter = (IReporter) context.getParameter(ValidationContextConstants.REPORTER);
		IConfiguration config = (IConfiguration) context.getParameter(ValidationContextConstants.CONFIGURATION);
		IModel model = (IModel) context.getParameter(ValidationContextConstants.MODEL);
		Set validatedCompSet = (Set) context.getParameter(ValidationContextConstants.VALIDATED_ELEMENTS);
		
		String nodeName = model.getNodeName(component);
		String nodeId = model.getNodeId(component);
		String description = AbstractNodeLocator.descriptionAsString(locator.describe(component));
		LoggingUtil.println(LoggingUtil.PROCESS_INFO, "Location information:" + locator.locate(component, null));
		
		config.setSymbolPool(ValidationConstants.VALIDATION_ID);
		boolean revalidating = config.getBooleanParameter(ValidationConstants.REVALIDATING_COMPONENTS_KEY);
		
		// TODO MAS->Phani
		// algorithm is close to what we want but modify as necessary
		if (revalidating || !validatedCompSet.contains(component)) {
			// TODO MAS->Phani
			// first layer
			// get the rule map and find the list of criteria to rule list map
			// for this component.
			ICriterionMapping criterionMapping = ((ValidationDocumentRuleSet) getRuleSet()).getCriterionMap();
			ICriterion []criteria = criterionMapping.getCriteria();
			Map<ICriterion, List<IRule>> criteriaRulesMap = criterionMapping.getAllRules();
			for(ICriterion criterion : criteria) {
				// first, test the criterion to see if rules are to be applied to this target
				Boolean meetsCriterion = false;
				try {
					meetsCriterion = criterion.apply(context, component);
				} catch(Exception e) {
					LoggingUtil.println(IReporter.SYSTEM_NONFATAL, e);
				}
				if (!meetsCriterion) {
					continue;					
				}
				
				List<IRule> rules = (List<IRule>) criteriaRulesMap.get(criterion);
				if(rules == null) {
					LoggingUtil.println(LoggingUtil.ALL, "Null rules for the criteria : "+ nodeName + "," +
							"Validation terminated.");
					continue;
				}
				//There are rules to evaluate. If it doesn't succeed then skip to the next set of criteria
				// Add the rules to the set of rules to validate.
				ruleList.add(rules);
			}			
			if (ruleList.size() > 0) {
				LoggingUtil.println(LoggingUtil.PROCESS_INFO,
						"Validating control of type " + nodeName + (description != null ? ": " + description : ""));
				MessageFormat catFormatter = reporter instanceof AbstractReporter
					? ((AbstractReporter) reporter).getCategoryFormatter() : AbstractReporter.CATEGORY_FORMATTER;
				String category = catFormatter.format(new Object[] {nodeName, description, nodeId});
				ICodeProcessor codeProc = 
					(ICodeProcessor) context.getParameter(ValidationContextConstants.CODE_PROCESSOR);
				reporter.startCategory(category);
				LoggingUtil.startSubprocess(category);
				if (codeProc != null) {
					codeProc.setLocal("thisComponent", component, "true");
				}
				context.setParameter(ValidationContextConstants.CURRENT_TARGET, component);
				int startErrorCount = reporter instanceof PrintWriterReporter ? 
						((PrintWriterReporter) reporter).getErrorCount() : 0;
				int startWarningCount = reporter instanceof PrintWriterReporter ? 
						((PrintWriterReporter) reporter).getWarningCount() : 0;
				validateRules(context, ruleList);
				int endErrorCount =  
					reporter instanceof PrintWriterReporter ? 
							((PrintWriterReporter) reporter).getErrorCount() : 0;
				int endWarningCount = reporter instanceof PrintWriterReporter ? 
						((PrintWriterReporter) reporter).getWarningCount() : 0;
				reporter.report(IReporter.DETAIL,"Completion of validation of component:");
				reporter.report(IReporter.DETAIL,(endErrorCount - startErrorCount) + " errors, "
								+ (endWarningCount - startWarningCount) + " warnings");
				reporter.endCategory();
				LoggingUtil.endSubprocess();
			}
			validatedCompSet.add(component);
		} else {
			LoggingUtil.println(LoggingUtil.PROCESS_INFO, "Component of type "
					+ component.getClass().getName() + " already validated");
		}
	} // validate

	protected boolean checkValidationContext(
			AccessibilityValidationContext context) {
		boolean check = true;
		if (context.getCodeProcessor() == null) {
			check = false;
			LoggingUtil.println(IReporter.SYSTEM_NONFATAL,
				"AccessibilityValidator.checkValidationContext() - " +
				"Null ICodeProcessor in IValidationContext.");
		}
		if (context.getModel() == null) {
			check = false;
			LoggingUtil.println(IReporter.SYSTEM_NONFATAL,
				"AccessibilityValidator.checkValidationContext() - " +
				"Null IRenderableModel in IValidationContext.");
		}
		if (context.getReporter() == null) {
			LoggingUtil.println(IReporter.SYSTEM_NONFATAL,
				"AccessibilityValidator.checkValidationContext() - Null Reporter in IValidationContext, " +
				"setting default Reporter.");
			context.setReporter(AbstractReporter.getReporter(IReporter.INFO,"stdout"));
		}
		if (context.getConfiguration() == null) {
			check = false;
			LoggingUtil.println(IReporter.SYSTEM_NONFATAL,
				"AccessibilityValidator.checkValidationContext() - " +
				"Null Configuration in IValidationContext.");
		}
		return check;
	}


	/**
	 * 
	 * evaluate the criteria for this component. Returns true/false for the
	 * result of the evaluation.
	 * 
	 * @param criteriaList -
	 *            list of criteria for this component
	 * @param component -
	 *            target component
	 */
/*
	protected boolean evaluateRules(IValidationContext context, List rulesList) {
		Object component = context.getParameter(ValidationContextConstants.CURRENT_TARGET);
		boolean result = true;
		Utils.println(Utils.ALL,"Evaluating criteria for component : " + component);
		
		for (Iterator iter = rulesList.iterator(); result && iter.hasNext(); ) {
			IRule rule = (IRule)iter.next();
			try {
				rule.setComponentCategory(((IReporter) context.getParameter(
						ValidationContextConstants.REPORTER)).getCategory());
				result = rule.apply(context, component);
			} catch (Exception e) {
				Utils.println(IReporter.SYSTEM_NONFATAL,
						"Exception caught while processing criteria "
								+ rule + ", for component "
								+ component.getClass().getName(), e);
				result = false;
			}
		}		
		return result;
	}  
*/
}
