/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  IBM Corporation - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.webelo.ui.browser;

import java.lang.reflect.Method;

import org.eclipse.actf.validation.ui.GuiUtils;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.LocationEvent;
import org.eclipse.swt.browser.LocationListener;
import org.eclipse.swt.browser.ProgressEvent;
import org.eclipse.swt.browser.ProgressListener;
import org.eclipse.swt.browser.StatusTextEvent;
import org.eclipse.swt.browser.StatusTextListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.ProgressBar;
import org.eclipse.ui.IWorkbenchPart;


public abstract class BrowserControl
	implements ProgressListener, StatusTextListener, LocationListener
{

	public static final String DEFAULT_URL = "http://about:8080";

	protected IWorkbenchPart enclosingPart;
	protected BrowserNavigationBar navBar;
	protected Composite browser;
	protected Label status;
	protected ProgressBar progressBar;
	protected String initialURL = DEFAULT_URL;
	protected boolean autoValidating = true;

	public BrowserControl (Composite parent, IWorkbenchPart enclosingPart) {
		Composite main = new Composite(parent, SWT.NONE);
		main.setLayout(new GridLayout());
		this.enclosingPart = enclosingPart;
		
		try {
			createNavigationBar(main);
			createBrowserControl(main);
		} catch (Exception e) {
			throw new RuntimeException("Could not create browser control", e);
		}
	}

	public Composite getBrowser () {
		return browser;
	}

	public String getLocation () {
		return navBar.getUrl();
	}

	public void setLocation (String url) {
		navBar.setUrl(url);
	}

	protected abstract Composite createBrowser (Composite parent) throws Exception;

	protected void createNavigationBar (Composite parent) {
		navBar = new BrowserNavigationBar(parent, SWT.NONE);
		navBar.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		// navigation actions
		addNavigateAction("back", BrowserNavigationBar.BACK_ACTION_ID, "Go back to previous history item");
		addNavigateAction("forward", BrowserNavigationBar.FORWARD_ACTION_ID, "Go forward to next history item");
		addNavigateAction("refresh", BrowserNavigationBar.REFRESH_ACTION_ID, "Refresh the current page");
		addNavigateAction("stop", BrowserNavigationBar.STOP_ACTION_ID, "Stop page load");
		
		// validation-related actions
		IAction goAction = new Action("Go", IAction.AS_PUSH_BUTTON) {
			public void run () {
				setUrl(getLocation());
			}
		};
		goAction.setId(BrowserNavigationBar.GO_ACTION_ID);
		navBar.addAction(goAction, BrowserNavigationBar.VALIDATION_GROUP_ID);
		
		IAction autoValidateAction = new Action("Auto-validate", IAction.AS_CHECK_BOX) {
			public void run () {
				autoValidating = !autoValidating;
			}
		};
		autoValidateAction.setId(BrowserNavigationBar.AUTOVALIDATE_ACTION_ID);
		autoValidateAction.setChecked(true);
		navBar.addAction(autoValidateAction, BrowserNavigationBar.VALIDATION_GROUP_ID);
	}

	protected void createBrowserControl (Composite parent) throws Exception {
		Composite browserArea = new Composite(parent, SWT.NONE);
		browserArea.setLayout(new GridLayout(3, false));
		browserArea.setLayoutData(new GridData(GridData.FILL_BOTH));
		browser = createBrowser(browserArea);
		if (browser != null && initialURL != null) {
			GridData bgd = new GridData(GridData.FILL_BOTH);
			bgd.horizontalSpan = 3;
			browser.setLayoutData(bgd);
			setLocation(initialURL);
		}
		initialURL = null;
		
		GridData sgd = new GridData(GridData.VERTICAL_ALIGN_CENTER | GridData.FILL_HORIZONTAL);
		sgd.horizontalSpan = 2;
		status = new Label(browserArea, SWT.NONE);
		status.setLayoutData(sgd);
		GuiUtils.configureControl(status, "statusLabel", "Displays the current page loading status", "Ready");
		
		GridData pbgd = new GridData(GridData.VERTICAL_ALIGN_CENTER);
		progressBar = new ProgressBar(browserArea, SWT.NONE);
		progressBar.setLayoutData(pbgd);
		GuiUtils.configureControl(progressBar, "statusProgress", "Displays the current page loading progress");
	}

	protected Runnable getBrowserRunnable (final String method) {
		return new Runnable() {
			public void run () {
				try {
					Method meth = browser.getClass().getMethod(method, (Class[]) null);
					meth.invoke(browser, (Object[]) null);
				}catch (Exception e) {
				}
			}
		};
	}

	protected void addNavigateAction (final String browserAction, String actionId, String tooltip) {
		IAction action = new Action(browserAction, IAction.AS_PUSH_BUTTON) {
			public void run () {
				getBrowserRunnable(browserAction).run();
			}
		};
		action.setId(actionId);
		action.setToolTipText(tooltip);
		navBar.addAction(action, BrowserNavigationBar.NAVIGATION_GROUP_ID);
	}

	protected abstract void setUrl (String url);

	public abstract Object getDocument ();

	public void changed (ProgressEvent event) {
		if (event.total > 0) {
			int ratio = event.current * 100 / event.total;
			progressBar.setSelection(ratio);
		}
	}

	public void completed (ProgressEvent event) {
		progressBar.setSelection(0);
	}

	public void changed (StatusTextEvent event) {
		if (status != null) {
			status.setText(event.text);
		}
	}

	public void changed (LocationEvent event) {
	}

	public void changing (LocationEvent event) {
	}
}
