/*******************************************************************************
 * Copyright (c) 2006, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/


package org.eclipse.actf.webelo.wst.ui;

import java.util.LinkedList;

import org.eclipse.actf.webelo.ui.MozillaValidationUIPlugin;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.wst.common.componentcore.ComponentCore;
import org.eclipse.wst.common.componentcore.resources.IVirtualComponent;
import org.eclipse.wst.common.componentcore.resources.IVirtualFolder;
import org.eclipse.wst.server.core.IModule;
import org.eclipse.wst.server.core.IModuleArtifact;
import org.eclipse.wst.server.core.IServer;
import org.eclipse.wst.server.core.IServerWorkingCopy;
import org.eclipse.wst.server.core.ServerCore;
import org.eclipse.wst.server.core.ServerUtil;
import org.eclipse.wst.server.core.internal.ChainedJob;
import org.eclipse.wst.server.core.internal.PublishServerJob;
import org.eclipse.wst.server.core.internal.RestartServerJob;
import org.eclipse.wst.server.core.internal.ServerPlugin;
import org.eclipse.wst.server.core.internal.ServerPreferences;
import org.eclipse.wst.server.core.internal.ServerType;
import org.eclipse.wst.server.core.internal.StartServerJob;
import org.eclipse.wst.server.ui.internal.Messages;
import org.eclipse.wst.server.ui.internal.ServerUIPlugin;
import org.eclipse.wst.server.ui.internal.ServerUIPreferences;
import org.eclipse.wst.server.ui.internal.wizard.RunOnServerWizard;

public class WstValidationUtils {

	public static final String WST_SERVER_UI_FAMILY = "org.eclipse.wst.server.ui.family";
	
	private static final String PHP_NATURE_ID =  "org.eclipse.php.core.PHPNature";
	private static final String MODULE_CORE_NATURE_ID = "org.eclipse.wst.common.modulecore.ModuleCoreNature";
	private static final String[] RESTART_SERVER_DIALOG_OPTIONS = {
		Messages.dialogModeWarningRestart, Messages.dialogModeWarningContinue, IDialogConstants.CANCEL_LABEL
	};
	
	public static String getWebContentRelativePath(IResource resource) {
		String result = null;
		IVirtualComponent component = ComponentCore.createComponent(resource.getProject());

		if(component != null) {
			IVirtualFolder rootFolder = component.getRootFolder();
			if(rootFolder.exists()
					&& resource.getProjectRelativePath().matchingFirstSegments(rootFolder.getProjectRelativePath()) == rootFolder.getProjectRelativePath().segmentCount()) {
				result = resource.getProjectRelativePath().removeFirstSegments(rootFolder.getProjectRelativePath().segmentCount()).makeRelative().toString();
			}
		} else if( hasPHPNature(resource.getProject())) {
			result = resource.getProjectRelativePath().makeRelative().toString();
		}

		return result;
	}
		
	public static String getFullAppPath(String projectName, String moduleName, IPath webContentPath) {
		String result = null;
		if(projectName != null && moduleName != null && moduleName.length() > 0) {
			IProject project = ResourcesPlugin.getWorkspace().getRoot().getProject(projectName);
			IContainer webContentFolder = hasPHPNature(project)
				? project : ComponentCore.createComponent(project).getRootFolder().getUnderlyingFolder();
			String projectRelPath = webContentFolder.getProjectRelativePath().makeRelative().toString();
			if( projectRelPath.length() == 0 ) {
				result = webContentPath.makeRelative().toString();
			} else {
				result = projectRelPath+"/"+ webContentPath.makeRelative().toString();
			}
		}
		return result;
	}
	
	private static boolean hasPHPNature (IProject project) {
		boolean hasPHPNature = false;
		try {
			hasPHPNature = project.hasNature(PHP_NATURE_ID);
		} catch (CoreException e) {
			hasPHPNature = false;
		}
		return hasPHPNature;
	}
	
	public static void openErrorDialog (String message) {
		openErrorDialog(message, null);
	}
	
	public static void openErrorDialog (String message, Throwable t) {
		// TODO should be made common, perhaps in GuiUtils.reportError via IErrorHandler
		ErrorDialog.openError(
			getActiveWorkbenchShell(), "Error in web validation", message,
			new Status(IStatus.ERROR, MozillaValidationUIPlugin.UI_PLUGIN_ID, IStatus.OK, message, t)
		);
	}
	
	public  static IServer getServer(IProject project, String moduleName,
							  		 String mode,  IProgressMonitor monitor) throws CoreException {
		IModuleArtifact[] artifacts = ServerPlugin.getModuleArtifacts(project);
		IModule module = artifacts != null && artifacts.length > 0 ? artifacts[0].getModule() : null;
		Shell activeShell = getActiveWorkbenchShell();
		
		if (module == null) {
			openErrorDialog("No module found");
			throw new CoreException(new Status(
				IStatus.ERROR, MozillaValidationUIPlugin.UI_PLUGIN_ID,
				IStatus.OK, "No module found", null
			));
		} else if (activeShell == null) {
			throw new CoreException(new Status(
					IStatus.ERROR, MozillaValidationUIPlugin.UI_PLUGIN_ID,
					IStatus.OK, "No workbench shell found", null
				));
		}
		
		IServer server = ServerCore.getDefaultServer(module);
		if (server == null) {
			server = findServer(module, artifacts[0], mode, activeShell, monitor);
		}
		if (server != null
			&& ServerUIPlugin.isCompatibleWithLaunchMode(server, mode)
			&& !ServerUtil.containsModule(server, module, monitor)) {
			IServerWorkingCopy wc = server.createWorkingCopy();
			try {
				ServerUtil.modifyModules(wc, new IModule[] { module }, new IModule[0], monitor);
				wc.save(false, monitor);
			} catch (CoreException ce) {
				server = null;
			}
		}
		
		try {
			if (server != null) {
				Platform.getJobManager().join(WST_SERVER_UI_FAMILY, null);
			}
		} catch (Exception e) {
			openErrorDialog("Join error", e);
		}
		
		return server;
}
	
	public static void startServer(IServer server, IModule module, String launchMode, Job validationJob) {
		int state = server.getServerState();
		IModule[] modules = new IModule[] { module };
		LinkedList jobsList = new LinkedList();
		
		if (state == IServer.STATE_STARTING) {
			waitForServer(server, modules);
		}
		
		state = server.getServerState();
		if (state == IServer.STATE_STARTED) {
			// do we need to restart because
			// 1) incompatible launch mode with server's current mode or
			// 2) user prefers to always restart
			boolean restart = false;
			String serverMode = server.getMode();
			if (launchMode.equals(ILaunchManager.DEBUG_MODE) && !serverMode.equals(ILaunchManager.DEBUG_MODE)) {
			String response = RESTART_SERVER_DIALOG_OPTIONS[shouldRestartServer()];
			if (response.equals(Messages.dialogModeWarningRestart)) {
				restart = true;
			} else if (response.equals(Messages.dialogModeWarningContinue)) {
				launchMode = serverMode;
			} else {
				return;
			}
		}
			
			if (server.getServerRestartState()) {
				int restartPref = ServerUIPlugin.getPreferences().getRestart();
				restart = restartPref == ServerUIPreferences.RESTART_ALWAYS
					|| RESTART_SERVER_DIALOG_OPTIONS[shouldRestartServer()].equals(Messages.dialogModeWarningRestart);
			}
			
			if (restart) {
				jobsList.add(new RestartServerJob(server, launchMode));
			}
			jobsList.add(new PublishServerJob(server));
			jobsList.add(validationJob);
		} else if (state == IServer.STATE_STOPPED) {
			if (((ServerType)server.getServerType()).startBeforePublish()) {
				jobsList.add(new StartServerJob(server, launchMode));
				jobsList.add(new PublishServerJob(server));
				jobsList.add(validationJob);
			} else {
				jobsList.add(new PublishServerJob(server));
				jobsList.add(new StartServerJob(server, launchMode));
				jobsList.add(validationJob);
			}
		}
		
		// chain the accumulated jobs
		Job[] jobs = (Job[]) jobsList.toArray(new Job[jobsList.size()]);
		for (int j = 0; j < jobs.length; ++j) {
			if (jobs[j] instanceof ChainedJob && j < jobs.length - 1) {
				((ChainedJob) jobs[j]).setNextJob(jobs[j + 1]);
			}
		}
		jobs[0].schedule();
	}
	
	public static Shell getActiveWorkbenchShell () {
		IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
		if (window == null) {
			IWorkbenchWindow[] windows = PlatformUI.getWorkbench().getWorkbenchWindows();
			window = windows.length > 0 ? windows[0] : null;
		}
		return window != null ? window.getShell() : null;
	}
	
	private static IServer findServer (final IModule module, final IModuleArtifact moduleArtifact,
							   final String mode, final Shell shell, final IProgressMonitor monitor) {
		final IServer[] servers = new IServer[1];
		PlatformUI.getWorkbench().getDisplay().syncExec(new Runnable() {
			public void run() {
				RunOnServerWizard wizard = new RunOnServerWizard(module, mode, moduleArtifact);
				WizardDialog dialog = new WizardDialog(shell, wizard);
				if (dialog.open() == Window.CANCEL) {
					monitor.setCanceled(true);
					return;
				}
				
				try {
					Platform.getJobManager().join(WST_SERVER_UI_FAMILY, null);
					servers[0] = wizard.getServer();
					if (servers[0] != null && wizard.isPreferredServer()) {
						ServerCore.setDefaultServer(module, servers[0], monitor);
					}
				} catch (Exception e) {
					openErrorDialog("Could not save default server preference", e);
				}
			}
		});

		return servers[0];
	}
	
	private static int shouldRestartServer () {
		MessageDialog dialog = new MessageDialog(
				getActiveWorkbenchShell(), "Restart server", null,
				"Current server mode and launch mode do not match. Restart server?",	MessageDialog.QUESTION,
				RESTART_SERVER_DIALOG_OPTIONS, 0
		);
		int index = dialog.open();
		if (index == -1) {
			// dialog dismissed so consider this a cancel
			index = 2;
		}
		return index;
	}
	
	private static void waitForServer (IServer server, IModule[] modules) {
		int state = server.getModuleState(modules);
		int count = ServerPreferences.getInstance().getModuleStartTimeout();
		
		while (state == IServer.STATE_STARTING && count > 0) {
			try {
				Thread.sleep(2000);
			} catch (InterruptedException e) {
			}
			count -= 2000;
			state = server.getModuleState(modules);
		}
	}

	}