/*******************************************************************************
 * Copyright (c) 2006, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/


package org.eclipse.actf.webelo.wst.ui.debug;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;

import org.eclipse.actf.webelo.wst.ui.ILaunchConfigurationConstants;
import org.eclipse.actf.webelo.wst.ui.WstValidationUtils;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Path;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.ui.AbstractLaunchConfigurationTab;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.ElementTreeSelectionDialog;
import org.eclipse.ui.model.BaseWorkbenchContentProvider;
import org.eclipse.ui.model.WorkbenchLabelProvider;

public abstract class WstValidationLaunchConfigurationTab extends AbstractLaunchConfigurationTab
{

	protected static final String URL_TYPE_BUTTON_TEXT = "&URL";
	protected static final String FILE_TYPE_BUTTON_TEXT = "&Local file";
	protected static final String PROJECT_TYPE_BUTTON_TEXT = "&Project file";
	
	protected String projectPath = "", filePath = "", urlPath = "";
	protected boolean isURL, isLocalFile;
	protected Button browseButton, urlTypeButton, fileTypeButton, projectTypeButton;
	protected Button lastSelected;
	protected Text pathField;
	protected String mode;
	
	public WstValidationLaunchConfigurationTab(String mode) {
		super();
		this.mode = mode;
	}	
	
	public boolean isValid(ILaunchConfiguration launchConfig) {
		return getErrorMessage() == null;
	}

	public boolean canSave() {
		return getErrorMessage() == null;
	}

	public void createControl(Composite parent) {
		Composite tab = new Composite(parent, SWT.NONE );
		tab.setFont(parent.getFont());
		tab.setLayout(new GridLayout());
		tab.setLayoutData(new GridData(GridData.FILL_BOTH));
		setControl(tab);
		
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		GridData data = new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL);

		Group webGroup = new Group(tab, SWT.NONE);
		webGroup.setFont(parent.getFont());
		gridLayout = new GridLayout();
		gridLayout.numColumns = 1;
		webGroup.setLayout(gridLayout);
		webGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		webGroup.setText("Launch Target:");

		Composite radioGroup = new Composite(webGroup, SWT.NONE);
		radioGroup.setLayout(new RowLayout());
		urlTypeButton = createTypeButton(radioGroup, URL_TYPE_BUTTON_TEXT, "Launch using given URL", isURL);
		fileTypeButton = createTypeButton(radioGroup, FILE_TYPE_BUTTON_TEXT, "Launch using file on local file system", isLocalFile);
		projectTypeButton = createTypeButton(radioGroup, PROJECT_TYPE_BUTTON_TEXT, "Launch using workspace project file", !(isURL || isLocalFile));
		lastSelected = projectTypeButton;
		
		final Composite temp = new Composite(webGroup, SWT.NULL);
		gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		temp.setLayout(gridLayout);
		temp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		pathField = new Text(temp, SWT.SINGLE | SWT.BORDER);
		pathField.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));
		pathField.setFont(parent.getFont());
		pathField.addFocusListener(new FocusListener() {
			public void focusGained (FocusEvent fe) {
				setErrorMessage(null);
			}
			public void focusLost (FocusEvent fe) {
				if (isURL) {
			String pathText = pathField.getText();
					if (pathText.trim().length() > 0) {
						try {
							new URL(pathText);
							setErrorMessage(null);
						} catch (MalformedURLException mue) {
							setErrorMessage("A valid URL must be specified");
						}
					}
					updateLaunchConfigurationDialog();
				}
	}
		});
		
		browseButton = createPushButton(temp, "Browse...", null);
		browseButton.setVisible(true);
		browseButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent evt) {
				doBrowse(temp.getShell());
				updateWidgetEnablements();
			}
		});
	}
	
	protected void doBrowse (Shell parent) {
		if (fileTypeButton.getSelection()) {
			FileDialog fd = new FileDialog(parent, SWT.OPEN);
			String path = fd.open();
			if (path != null && path.trim().length() > 0) {
				pathField.setText(path);
				updateLaunchConfigurationDialog();
			}
		} else if (projectTypeButton.getSelection()) {
			IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
			ElementTreeSelectionDialog dialog = new ElementTreeSelectionDialog(
				getShell(), new WorkbenchLabelProvider(), new BaseWorkbenchContentProvider());
			dialog.setTitle("Web project file selection");
			dialog.setMessage("Select a target web application file from the project's contents");
			// TODO need to add filter
			dialog.setInput(root);
			dialog.setAllowMultiple(false);
			dialog.open();
			if(dialog.getReturnCode() == Window.OK) {
				Object[] targetFiles = dialog.getResult();
				IFile targetFile = (IFile)targetFiles[0];
				if(targetFile != null) {
					pathField.setText(targetFile.getProject().getName()+"/"+targetFile.getProjectRelativePath());
					updateLaunchConfigurationDialog();
				}
			}
		}
	}
	
	protected void updateWidgetEnablements() {
		setErrorMessage(null);
		if(isURL) {
			if(pathField.getText().trim().length() == 0) {
				setErrorMessage("A valid URL must be specified");
			} 
		} else if(pathField.getText().trim().length() == 0 || !isValidPath(pathField.getText())) {
				setErrorMessage("A valid file must be specified");
		} 
	}
	
	protected boolean isValidPath(String path) {
		IFile projectFile = null;
		File localFile = null;
		try {
			projectFile = ResourcesPlugin.getWorkspace().getRoot().getFile(new Path(path));
		} catch( Exception e ) {
			localFile = new File(path);
		}
		return (projectFile != null && projectFile.exists())
			|| (localFile != null && localFile.exists());
	}

	public void setDefaults(ILaunchConfigurationWorkingCopy configuration) {
		isURL = false;
		isLocalFile = false;
	}

	public void initializeFrom(ILaunchConfiguration config) {
		try {
			String webModule = config.getAttribute(ILaunchConfigurationConstants.ATTR_WEB_MODULE, "");
			String projectName = config.getAttribute(ILaunchConfigurationConstants.ATTR_PROJECT, "");
			String appPath = config.getAttribute(ILaunchConfigurationConstants.ATTR_APP_PATH, "");
			
			isURL = config.getAttribute(ILaunchConfigurationConstants.ATTR_IS_URL, false);
			isLocalFile = !isURL && webModule.length() == 0 && projectName.length() == 0;
			urlTypeButton.setSelection(isURL);
			fileTypeButton.setSelection(isLocalFile);
			projectTypeButton.setSelection(!(isURL || isLocalFile));
			browseButton.setVisible(!isURL);
			
			if(isURL) {
				pathField.setText(appPath);
			} else {
				if( !config.getAttribute(ILaunchConfigurationConstants.ATTR_FULL_PATH, "").equals("") ) {
					pathField.setText(config.getAttribute(ILaunchConfigurationConstants.ATTR_FULL_PATH, ""));
				} else {
					String fullAppPath = WstValidationUtils.getFullAppPath(projectName, webModule, new Path(appPath));
					pathField.setText(fullAppPath == null ? "" : projectName+"/"+fullAppPath);
				}
			}

			updateWidgetEnablements();
		} catch (CoreException ce) {
		}
	}

	private Button createTypeButton (final Composite parent, final String text, String tooltip, boolean select) {
		final Button button = new Button(parent, SWT.RADIO);
		button.setFont(parent.getFont());
		button.setText(text);
		button.setToolTipText(tooltip);
		button.setSelection(select);
		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				if (text.equals(URL_TYPE_BUTTON_TEXT) && button.getSelection()) {
					isURL = true;
					isLocalFile = false;
					browseButton.setVisible(false);
					if (lastSelected == fileTypeButton) {
						filePath = pathField.getText();
					} else {
						projectPath = pathField.getText();
					}
					if(urlPath != null)  {
						pathField.setText(urlPath);
					}
				} else if (text.equals(FILE_TYPE_BUTTON_TEXT) && button.getSelection()) {
					isURL = false;
					isLocalFile = true;
					browseButton.setVisible(true);
					if (lastSelected == urlTypeButton) {
						urlPath = pathField.getText();
					} else {
						projectPath = pathField.getText();
					}
						if(filePath != null) {
							pathField.setText(filePath);
						}
				} else if (text.equals(PROJECT_TYPE_BUTTON_TEXT) && button.getSelection()) {
					isURL = false;
					isLocalFile = false;
					browseButton.setVisible(true);
					if (lastSelected == urlTypeButton) {
						urlPath = pathField.getText();
					} else {
						filePath = pathField.getText();
					}
						if(projectPath != null) {
							pathField.setText(projectPath);
						}
				}
				
				updateWidgetEnablements();
				updateLaunchConfigurationDialog();
				lastSelected = (Button) event.getSource();
			}
		});
		
		return button;
	}

}
