/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 


package org.eclipse.actf.webelo.wst.ui.debug;

import java.net.URL;

import org.eclipse.actf.webelo.wst.ui.ILaunchConfigurationConstants;
import org.eclipse.actf.webelo.wst.ui.WstValidationUtils;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Path;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationType;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.ui.ILaunchShortcut;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;

public class WstValidationLaunchShortcut implements ILaunchShortcut
{

	public void launch(ISelection selection, String mode) {
		if (selection instanceof IStructuredSelection) {
			IStructuredSelection sel = (IStructuredSelection)selection;
			if (sel.size() == 1) {
				launch(sel.getFirstElement(), mode);
			} else {
				WstValidationUtils.openErrorDialog(sel.size() == 0 ? "Selection is empty" : "Multiple selection is not permitted");
			}
		}
	}

	public void launch(IEditorPart editor, String mode) {
		IEditorInput input = editor.getEditorInput();
		IFile file = (IFile)input.getAdapter(IFile.class);
		if (file != null) {
			launch(file, mode);
		} 
	}
	
	protected void launch (Object input, String mode) {
		ILaunchConfigurationType configType = DebugPlugin.getDefault()
			.getLaunchManager().getLaunchConfigurationType(ILaunchConfigurationConstants.MOZILLA_APP_LAUNCH_CONFIGURATION_TYPE);
		String appPath = null;
		IProject project = null;
		IFile file = null;
		boolean isURL = false;
		
		if (input instanceof IFile) {
			file = (IFile) input;
			project = file.getProject();
			appPath = WstValidationUtils.getWebContentRelativePath(file);
			isURL = false;
			if (appPath == null) {
				WstValidationUtils.openErrorDialog("File must be in a web content directory");
			}
		} else if (input instanceof URL) {
			URL u = (URL) input;
			appPath = u.toExternalForm();
			isURL = true;
		} else {
			WstValidationUtils.openErrorDialog("Input must be either a URL or a file in a web content directory");
		}

		if (appPath != null) {
			String projectName = project.getName();
			ILaunchConfiguration config = findLaunchConfiguration(projectName, projectName, appPath, isURL, mode, configType);
			if (config != null) {
				try {
					config.launch(mode, null, true);
				} catch (CoreException e) {
					WstValidationUtils.openErrorDialog("Error in launch of web application", e);
				}
			} else {
				WstValidationUtils.openErrorDialog("Could not find appropriate web launch configuration type");
			}
		}
	}
	
	protected ILaunchConfiguration findLaunchConfiguration(String project, String module,
														   String appPathString, boolean isURL,
														   String mode, ILaunchConfigurationType configType) {
		ILaunchConfiguration result = null;

		try {
			ILaunchConfiguration[] configs = DebugPlugin.getDefault().getLaunchManager().getLaunchConfigurations(configType);
			for (ILaunchConfiguration config : configs) {	
				String cPathString = config.getAttribute(ILaunchConfigurationConstants.ATTR_APP_PATH, (String)null);
				String cProject = config.getAttribute(ILaunchConfigurationConstants.ATTR_PROJECT, (String)null);
				String cModule = config.getAttribute(ILaunchConfigurationConstants.ATTR_WEB_MODULE, (String)null);
				boolean cIsURL = config.getAttribute(ILaunchConfigurationConstants.ATTR_IS_URL, false);
				if(!cIsURL) {
					//TODO normalized path comparison
					if (appPathString.equals(cPathString) && config.supportsMode(mode)
						&& project.equals(cProject) && module.equals(cModule)) {
						result = config.getWorkingCopy();
						break;
					}
				} else {
					if (appPathString.equals(cPathString) &&  config.supportsMode(mode) && cIsURL == isURL) {
						result = config.getWorkingCopy();
						break;
					}
				}
			}

			if (result == null)  {
				result = createConfiguration(project, module, appPathString, isURL, configType);
			}
		} catch (CoreException e) {
			e.printStackTrace();
		}
		
		return result;
	}
	
	protected ILaunchConfiguration createConfiguration(String project, String module, String appPath,
													   boolean isURL, ILaunchConfigurationType configType) throws CoreException {
		ILaunchConfigurationWorkingCopy copy = configType.newInstance(null,
				DebugPlugin.getDefault().getLaunchManager()
					.generateUniqueLaunchConfigurationNameFrom(module+"-"+new Path(appPath).lastSegment())); 
	    copy.setAttribute(ILaunchConfigurationConstants.ATTR_PROJECT, project);
		copy.setAttribute(ILaunchConfigurationConstants.ATTR_WEB_MODULE, module);
		copy.setAttribute(ILaunchConfigurationConstants.ATTR_PROCESS_TYPE, "Mozilla");
	   	copy.setAttribute(ILaunchConfigurationConstants.ATTR_IS_URL, isURL);
	   	copy.setAttribute(ILaunchConfigurationConstants.ATTR_APP_PATH, appPath);
		copy.setAttribute(ILaunchConfigurationConstants.ATTR_FULL_PATH, "");
		return copy.doSave();
	}
	
}
